@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application;
@@ -12,16 +12,11 @@ use Nette;
 
 /**
  * Front Controller.
- *
- * @author     David Grudl
- *
- * @property-read array $requests
- * @property-read IPresenter $presenter
- * @property-read IRouter $router
- * @property-read IPresenterFactory $presenterFactory
  */
-class Application extends Nette\Object
+class Application
 {
+	use Nette\SmartObject;
+
 	/** @var int */
 	public static $maxLoop = 20;
 
@@ -31,26 +26,26 @@ class Application extends Nette\Object
 	/** @var string */
 	public $errorPresenter;
 
-	/** @var array of function(Application $sender); Occurs before the application loads presenter */
+	/** @var callable[]  function (Application $sender); Occurs before the application loads presenter */
 	public $onStartup;
 
-	/** @var array of function(Application $sender, \Exception $e = NULL); Occurs before the application shuts down */
+	/** @var callable[]  function (Application $sender, \Exception|\Throwable $e = NULL); Occurs before the application shuts down */
 	public $onShutdown;
 
-	/** @var array of function(Application $sender, Request $request); Occurs when a new request is received */
+	/** @var callable[]  function (Application $sender, Request $request); Occurs when a new request is received */
 	public $onRequest;
 
-	/** @var array of function(Application $sender, Presenter $presenter); Occurs when a presenter is created */
+	/** @var callable[]  function (Application $sender, Presenter $presenter); Occurs when a presenter is created */
 	public $onPresenter;
 
-	/** @var array of function(Application $sender, IResponse $response); Occurs when a new response is ready for dispatch */
+	/** @var callable[]  function (Application $sender, IResponse $response); Occurs when a new response is ready for dispatch */
 	public $onResponse;
 
-	/** @var array of function(Application $sender, \Exception $e); Occurs when an unhandled exception occurs in the application */
+	/** @var callable[]  function (Application $sender, \Exception|\Throwable $e); Occurs when an unhandled exception occurs in the application */
 	public $onError;
 
 	/** @var Request[] */
-	private $requests = array();
+	private $requests = [];
 
 	/** @var IPresenter */
 	private $presenter;
@@ -88,7 +83,10 @@ class Application extends Nette\Object
 			$this->processRequest($this->createInitialRequest());
 			$this->onShutdown($this);
 
+		} catch (\Throwable $e) {
 		} catch (\Exception $e) {
+		}
+		if (isset($e)) {
 			$this->onError($this, $e);
 			if ($this->catchExceptions && $this->errorPresenter) {
 				try {
@@ -96,6 +94,8 @@ class Application extends Nette\Object
 					$this->onShutdown($this, $e);
 					return;
 
+				} catch (\Throwable $e) {
+					$this->onError($this, $e);
 				} catch (\Exception $e) {
 					$this->onError($this, $e);
 				}
@@ -123,7 +123,6 @@ class Application extends Nette\Object
 		try {
 			$name = $request->getPresenterName();
 			$this->presenterFactory->getPresenterClass($name);
-			$request->setPresenterName($name);
 		} catch (InvalidPresenterException $e) {
 			throw new BadRequestException($e->getMessage(), 0, $e);
 		}
@@ -137,6 +136,7 @@ class Application extends Nette\Object
 	 */
 	public function processRequest(Request $request)
 	{
+		process:
 		if (count($this->requests) > self::$maxLoop) {
 			throw new ApplicationException('Too many loops detected in application life cycle.');
 		}
@@ -146,10 +146,11 @@ class Application extends Nette\Object
 
 		$this->presenter = $this->presenterFactory->createPresenter($request->getPresenterName());
 		$this->onPresenter($this, $this->presenter);
-		$response = $this->presenter->run($request);
+		$response = $this->presenter->run(clone $request);
 
 		if ($response instanceof Responses\ForwardResponse) {
-			$this->processRequest($response->getRequest());
+			$request = $response->getRequest();
+			goto process;
 
 		} elseif ($response) {
 			$this->onResponse($this, $response);
@@ -159,18 +160,19 @@ class Application extends Nette\Object
 
 
 	/**
+	 * @param  \Exception|\Throwable
 	 * @return void
 	 */
-	public function processException(\Exception $e)
+	public function processException($e)
 	{
-		if (!$e instanceof BadRequestException) {
+		if (!$e instanceof BadRequestException && $this->httpResponse instanceof Nette\Http\Response) {
 			$this->httpResponse->warnOnBuffer = FALSE;
 		}
 		if (!$this->httpResponse->isSent()) {
 			$this->httpResponse->setCode($e instanceof BadRequestException ? ($e->getCode() ?: 404) : 500);
 		}
 
-		$args = array('exception' => $e, 'request' => end($this->requests) ?: NULL);
+		$args = ['exception' => $e, 'request' => end($this->requests) ?: NULL];
 		if ($this->presenter instanceof UI\Presenter) {
 			try {
 				$this->presenter->forward(":$this->errorPresenter:", $args);
@@ -225,22 +227,4 @@ class Application extends Nette\Object
 		return $this->presenterFactory;
 	}
 
-
-	/********************* request serialization ****************d*g**/
-
-
-	/** @deprecated */
-	function storeRequest($expiration = '+ 10 minutes')
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use $presenter->storeRequest() instead.', E_USER_DEPRECATED);
-		return $this->presenter->storeRequest($expiration);
-	}
-
-	/** @deprecated */
-	function restoreRequest($key)
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use $presenter->restoreRequest() instead.', E_USER_DEPRECATED);
-		return $this->presenter->restoreRequest($key);
-	}
-
 }
