@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application;
 
-use Nette;
-
 
 /**
  * Responsible for creating a new instance of given presenter.
- *
- * @author Jan Tichý <tichy@medio.cz>
  */
 interface IPresenterFactory
 {
