@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application;
@@ -12,16 +12,14 @@ use Nette;
 
 /**
  * The bi-directional router.
- *
- * @author     David Grudl
  */
 interface IRouter
 {
 	/** only matching route */
-	const ONE_WAY = 1;
+	const ONE_WAY = 0b0001;
 
-	/** HTTPS route */
-	const SECURED = 2;
+	/** @deprecated */
+	const SECURED = 0b0010;
 
 	/**
 	 * Maps HTTP request to a Request object.
