@@ -13,19 +13,18 @@ use Nette;
 /**
  * Default presenter loader.
  */
-class PresenterFactory extends Nette\Object implements IPresenterFactory
+class PresenterFactory implements IPresenterFactory
 {
-	/** @deprecated */
-	public $caseSensitive = TRUE;
+	use Nette\SmartObject;
 
 	/** @var array[] of module => splited mask */
-	private $mapping = array(
-		'*' => array('', '*Module\\', '*Presenter'),
-		'Nette' => array('NetteModule\\', '*\\', '*Presenter'),
-	);
+	private $mapping = [
+		'*' => ['', '*Module\\', '*Presenter'],
+		'Nette' => ['NetteModule\\', '*\\', '*Presenter'],
+	];
 
 	/** @var array */
-	private $cache = array();
+	private $cache = [];
 
 	/** @var callable */
 	private $factory;
@@ -34,7 +33,7 @@ class PresenterFactory extends Nette\Object implements IPresenterFactory
 	/**
 	 * @param  callable  function (string $class): IPresenter
 	 */
-	public function __construct($factory = NULL)
+	public function __construct(callable $factory = NULL)
 	{
 		$this->factory = $factory ?: function ($class) { return new $class; };
 	}
@@ -75,7 +74,7 @@ class PresenterFactory extends Nette\Object implements IPresenterFactory
 		$reflection = new \ReflectionClass($class);
 		$class = $reflection->getName();
 
-		if (!$reflection->implementsInterface('Nette\Application\IPresenter')) {
+		if (!$reflection->implementsInterface(IPresenter::class)) {
 			throw new InvalidPresenterException("Cannot load presenter '$name', class '$class' is not Nette\\Application\\IPresenter implementor.");
 		} elseif ($reflection->isAbstract()) {
 			throw new InvalidPresenterException("Cannot load presenter '$name', class '$class' is abstract.");
@@ -103,9 +102,9 @@ class PresenterFactory extends Nette\Object implements IPresenterFactory
 				if (!preg_match('#^\\\\?([\w\\\\]*\\\\)?(\w*\*\w*?\\\\)?([\w\\\\]*\*\w*)\z#', $mask, $m)) {
 					throw new Nette\InvalidStateException("Invalid mapping mask '$mask'.");
 				}
-				$this->mapping[$module] = array($m[1], $m[2] ?: '*Module\\', $m[3]);
+				$this->mapping[$module] = [$m[1], $m[2] ?: '*Module\\', $m[3]];
 			} elseif (is_array($mask) && count($mask) === 3) {
-				$this->mapping[$module] = array($mask[0] ? $mask[0] . '\\' : '', $mask[1] . '\\', $mask[2]);
+				$this->mapping[$module] = [$mask[0] ? $mask[0] . '\\' : '', $mask[1] . '\\', $mask[2]];
 			} else {
 				throw new Nette\InvalidStateException("Invalid mapping mask for module $module.");
 			}
@@ -143,7 +142,7 @@ class PresenterFactory extends Nette\Object implements IPresenterFactory
 	public function unformatPresenterClass($class)
 	{
 		foreach ($this->mapping as $module => $mapping) {
-			$mapping = str_replace(array('\\', '*'), array('\\\\', '(\w+)'), $mapping);
+			$mapping = str_replace(['\\', '*'], ['\\\\', '(\w+)'], $mapping);
 			if (preg_match("#^\\\\?$mapping[0]((?:$mapping[1])*)$mapping[2]\\z#i", $class, $matches)) {
 				return ($module === '*' ? '' : $module . ':')
 					. preg_replace("#$mapping[1]#iA", '$1:', $matches[1]) . $matches[3];
