@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\Responses;
@@ -12,15 +12,11 @@ use Nette;
 
 /**
  * File download response.
- *
- * @author     David Grudl
- *
- * @property-read string $file
- * @property-read string $name
- * @property-read string $contentType
  */
-class FileResponse extends Nette\Object implements Nette\Application\IResponse
+class FileResponse implements Nette\Application\IResponse
 {
+	use Nette\SmartObject;
+
 	/** @var string */
 	private $file;
 
@@ -33,13 +29,16 @@ class FileResponse extends Nette\Object implements Nette\Application\IResponse
 	/** @var bool */
 	public $resuming = TRUE;
 
+	/** @var bool */
+	private $forceDownload;
+
 
 	/**
 	 * @param  string  file path
 	 * @param  string  imposed file name
 	 * @param  string  MIME content type
 	 */
-	public function __construct($file, $name = NULL, $contentType = NULL)
+	public function __construct($file, $name = NULL, $contentType = NULL, $forceDownload = TRUE)
 	{
 		if (!is_file($file)) {
 			throw new Nette\Application\BadRequestException("File '$file' doesn't exist.");
@@ -48,6 +47,7 @@ class FileResponse extends Nette\Object implements Nette\Application\IResponse
 		$this->file = $file;
 		$this->name = $name ? $name : basename($file);
 		$this->contentType = $contentType ? $contentType : 'application/octet-stream';
+		$this->forceDownload = $forceDownload;
 	}
 
 
@@ -88,7 +88,10 @@ class FileResponse extends Nette\Object implements Nette\Application\IResponse
 	public function send(Nette\Http\IRequest $httpRequest, Nette\Http\IResponse $httpResponse)
 	{
 		$httpResponse->setContentType($this->contentType);
-		$httpResponse->setHeader('Content-Disposition', 'attachment; filename="' . $this->name . '"');
+		$httpResponse->setHeader('Content-Disposition',
+			($this->forceDownload ? 'attachment' : 'inline')
+				. '; filename="' . $this->name . '"'
+				. '; filename*=utf-8\'\'' . rawurlencode($this->name));
 
 		$filesize = $length = filesize($this->file);
 		$handle = fopen($this->file, 'r');
