@@ -13,8 +13,10 @@ use Nette;
 /**
  * JSON response used mainly for AJAX requests.
  */
-class JsonResponse extends Nette\Object implements Nette\Application\IResponse
+class JsonResponse implements Nette\Application\IResponse
 {
+	use Nette\SmartObject;
+
 	/** @var array|\stdClass */
 	private $payload;
 
@@ -62,7 +64,6 @@ class JsonResponse extends Nette\Object implements Nette\Application\IResponse
 	public function send(Nette\Http\IRequest $httpRequest, Nette\Http\IResponse $httpResponse)
 	{
 		$httpResponse->setContentType($this->contentType, 'utf-8');
-		$httpResponse->setExpiration(FALSE);
 		echo Nette\Utils\Json::encode($this->payload);
 	}
 
