@@ -16,14 +16,13 @@ use Nette\Utils\Strings;
  * The bidirectional route is responsible for mapping
  * HTTP request to a Request object for dispatch and vice-versa.
  */
-class Route extends Nette\Object implements Application\IRouter
+class Route implements Application\IRouter
 {
+	use Nette\SmartObject;
+
 	const PRESENTER_KEY = 'presenter';
 	const MODULE_KEY = 'module';
 
-	/** @deprecated */
-	const CASE_SENSITIVE = 256;
-
 	/** @internal url type */
 	const HOST = 1,
 		PATH = 2,
@@ -42,39 +41,39 @@ class Route extends Nette\Object implements Application\IRouter
 		PATH_OPTIONAL = 1,
 		CONSTANT = 2;
 
-	/** @var int */
+	/** @deprecated */
 	public static $defaultFlags = 0;
 
 	/** @var array */
-	public static $styles = array(
-		'#' => array( // default style for path parameters
+	public static $styles = [
+		'#' => [ // default style for path parameters
 			self::PATTERN => '[^/]+',
-			self::FILTER_OUT => array(__CLASS__, 'param2path'),
-		),
-		'?#' => array( // default style for query parameters
-		),
-		'module' => array(
+			self::FILTER_OUT => [__CLASS__, 'param2path'],
+		],
+		'?#' => [ // default style for query parameters
+		],
+		'module' => [
 			self::PATTERN => '[a-z][a-z0-9.-]*',
-			self::FILTER_IN => array(__CLASS__, 'path2presenter'),
-			self::FILTER_OUT => array(__CLASS__, 'presenter2path'),
-		),
-		'presenter' => array(
+			self::FILTER_IN => [__CLASS__, 'path2presenter'],
+			self::FILTER_OUT => [__CLASS__, 'presenter2path'],
+		],
+		'presenter' => [
 			self::PATTERN => '[a-z][a-z0-9.-]*',
-			self::FILTER_IN => array(__CLASS__, 'path2presenter'),
-			self::FILTER_OUT => array(__CLASS__, 'presenter2path'),
-		),
-		'action' => array(
+			self::FILTER_IN => [__CLASS__, 'path2presenter'],
+			self::FILTER_OUT => [__CLASS__, 'presenter2path'],
+		],
+		'action' => [
 			self::PATTERN => '[a-z][a-z0-9-]*',
-			self::FILTER_IN => array(__CLASS__, 'path2action'),
-			self::FILTER_OUT => array(__CLASS__, 'action2path'),
-		),
-		'?module' => array(
-		),
-		'?presenter' => array(
-		),
-		'?action' => array(
-		),
-	);
+			self::FILTER_IN => [__CLASS__, 'path2action'],
+			self::FILTER_OUT => [__CLASS__, 'action2path'],
+		],
+		'?module' => [
+		],
+		'?presenter' => [
+		],
+		'?action' => [
+		],
+	];
 
 	/** @var string */
 	private $mask;
@@ -89,7 +88,7 @@ class Route extends Nette\Object implements Application\IRouter
 	private $aliases;
 
 	/** @var array of [value & fixity, filterIn, filterOut] */
-	private $metadata = array();
+	private $metadata = [];
 
 	/** @var array  */
 	private $xlat;
@@ -97,6 +96,9 @@ class Route extends Nette\Object implements Application\IRouter
 	/** @var int HOST, PATH, RELATIVE */
 	private $type;
 
+	/** @var string  http | https */
+	private $scheme;
+
 	/** @var int */
 	private $flags;
 
@@ -112,26 +114,35 @@ class Route extends Nette\Object implements Application\IRouter
 	 * @param  array|string|\Closure  default values or metadata or callback for NetteModule\MicroPresenter
 	 * @param  int     flags
 	 */
-	public function __construct($mask, $metadata = array(), $flags = 0)
+	public function __construct($mask, $metadata = [], $flags = 0)
 	{
 		if (is_string($metadata)) {
-			$a = strrpos($tmp = $metadata, ':');
-			if (!$a) {
+			list($presenter, $action) = Nette\Application\Helpers::splitName($metadata);
+			if (!$presenter) {
 				throw new Nette\InvalidArgumentException("Second argument must be array or string in format Presenter:action, '$metadata' given.");
 			}
-			$metadata = array(self::PRESENTER_KEY => substr($tmp, 0, $a));
-			if ($a < strlen($tmp) - 1) {
-				$metadata['action'] = substr($tmp, $a + 1);
+			$metadata = [self::PRESENTER_KEY => $presenter];
+			if ($action !== '') {
+				$metadata['action'] = $action;
 			}
 		} elseif ($metadata instanceof \Closure || $metadata instanceof Nette\Callback) {
-			$metadata = array(
+			if ($metadata instanceof Nette\Callback) {
+				trigger_error('Nette\Callback is deprecated, use Nette\Utils\Callback::closure().', E_USER_DEPRECATED);
+			}
+			$metadata = [
 				self::PRESENTER_KEY => 'Nette:Micro',
 				'callback' => $metadata,
-			);
+			];
 		}
 
 		$this->flags = $flags | static::$defaultFlags;
 		$this->setMask($mask, $metadata);
+		if (static::$defaultFlags) {
+			trigger_error('Route::$defaultFlags is deprecated, router by default keeps the used protocol.', E_USER_DEPRECATED);
+		} elseif ($flags & self::SECURED) {
+			trigger_error('Router::SECURED is deprecated, specify scheme in mask.', E_USER_DEPRECATED);
+			$this->scheme = 'https';
+		}
 	}
 
 
@@ -150,12 +161,14 @@ class Route extends Nette\Object implements Application\IRouter
 		if ($this->type === self::HOST) {
 			$host = $url->getHost();
 			$path = '//' . $host . $url->getPath();
-			$host = ip2long($host) ? array($host) : array_reverse(explode('.', $host));
-			$re = strtr($re, array(
+			$parts = ip2long($host) ? [$host] : array_reverse(explode('.', $host));
+			$re = strtr($re, [
 				'/%basePath%/' => preg_quote($url->getBasePath(), '#'),
-				'%tld%' => preg_quote($host[0], '#'),
-				'%domain%' => preg_quote(isset($host[1]) ? "$host[1].$host[0]" : $host[0], '#'),
-			));
+				'%tld%' => preg_quote($parts[0], '#'),
+				'%domain%' => preg_quote(isset($parts[1]) ? "$parts[1].$parts[0]" : $parts[0], '#'),
+				'%sld%' => preg_quote(isset($parts[1]) ? $parts[1] : '', '#'),
+				'%host%' => preg_quote($host, '#'),
+			]);
 
 		} elseif ($this->type === self::RELATIVE) {
 			$basePath = $url->getBasePath();
@@ -178,7 +191,7 @@ class Route extends Nette\Object implements Application\IRouter
 		}
 
 		// assigns matched values to parameters
-		$params = array();
+		$params = [];
 		foreach ($matches as $k => $v) {
 			if (is_string($k) && $v !== '') {
 				$params[$this->aliases[$k]] = $v;
@@ -252,7 +265,7 @@ class Route extends Nette\Object implements Application\IRouter
 			$params,
 			$httpRequest->getPost(),
 			$httpRequest->getFiles(),
-			array(Application\Request::SECURED => $httpRequest->isSecured())
+			[Application\Request::SECURED => $httpRequest->isSecured()]
 		);
 	}
 
@@ -333,7 +346,7 @@ class Route extends Nette\Object implements Application\IRouter
 
 		// compositing path
 		$sequence = $this->sequence;
-		$brackets = array();
+		$brackets = [];
 		$required = NULL; // NULL for auto-optional
 		$url = '';
 		$i = count($sequence) - 1;
@@ -380,24 +393,25 @@ class Route extends Nette\Object implements Application\IRouter
 		} while (TRUE);
 
 
-		if ($this->type !== self::HOST) {
+		if ($this->type === self::HOST) {
+			$host = $refUrl->getHost();
+			$parts = ip2long($host) ? [$host] : array_reverse(explode('.', $host));
+			$url = strtr($url, [
+				'/%basePath%/' => $refUrl->getBasePath(),
+				'%tld%' => $parts[0],
+				'%domain%' => isset($parts[1]) ? "$parts[1].$parts[0]" : $parts[0],
+				'%sld%' => isset($parts[1]) ? $parts[1] : '',
+				'%host%' => $host,
+			]);
+			$url = ($this->scheme ?: $refUrl->getScheme()) . ':' . $url;
+		} else {
 			if ($this->lastRefUrl !== $refUrl) {
-				$scheme = ($this->flags & self::SECURED ? 'https://' : 'http://');
+				$scheme = ($this->scheme ?: $refUrl->getScheme());
 				$basePath = ($this->type === self::RELATIVE ? $refUrl->getBasePath() : '');
-				$this->lastBaseUrl = $scheme . $refUrl->getAuthority() . $basePath;
+				$this->lastBaseUrl = $scheme . '://' . $refUrl->getAuthority() . $basePath;
 				$this->lastRefUrl = $refUrl;
 			}
 			$url = $this->lastBaseUrl . $url;
-
-		} else {
-			$host = $refUrl->getHost();
-			$host = ip2long($host) ? array($host) : array_reverse(explode('.', $host));
-			$url = strtr($url, array(
-				'/%basePath%/' => $refUrl->getBasePath(),
-				'%tld%' => $host[0],
-				'%domain%' => isset($host[1]) ? "$host[1].$host[0]" : $host[0],
-			));
-			$url = ($this->flags & self::SECURED ? 'https:' : 'http:') . $url;
 		}
 
 		if (strpos($url, '//', 7) !== FALSE) {
@@ -430,8 +444,9 @@ class Route extends Nette\Object implements Application\IRouter
 		$this->mask = $mask;
 
 		// detect '//host/path' vs. '/abs. path' vs. 'relative path'
-		if (substr($mask, 0, 2) === '//') {
+		if (preg_match('#(?:(https?):)?(//.*)#A', $mask, $m)) {
 			$this->type = self::HOST;
+			list(, $this->scheme, $mask) = $m;
 
 		} elseif (substr($mask, 0, 1) === '/') {
 			$this->type = self::PATH;
@@ -442,7 +457,7 @@ class Route extends Nette\Object implements Application\IRouter
 
 		foreach ($metadata as $name => $meta) {
 			if (!is_array($meta)) {
-				$metadata[$name] = $meta = array(self::VALUE => $meta);
+				$metadata[$name] = $meta = [self::VALUE => $meta];
 			}
 
 			if (array_key_exists(self::VALUE, $meta)) {
@@ -453,37 +468,28 @@ class Route extends Nette\Object implements Application\IRouter
 			}
 		}
 
-		if (strpbrk($mask, '?<[') === FALSE) {
+		if (strpbrk($mask, '?<>[]') === FALSE) {
 			$this->re = '#' . preg_quote($mask, '#') . '/?\z#A';
-			$this->sequence = array($mask);
+			$this->sequence = [$mask];
 			$this->metadata = $metadata;
 			return;
 		}
 
 		// PARSE MASK
-		// <parameter-name[=default] [pattern] [#class]> or [ or ] or ?...
-		$parts = Strings::split($mask, '/<([^>#= ]+)(=[^># ]*)? *([^>#]*)(#?[^>\[\]]*)>|(\[!?|\]|\s*\?.*)/');
+		// <parameter-name[=default] [pattern]> or [ or ] or ?...
+		$parts = Strings::split($mask, '/<([^<>= ]+)(=[^<> ]*)? *([^<>]*)>|(\[!?|\]|\s*\?.*)/');
 
-		$this->xlat = array();
+		$this->xlat = [];
 		$i = count($parts) - 1;
 
 		// PARSE QUERY PART OF MASK
 		if (isset($parts[$i - 1]) && substr(ltrim($parts[$i - 1]), 0, 1) === '?') {
-			// name=<parameter-name [pattern][#class]>
-			$matches = Strings::matchAll($parts[$i - 1], '/(?:([a-zA-Z0-9_.-]+)=)?<([^># ]+) *([^>#]*)(#?[^>]*)>/');
+			// name=<parameter-name [pattern]>
+			$matches = Strings::matchAll($parts[$i - 1], '/(?:([a-zA-Z0-9_.-]+)=)?<([^> ]+) *([^>]*)>/');
 
-			foreach ($matches as $match) {
-				list(, $param, $name, $pattern, $class) = $match;  // $pattern is not used
-
-				if ($class !== '') {
-					if (!isset(static::$styles[$class])) {
-						throw new Nette\InvalidStateException("Parameter '$name' has '$class' flag, but Route::\$styles['$class'] is not set.");
-					}
-					$meta = static::$styles[$class];
-
-				} elseif (isset(static::$styles['?' . $name])) {
+			foreach ($matches as list(, $param, $name, $pattern)) { // $pattern is not used
+				if (isset(static::$styles['?' . $name])) {
 					$meta = static::$styles['?' . $name];
-
 				} else {
 					$meta = static::$styles['?#'];
 				}
@@ -504,18 +510,22 @@ class Route extends Nette\Object implements Application\IRouter
 					$this->xlat[$name] = $param;
 				}
 			}
-			$i -= 6;
+			$i -= 5;
 		}
 
 		// PARSE PATH PART OF MASK
 		$brackets = 0; // optional level
 		$re = '';
-		$sequence = array();
+		$sequence = [];
 		$autoOptional = TRUE;
-		$aliases = array();
+		$aliases = [];
 		do {
-			array_unshift($sequence, $parts[$i]);
-			$re = preg_quote($parts[$i], '#') . $re;
+			$part = $parts[$i]; // part of path
+			if (strpbrk($part, '<>') !== FALSE) {
+				throw new Nette\InvalidArgumentException("Unexpected '$part' in mask '$mask'.");
+			}
+			array_unshift($sequence, $part);
+			$re = preg_quote($part, '#') . $re;
 			if ($i === 0) {
 				break;
 			}
@@ -529,11 +539,10 @@ class Route extends Nette\Object implements Application\IRouter
 				}
 				array_unshift($sequence, $part);
 				$re = ($part[0] === '[' ? '(?:' : ')?') . $re;
-				$i -= 5;
+				$i -= 4;
 				continue;
 			}
 
-			$class = $parts[$i]; $i--; // validation class
 			$pattern = trim($parts[$i]); $i--; // validation condition (as regexp)
 			$default = $parts[$i]; $i--; // default value
 			$name = $parts[$i]; $i--; // parameter name
@@ -547,15 +556,8 @@ class Route extends Nette\Object implements Application\IRouter
 			}
 
 			// pattern, condition & metadata
-			if ($class !== '') {
-				if (!isset(static::$styles[$class])) {
-					throw new Nette\InvalidStateException("Parameter '$name' has '$class' flag, but Route::\$styles['$class'] is not set.");
-				}
-				$meta = static::$styles[$class];
-
-			} elseif (isset(static::$styles[$name])) {
+			if (isset(static::$styles[$name])) {
 				$meta = static::$styles[$name];
-
 			} else {
 				$meta = static::$styles['#'];
 			}
@@ -611,7 +613,7 @@ class Route extends Nette\Object implements Application\IRouter
 		} while (TRUE);
 
 		if ($brackets) {
-			throw new Nette\InvalidArgumentException("Missing closing ']' in mask '$mask'.");
+			throw new Nette\InvalidArgumentException("Missing '[' in mask '$mask'.");
 		}
 
 		$this->aliases = $aliases;
@@ -637,7 +639,7 @@ class Route extends Nette\Object implements Application\IRouter
 	 */
 	public function getDefaults()
 	{
-		$defaults = array();
+		$defaults = [];
 		foreach ($this->metadata as $name => $meta) {
 			if (isset($meta['fixity'])) {
 				$defaults[$name] = $meta[self::VALUE];
@@ -668,7 +670,7 @@ class Route extends Nette\Object implements Application\IRouter
 	public function getTargetPresenters()
 	{
 		if ($this->flags & self::ONE_WAY) {
-			return array();
+			return [];
 		}
 
 		$m = $this->metadata;
@@ -683,7 +685,7 @@ class Route extends Nette\Object implements Application\IRouter
 		}
 
 		if (isset($m[self::PRESENTER_KEY]['fixity']) && $m[self::PRESENTER_KEY]['fixity'] === self::CONSTANT) {
-			return array($module . $m[self::PRESENTER_KEY][self::VALUE]);
+			return [$module . $m[self::PRESENTER_KEY][self::VALUE]];
 		}
 		return NULL;
 	}
@@ -701,7 +703,7 @@ class Route extends Nette\Object implements Application\IRouter
 			return $arr;
 		}
 
-		$res = array();
+		$res = [];
 		$occupied = array_flip($xlat);
 		foreach ($arr as $k => $v) {
 			if (isset($xlat[$k])) {
@@ -786,42 +788,4 @@ class Route extends Nette\Object implements Application\IRouter
 		return str_replace('%2F', '/', rawurlencode($s));
 	}
 
-
-	/********************* Route::$styles manipulator ****************d*g**/
-
-
-	/**
-	 * @deprecated
-	 */
-	public static function addStyle($style, $parent = '#')
-	{
-		trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
-		if (isset(static::$styles[$style])) {
-			throw new Nette\InvalidArgumentException("Style '$style' already exists.");
-		}
-
-		if ($parent !== NULL) {
-			if (!isset(static::$styles[$parent])) {
-				throw new Nette\InvalidArgumentException("Parent style '$parent' doesn't exist.");
-			}
-			static::$styles[$style] = static::$styles[$parent];
-
-		} else {
-			static::$styles[$style] = array();
-		}
-	}
-
-
-	/**
-	 * @deprecated
-	 */
-	public static function setStyleProperty($style, $key, $value)
-	{
-		trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
-		if (!isset(static::$styles[$style])) {
-			throw new Nette\InvalidArgumentException("Style '$style' doesn't exist.");
-		}
-		static::$styles[$style][$key] = $value;
-	}
-
 }
