@@ -1,26 +1,23 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\Routers;
 
-use Nette,
-	Nette\Application;
+use Nette;
+use Nette\Application;
 
 
 /**
  * The bidirectional route for trivial routing via query parameters.
- *
- * @author     David Grudl
- *
- * @property-read array $defaults
- * @property-read int $flags
  */
-class SimpleRouter extends Nette\Object implements Application\IRouter
+class SimpleRouter implements Application\IRouter
 {
+	use Nette\SmartObject;
+
 	const PRESENTER_KEY = 'presenter';
 	const MODULE_KEY = 'module';
 
@@ -38,17 +35,17 @@ class SimpleRouter extends Nette\Object implements Application\IRouter
 	 * @param  array   default values
 	 * @param  int     flags
 	 */
-	public function __construct($defaults = array(), $flags = 0)
+	public function __construct($defaults = [], $flags = 0)
 	{
 		if (is_string($defaults)) {
-			$a = strrpos($defaults, ':');
-			if (!$a) {
+			list($presenter, $action) = Nette\Application\Helpers::splitName($defaults);
+			if (!$presenter) {
 				throw new Nette\InvalidArgumentException("Argument must be array or string in format Presenter:action, '$defaults' given.");
 			}
-			$defaults = array(
-				self::PRESENTER_KEY => substr($defaults, 0, $a),
-				'action' => $a === strlen($defaults) - 1 ? Application\UI\Presenter::DEFAULT_ACTION : substr($defaults, $a + 1),
-			);
+			$defaults = [
+				self::PRESENTER_KEY => $presenter,
+				'action' => $action === '' ? Application\UI\Presenter::DEFAULT_ACTION : $action,
+			];
 		}
 
 		if (isset($defaults[self::MODULE_KEY])) {
@@ -58,6 +55,9 @@ class SimpleRouter extends Nette\Object implements Application\IRouter
 
 		$this->defaults = $defaults;
 		$this->flags = $flags;
+		if ($flags & self::SECURED) {
+			trigger_error('IRouter::SECURED is deprecated, router by default keeps the used protocol.', E_USER_DEPRECATED);
+		}
 	}
 
 
@@ -87,7 +87,7 @@ class SimpleRouter extends Nette\Object implements Application\IRouter
 			$params,
 			$httpRequest->getPost(),
 			$httpRequest->getFiles(),
-			array(Application\Request::SECURED => $httpRequest->isSecured())
+			[Application\Request::SECURED => $httpRequest->isSecured()]
 		);
 	}
 
@@ -105,7 +105,7 @@ class SimpleRouter extends Nette\Object implements Application\IRouter
 
 		// presenter name
 		$presenter = $appRequest->getPresenterName();
-		if (strncasecmp($presenter, $this->module, strlen($this->module)) === 0) {
+		if (strncmp($presenter, $this->module, strlen($this->module)) === 0) {
 			$params[self::PRESENTER_KEY] = substr($presenter, strlen($this->module));
 		} else {
 			return NULL;
@@ -118,7 +118,7 @@ class SimpleRouter extends Nette\Object implements Application\IRouter
 			}
 		}
 
-		$url = ($this->flags & self::SECURED ? 'https://' : 'http://') . $refUrl->getAuthority() . $refUrl->getPath();
+		$url = ($this->flags & self::SECURED ? 'https://' : $refUrl->getScheme() . '://') . $refUrl->getAuthority() . $refUrl->getPath();
 		$sep = ini_get('arg_separator.input');
 		$query = http_build_query($params, '', $sep ? $sep[0] : '&');
 		if ($query != '') { // intentionally ==
