@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\UI;
@@ -12,12 +12,10 @@ use Nette;
 
 /**
  * Signal exception.
- *
- * @author     David Grudl
  */
 class BadSignalException extends Nette\Application\BadRequestException
 {
 	/** @var int */
-	protected $defaultCode = 403;
+	protected $code = 403;
 
 }
