@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\UI;
@@ -13,18 +13,18 @@ use Nette;
 /**
  * Control is renderable Presenter component.
  *
- * @author     David Grudl
- *
- * @property-read Nette\Templating\ITemplate $template
- * @property-read string $snippetId
+ * @property-read ITemplate $template
  */
-abstract class Control extends PresenterComponent implements IRenderable
+abstract class Control extends Component implements IRenderable
 {
-	/** @var Nette\Templating\ITemplate */
+	/** @var ITemplateFactory */
+	private $templateFactory;
+
+	/** @var ITemplate */
 	private $template;
 
 	/** @var array */
-	private $invalidSnippets = array();
+	private $invalidSnippets = [];
 
 	/** @var bool */
 	public $snippetMode;
@@ -33,16 +33,22 @@ abstract class Control extends PresenterComponent implements IRenderable
 	/********************* template factory ****************d*g**/
 
 
+	public function setTemplateFactory(ITemplateFactory $templateFactory)
+	{
+		$this->templateFactory = $templateFactory;
+	}
+
+
 	/**
-	 * @return Nette\Templating\ITemplate
+	 * @return ITemplate
 	 */
 	public function getTemplate()
 	{
 		if ($this->template === NULL) {
 			$value = $this->createTemplate();
-			if (!$value instanceof Nette\Templating\ITemplate && $value !== NULL) {
+			if (!$value instanceof ITemplate && $value !== NULL) {
 				$class2 = get_class($value); $class = get_class($this);
-				throw new Nette\UnexpectedValueException("Object returned by $class::createTemplate() must be instance of Nette\\Templating\\ITemplate, '$class2' given.");
+				throw new Nette\UnexpectedValueException("Object returned by $class::createTemplate() must be instance of Nette\\Application\\UI\\ITemplate, '$class2' given.");
 			}
 			$this->template = $value;
 		}
@@ -51,49 +57,22 @@ abstract class Control extends PresenterComponent implements IRenderable
 
 
 	/**
-	 * @param  string|NULL
-	 * @return Nette\Templating\ITemplate
+	 * @return ITemplate
 	 */
-	protected function createTemplate($class = NULL)
+	protected function createTemplate()
 	{
-		$template = $class ? new $class : new Nette\Templating\FileTemplate;
-		$presenter = $this->getPresenter(FALSE);
-		$template->onPrepareFilters[] = $this->templatePrepareFilters;
-		$template->registerHelperLoader('Nette\Templating\Helpers::loader');
-
-		// default parameters
-		$template->control = $template->_control = $this;
-		$template->presenter = $template->_presenter = $presenter;
-		if ($presenter instanceof Presenter) {
-			$template->setCacheStorage($presenter->getContext()->getService('nette.templateCacheStorage'));
-			$template->user = $presenter->getUser();
-			$template->netteHttpResponse = $presenter->getHttpResponse();
-			$template->netteCacheStorage = $presenter->getContext()->getByType('Nette\Caching\IStorage');
-			$template->baseUri = $template->baseUrl = rtrim($presenter->getHttpRequest()->getUrl()->getBaseUrl(), '/');
-			$template->basePath = preg_replace('#https?://[^/]+#A', '', $template->baseUrl);
-
-			// flash message
-			if ($presenter->hasFlashSession()) {
-				$id = $this->getParameterId('flash');
-				$template->flashes = $presenter->getFlashSession()->$id;
-			}
-		}
-		if (!isset($template->flashes) || !is_array($template->flashes)) {
-			$template->flashes = array();
-		}
-
-		return $template;
+		$templateFactory = $this->templateFactory ?: $this->getPresenter()->getTemplateFactory();
+		return $templateFactory->createTemplate($this);
 	}
 
 
 	/**
 	 * Descendant can override this method to customize template compile-time filters.
-	 * @param  Nette\Templating\Template
+	 * @param  ITemplate
 	 * @return void
 	 */
 	public function templatePrepareFilters($template)
 	{
-		$template->registerFilter($this->getPresenter()->getContext()->createService('nette.latte'));
 	}
 
 
@@ -107,10 +86,10 @@ abstract class Control extends PresenterComponent implements IRenderable
 	{
 		$id = $this->getParameterId('flash');
 		$messages = $this->getPresenter()->getFlashSession()->$id;
-		$messages[] = $flash = (object) array(
+		$messages[] = $flash = (object) [
 			'message' => $message,
 			'type' => $type,
-		);
+		];
 		$this->getTemplate()->flashes = $messages;
 		$this->getPresenter()->getFlashSession()->$id = $messages;
 		return $flash;
@@ -127,13 +106,13 @@ abstract class Control extends PresenterComponent implements IRenderable
 	public function redrawControl($snippet = NULL, $redraw = TRUE)
 	{
 		if ($redraw) {
-			$this->invalidSnippets[$snippet] = TRUE;
+			$this->invalidSnippets[$snippet === NULL ? "\0" : $snippet] = TRUE;
 
 		} elseif ($snippet === NULL) {
-			$this->invalidSnippets = array();
+			$this->invalidSnippets = [];
 
 		} else {
-			unset($this->invalidSnippets[$snippet]);
+			$this->invalidSnippets[$snippet] = FALSE;
 		}
 	}
 
@@ -141,12 +120,14 @@ abstract class Control extends PresenterComponent implements IRenderable
 	/** @deprecated */
 	function invalidateControl($snippet = NULL)
 	{
+		trigger_error(__METHOD__ . '() is deprecated; use $this->redrawControl($snippet) instead.', E_USER_DEPRECATED);
 		$this->redrawControl($snippet);
 	}
 
 	/** @deprecated */
 	function validateControl($snippet = NULL)
 	{
+		trigger_error(__METHOD__ . '() is deprecated; use $this->redrawControl($snippet, FALSE) instead.', E_USER_DEPRECATED);
 		$this->redrawControl($snippet, FALSE);
 	}
 
@@ -163,7 +144,7 @@ abstract class Control extends PresenterComponent implements IRenderable
 				return TRUE;
 
 			} else {
-				$queue = array($this);
+				$queue = [$this];
 				do {
 					foreach (array_shift($queue)->getComponents() as $component) {
 						if ($component instanceof IRenderable) {
@@ -181,8 +162,10 @@ abstract class Control extends PresenterComponent implements IRenderable
 				return FALSE;
 			}
 
+		} elseif (isset($this->invalidSnippets[$snippet])) {
+			return $this->invalidSnippets[$snippet];
 		} else {
-			return isset($this->invalidSnippets[NULL]) || isset($this->invalidSnippets[$snippet]);
+			return isset($this->invalidSnippets["\0"]);
 		}
 	}
 
