@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\UI;
 
-use Nette;
-
 
 /**
  * Component with ability to repaint.
- *
- * @author     David Grudl
  */
 interface IRenderable
 {
@@ -22,7 +18,7 @@ interface IRenderable
 	 * Forces control to repaint.
 	 * @return void
 	 */
-	function invalidateControl();
+	function redrawControl();
 
 	/**
 	 * Is required to repaint the control?
