@@ -14,8 +14,9 @@ use Tracy;
 /**
  * Session panel for Debugger Bar.
  */
-class SessionPanel extends Nette\Object implements Tracy\IBarPanel
+class SessionPanel implements Tracy\IBarPanel
 {
+	use Nette\SmartObject;
 
 	/**
 	 * Renders tab.
