@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Caching;
@@ -12,11 +12,11 @@ use Nette;
 
 /**
  * Output caching helper.
- *
- * @author     David Grudl
  */
-class OutputHelper extends Nette\Object
+class OutputHelper
 {
+	use Nette\SmartObject;
+
 	/** @var array */
 	public $dependencies;
 
