@@ -1,27 +1,23 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Caching\Storages;
 
-use Nette;
-
 
 /**
  * Cache journal provider.
- *
- * @author     Jan Smitka
  */
 interface IJournal
 {
 
 	/**
 	 * Writes entry information into the journal.
-	 * @param  string $key
-	 * @param  array  $dependencies
+	 * @param  string
+	 * @param  array
 	 * @return void
 	 */
 	function write($key, array $dependencies);
@@ -29,7 +25,7 @@ interface IJournal
 
 	/**
 	 * Cleans entries from journal.
-	 * @param  array  $conditions
+	 * @param  array
 	 * @return array of removed items or NULL when performing a full cleanup
 	 */
 	function clean(array $conditions);
