@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\ComponentModel;
@@ -15,13 +15,13 @@ use Nette;
  *
  * Components are objects implementing IComponent. They has parent component and own name.
  *
- * @author     David Grudl
- *
  * @property-read string $name
  * @property-read IContainer|NULL $parent
  */
-abstract class Component extends Nette\Object implements IComponent
+abstract class Component implements IComponent
 {
+	use Nette\SmartObject;
+
 	/** @var IContainer */
 	private $parent;
 
@@ -29,11 +29,12 @@ abstract class Component extends Nette\Object implements IComponent
 	private $name;
 
 	/** @var array of [type => [obj, depth, path, is_monitored?]] */
-	private $monitors = array();
+	private $monitors = [];
 
 
-	public function __construct(IContainer $parent = NULL, $name = NULL)
+	public function __construct()
 	{
+		list($parent, $name) = func_get_args() + [NULL, NULL];
 		if ($parent !== NULL) {
 			$parent->addComponent($this, $name);
 
@@ -69,10 +70,10 @@ abstract class Component extends Nette\Object implements IComponent
 			}
 
 			if ($obj) {
-				$this->monitors[$type] = array($obj, $depth, substr($path, 1), FALSE);
+				$this->monitors[$type] = [$obj, $depth, substr($path, 1), FALSE];
 
 			} else {
-				$this->monitors[$type] = array(NULL, NULL, NULL, FALSE); // not found
+				$this->monitors[$type] = [NULL, NULL, NULL, FALSE]; // not found
 			}
 		}
 
@@ -91,7 +92,7 @@ abstract class Component extends Nette\Object implements IComponent
 	 * @param  bool   throw exception if component doesn't exist?
 	 * @return string
 	 */
-	public function lookupPath($type, $need = TRUE)
+	public function lookupPath($type = NULL, $need = TRUE)
 	{
 		$this->lookup($type, $need);
 		return $this->monitors[$type][2];
@@ -205,7 +206,7 @@ abstract class Component extends Nette\Object implements IComponent
 				$this->name = $name;
 			}
 
-			$tmp = array();
+			$tmp = [];
 			$this->refreshMonitors(0, $tmp);
 		}
 		return $this;
@@ -230,11 +231,11 @@ abstract class Component extends Nette\Object implements IComponent
 	 * @param  array
 	 * @return void
 	 */
-	private function refreshMonitors($depth, & $missing = NULL, & $listeners = array())
+	private function refreshMonitors($depth, & $missing = NULL, & $listeners = [])
 	{
 		if ($this instanceof IContainer) {
 			foreach ($this->getComponents() as $component) {
-				if ($component instanceof Component) {
+				if ($component instanceof self) {
 					$component->refreshMonitors($depth + 1, $missing, $listeners);
 				}
 			}
@@ -244,8 +245,8 @@ abstract class Component extends Nette\Object implements IComponent
 			foreach ($this->monitors as $type => $rec) {
 				if (isset($rec[1]) && $rec[1] > $depth) {
 					if ($rec[3]) { // monitored
-						$this->monitors[$type] = array(NULL, NULL, NULL, TRUE);
-						$listeners[] = array($this, $rec[0]);
+						$this->monitors[$type] = [NULL, NULL, NULL, TRUE];
+						$listeners[] = [$this, $rec[0]];
 					} else { // not monitored, just randomly cached
 						unset($this->monitors[$type]);
 					}
@@ -261,12 +262,12 @@ abstract class Component extends Nette\Object implements IComponent
 					unset($this->monitors[$type]);
 
 				} elseif (isset($missing[$type])) { // known from previous lookup
-					$this->monitors[$type] = array(NULL, NULL, NULL, TRUE);
+					$this->monitors[$type] = [NULL, NULL, NULL, TRUE];
 
 				} else {
 					$this->monitors[$type] = NULL; // forces re-lookup
 					if ($obj = $this->lookup($type, FALSE)) {
-						$listeners[] = array($this, $obj);
+						$listeners[] = [$this, $obj];
 					} else {
 						$missing[$type] = TRUE;
 					}
@@ -277,8 +278,12 @@ abstract class Component extends Nette\Object implements IComponent
 
 		if ($depth === 0) { // call listeners
 			$method = $missing === NULL ? 'detached' : 'attached';
+			$prev = [];
 			foreach ($listeners as $item) {
-				$item[0]->$method($item[1]);
+				if (!in_array($item, $prev, TRUE)) {
+					$item[0]->$method($item[1]);
+					$prev[] = $item;
+				}
 			}
 		}
 	}
