@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\DI\Extensions;
@@ -12,8 +12,6 @@ use Nette;
 
 /**
  * Constant definitions.
- *
- * @author     David Grudl
  */
 class ConstantsExtension extends Nette\DI\CompilerExtension
 {
@@ -21,7 +19,7 @@ class ConstantsExtension extends Nette\DI\CompilerExtension
 	public function afterCompile(Nette\PhpGenerator\ClassType $class)
 	{
 		foreach ($this->getConfig() as $name => $value) {
-			$class->methods['initialize']->addBody('define(?, ?);', array($name, $value));
+			$class->getMethod('initialize')->addBody('define(?, ?);', [$name, $value]);
 		}
 	}
 
