@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database\Drivers;
@@ -12,8 +12,6 @@ use Nette;
 
 /**
  * Supplemental SQLite2 database driver.
- *
- * @author     David Grudl
  */
 class Sqlite2Driver extends SqliteDriver
 {
@@ -35,4 +33,12 @@ class Sqlite2Driver extends SqliteDriver
 		throw new Nette\NotSupportedException; // @see http://www.sqlite.org/foreignkeys.html
 	}
 
+
+	/**
+	 * Not supported.
+	 */
+	public function getColumnTypes(\PDOStatement $statement)
+	{
+	}
+
 }
