@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database;
 
-use Nette;
-
 
 /**
  * Supplemental PDO database driver.
- *
- * @author     David Grudl
  */
 interface ISupplementalDriver
 {
@@ -21,7 +17,13 @@ interface ISupplementalDriver
 		SUPPORT_SELECT_UNGROUPED_COLUMNS = 'ungrouped_cols',
 		SUPPORT_MULTI_INSERT_AS_SELECT = 'insert_as_select',
 		SUPPORT_MULTI_COLUMN_AS_OR_COND = 'multi_column_as_or',
-		SUPPORT_SUBSELECT = 'subselect';
+		SUPPORT_SUBSELECT = 'subselect',
+		SUPPORT_SCHEMA = 'schema';
+
+	/**
+	 * @return DriverException
+	 */
+	function convertException(\PDOException $e);
 
 	/**
 	 * Delimites identifier for use in a SQL statement.
@@ -43,6 +45,12 @@ interface ISupplementalDriver
 	 */
 	function formatDateTime(/*\DateTimeInterface*/ $value);
 
+	/**
+	 * Formats date-time interval for use in a SQL statement.
+	 * @return string
+	 */
+	//function formatDateInterval(\DateInterval $value);
+
 	/**
 	 * Encodes string for use in a LIKE statement.
 	 * @param  string
@@ -54,8 +62,8 @@ interface ISupplementalDriver
 	/**
 	 * Injects LIMIT/OFFSET to the SQL query.
 	 * @param  string  SQL query that will be modified.
-	 * @param  int
-	 * @param  int
+	 * @param  int|NULL
+	 * @param  int|NULL
 	 * @return void
 	 */
 	function applyLimit(& $sql, $limit, $offset);
@@ -80,7 +88,7 @@ interface ISupplementalDriver
 	/**
 	 * Returns metadata for all columns in a table.
 	 * @param  string
-	 * @return array of [name, nativetype [, table, fullname, (int) size, (bool) nullable, (mixed) default, (bool) autoincrement, (array) vendor]]
+	 * @return array of [name, nativetype, primary [, table, fullname, (int) size, (bool) nullable, (mixed) default, (bool) autoincrement, (array) vendor]]
 	 */
 	function getColumns($table);
 
@@ -99,13 +107,15 @@ interface ISupplementalDriver
 	function getForeignKeys($table);
 
 	/**
-	 * Returns associative array of detected types (IReflection::FIELD_*) in result set.
+	 * Returns associative array of detected types (IStructure::FIELD_*) in result set.
+	 * @param  \PDOStatement
 	 * @return array
 	 */
 	function getColumnTypes(\PDOStatement $statement);
 
 	/**
 	 * Cheks if driver supports specific property
+	 * @param  string self::SUPPORT_* property
 	 * @return bool
 	 */
 	function isSupported($item);
