@@ -1,26 +1,23 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database;
 
-use Nette,
-	PDO;
+use Nette;
+use PDO;
 
 
 /**
  * Represents a result set.
- *
- * @author     David Grudl
- * @author     Jan Skrasek
- *
- * @property-read Connection $connection
  */
-class ResultSet extends Nette\Object implements \Iterator, IRowContainer
+class ResultSet implements \Iterator, IRowContainer
 {
+	use Nette\SmartObject;
+
 	/** @var Connection */
 	private $connection;
 
@@ -60,12 +57,24 @@ class ResultSet extends Nette\Object implements \Iterator, IRowContainer
 		$this->queryString = $queryString;
 		$this->params = $params;
 
-		if (substr($queryString, 0, 2) === '::') {
-			$connection->getPdo()->{substr($queryString, 2)}();
-		} elseif ($queryString !== NULL) {
-			$this->pdoStatement = $connection->getPdo()->prepare($queryString);
-			$this->pdoStatement->setFetchMode(PDO::FETCH_ASSOC);
-			$this->pdoStatement->execute($params);
+		try {
+			if (substr($queryString, 0, 2) === '::') {
+				$connection->getPdo()->{substr($queryString, 2)}();
+			} elseif ($queryString !== NULL) {
+				static $types = ['boolean' => PDO::PARAM_BOOL, 'integer' => PDO::PARAM_INT,
+					'resource' => PDO::PARAM_LOB, 'NULL' => PDO::PARAM_NULL];
+				$this->pdoStatement = $connection->getPdo()->prepare($queryString);
+				foreach ($params as $key => $value) {
+					$type = gettype($value);
+					$this->pdoStatement->bindValue(is_int($key) ? $key + 1 : $key, $value, isset($types[$type]) ? $types[$type] : PDO::PARAM_STR);
+				}
+				$this->pdoStatement->setFetchMode(PDO::FETCH_ASSOC);
+				$this->pdoStatement->execute();
+			}
+		} catch (\PDOException $e) {
+			$e = $this->supplementalDriver->convertException($e);
+			$e->queryString = $queryString;
+			throw $e;
 		}
 		$this->time = microtime(TRUE) - $time;
 	}
@@ -148,31 +157,31 @@ class ResultSet extends Nette\Object implements \Iterator, IRowContainer
 
 		foreach ($this->types as $key => $type) {
 			$value = $row[$key];
-			if ($value === NULL || $value === FALSE || $type === IReflection::FIELD_TEXT) {
+			if ($value === NULL || $value === FALSE || $type === IStructure::FIELD_TEXT) {
 
-			} elseif ($type === IReflection::FIELD_INTEGER) {
+			} elseif ($type === IStructure::FIELD_INTEGER) {
 				$row[$key] = is_float($tmp = $value * 1) ? $value : $tmp;
 
-			} elseif ($type === IReflection::FIELD_FLOAT) {
+			} elseif ($type === IStructure::FIELD_FLOAT) {
 				if (($pos = strpos($value, '.')) !== FALSE) {
 					$value = rtrim(rtrim($pos === 0 ? "0$value" : $value, '0'), '.');
 				}
 				$float = (float) $value;
 				$row[$key] = (string) $float === $value ? $float : $value;
 
-			} elseif ($type === IReflection::FIELD_BOOL) {
+			} elseif ($type === IStructure::FIELD_BOOL) {
 				$row[$key] = ((bool) $value) && $value !== 'f' && $value !== 'F';
 
-			} elseif ($type === IReflection::FIELD_DATETIME || $type === IReflection::FIELD_DATE || $type === IReflection::FIELD_TIME) {
-				$row[$key] = new Nette\DateTime($value);
+			} elseif ($type === IStructure::FIELD_DATETIME || $type === IStructure::FIELD_DATE || $type === IStructure::FIELD_TIME) {
+				$row[$key] = new Nette\Utils\DateTime($value);
 
-			} elseif ($type === IReflection::FIELD_TIME_INTERVAL) {
+			} elseif ($type === IStructure::FIELD_TIME_INTERVAL) {
 				preg_match('#^(-?)(\d+)\D(\d+)\D(\d+)\z#', $value, $m);
 				$row[$key] = new \DateInterval("PT$m[2]H$m[3]M$m[4]S");
 				$row[$key]->invert = (int) (bool) $m[1];
 
-			} elseif ($type === IReflection::FIELD_UNIX_TIMESTAMP) {
-				$row[$key] = Nette\DateTime::from($value);
+			} elseif ($type === IStructure::FIELD_UNIX_TIMESTAMP) {
+				$row[$key] = Nette\Utils\DateTime::from($value);
 			}
 		}
 
@@ -244,6 +253,10 @@ class ResultSet extends Nette\Object implements \Iterator, IRowContainer
 		if (!$data) {
 			$this->pdoStatement->closeCursor();
 			return FALSE;
+
+		} elseif ($this->result === NULL && count($data) !== $this->pdoStatement->columnCount()) {
+			$duplicates = Helpers::findDuplicates($this->pdoStatement);
+			trigger_error("Found duplicate columns in database result set: $duplicates.", E_USER_NOTICE);
 		}
 
 		$row = new Row;
@@ -253,10 +266,6 @@ class ResultSet extends Nette\Object implements \Iterator, IRowContainer
 			}
 		}
 
-		if ($this->result === NULL && count($data) !== $this->pdoStatement->columnCount()) {
-			trigger_error('Found duplicate columns in database result set.', E_USER_NOTICE);
-		}
-
 		$this->resultKey++;
 		return $this->result = $row;
 	}
@@ -264,6 +273,7 @@ class ResultSet extends Nette\Object implements \Iterator, IRowContainer
 
 	/**
 	 * Fetches single field.
+	 * @param  int
 	 * @return mixed|FALSE
 	 */
 	public function fetchField($column = 0)
@@ -294,19 +304,12 @@ class ResultSet extends Nette\Object implements \Iterator, IRowContainer
 	}
 
 
-	/** @deprecated */
-	function columnCount()
+	/**
+	 * @inheritDoc
+	 */
+	public function fetchAssoc($path)
 	{
-		trigger_error(__METHOD__ . '() is deprecated; use getColumnCount() instead.', E_USER_DEPRECATED);
-		return $this->getColumnCount();
-	}
-
-
-	/** @deprecated */
-	function rowCount()
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use getRowCount() instead.', E_USER_DEPRECATED);
-		return $this->getRowCount();
+		return Nette\Utils\Arrays::associate($this->fetchAll(), $path);
 	}
 
 }
