@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database;
@@ -12,13 +12,17 @@ use Nette;
 
 /**
  * Represents a single table row.
- *
- * @author     David Grudl
- * @author     Jan Skrasek
  */
-class Row extends Nette\ArrayHash implements IRow
+class Row extends Nette\Utils\ArrayHash implements IRow
 {
 
+	public function __get($key)
+	{
+		$hint = Nette\Utils\ObjectMixin::getSuggestion(array_keys((array) $this), $key);
+		throw new Nette\MemberAccessException("Cannot read an undeclared column '$key'" . ($hint ? ", did you mean '$hint'?" : '.'));
+	}
+
+
 	/**
 	 * Returns a item.
 	 * @param  mixed  key or index
@@ -29,7 +33,7 @@ class Row extends Nette\ArrayHash implements IRow
 		if (is_int($key)) {
 			$arr = array_slice((array) $this, $key, 1);
 			if (!$arr) {
-				trigger_error('Undefined offset: ' . __CLASS__ . "[$key]", E_USER_NOTICE);
+				throw new Nette\MemberAccessException("Cannot read an undeclared column '$key'.");
 			}
 			return current($arr);
 		}
