@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database;
@@ -12,12 +12,11 @@ use Nette;
 
 /**
  * SQL literal value.
- *
- * @author     Jakub Vrana
- * @author     Jan Skrasek
  */
-class SqlLiteral extends Nette\Object
+class SqlLiteral
 {
+	use Nette\SmartObject;
+
 	/** @var string */
 	private $value;
 
@@ -25,7 +24,7 @@ class SqlLiteral extends Nette\Object
 	private $parameters;
 
 
-	public function __construct($value, array $parameters = array())
+	public function __construct($value, array $parameters = [])
 	{
 		$this->value = (string) $value;
 		$this->parameters = $parameters;
