@@ -1,22 +1,18 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database\Table;
 
-use Nette,
-	Nette\Database\Reflection\MissingReferenceException;
+use Nette;
 
 
 /**
  * Single row representation.
  * ActiveRow is based on the great library NotORM http://www.notorm.com written by Jakub Vrana.
- *
- * @author     Jakub Vrana
- * @author     Jan Skrasek
  */
 class ActiveRow implements \IteratorAggregate, IRow
 {
@@ -39,7 +35,6 @@ class ActiveRow implements \IteratorAggregate, IRow
 
 	/**
 	 * @internal
-	 * @ignore
 	 */
 	public function setTable(Selection $table)
 	{
@@ -60,7 +55,10 @@ class ActiveRow implements \IteratorAggregate, IRow
 	{
 		try {
 			return (string) $this->getPrimary();
+		} catch (\Throwable $e) {
 		} catch (\Exception $e) {
+		}
+		if (isset($e)) {
 			if (func_num_args()) {
 				throw $e;
 			}
@@ -82,7 +80,7 @@ class ActiveRow implements \IteratorAggregate, IRow
 	/**
 	 * Returns primary key value.
 	 * @param  bool
-	 * @return mixed possible int, string, array, object (Nette\DateTime)
+	 * @return mixed possible int, string, array, object (Nette\Utils\DateTime)
 	 */
 	public function getPrimary($need = TRUE)
 	{
@@ -100,7 +98,7 @@ class ActiveRow implements \IteratorAggregate, IRow
 			}
 
 		} else {
-			$primaryVal = array();
+			$primaryVal = [];
 			foreach ($primary as $key) {
 				if (!isset($this->data[$key])) {
 					if ($need) {
@@ -135,11 +133,12 @@ class ActiveRow implements \IteratorAggregate, IRow
 	 */
 	public function ref($key, $throughColumn = NULL)
 	{
-		if (!$throughColumn) {
-			list($key, $throughColumn) = $this->table->getDatabaseReflection()->getBelongsToReference($this->table->getName(), $key);
+		$row = $this->table->getReferencedTable($this, $key, $throughColumn);
+		if ($row === FALSE) {
+			throw new Nette\MemberAccessException("No reference found for \${$this->table->getName()}->ref($key).");
 		}
 
-		return $this->getReference($key, $throughColumn);
+		return $row;
 	}
 
 
@@ -151,13 +150,12 @@ class ActiveRow implements \IteratorAggregate, IRow
 	 */
 	public function related($key, $throughColumn = NULL)
 	{
-		if (strpos($key, '.') !== FALSE) {
-			list($key, $throughColumn) = explode('.', $key);
-		} elseif (!$throughColumn) {
-			list($key, $throughColumn) = $this->table->getDatabaseReflection()->getHasManyReference($this->table->getName(), $key);
+		$groupedSelection = $this->table->getReferencingTable($key, $throughColumn, $this[$this->table->getPrimary()]);
+		if (!$groupedSelection) {
+			throw new Nette\MemberAccessException("No reference found for \${$this->table->getName()}->related($key).");
 		}
 
-		return $this->table->getReferencingTable($key, $throughColumn, $this[$this->table->getPrimary()]);
+		return $groupedSelection;
 	}
 
 
@@ -168,10 +166,23 @@ class ActiveRow implements \IteratorAggregate, IRow
 	 */
 	public function update($data)
 	{
+		if ($data instanceof \Traversable) {
+			$data = iterator_to_array($data);
+		}
+
+		$primary = $this->getPrimary();
+		if (!is_array($primary)) {
+			$primary = [$this->table->getPrimary() => $primary];
+		}
+
 		$selection = $this->table->createSelectionInstance()
-			->wherePrimary($this->getPrimary());
+			->wherePrimary($primary);
 
 		if ($selection->update($data)) {
+			if ($tmp = array_intersect_key($data, $primary)) {
+				$selection = $this->table->createSelectionInstance()
+					->wherePrimary($tmp + $primary);
+			}
 			$selection->select('*');
 			if (($row = $selection->fetch()) === FALSE) {
 				throw new Nette\InvalidStateException('Database refetch failed; row does not exist!');
@@ -266,31 +277,32 @@ class ActiveRow implements \IteratorAggregate, IRow
 	}
 
 
+	/**
+	 * @param  string
+	 * @return ActiveRow|mixed
+	 * @throws Nette\MemberAccessException
+	 */
 	public function &__get($key)
 	{
-		$this->accessColumn($key);
-		if (array_key_exists($key, $this->data)) {
+		if ($this->accessColumn($key)) {
 			return $this->data[$key];
 		}
 
-		try {
-			list($table, $column) = $this->table->getDatabaseReflection()->getBelongsToReference($this->table->getName(), $key);
-			$referenced = $this->getReference($table, $column);
-			if ($referenced !== FALSE) {
-				$this->accessColumn($key, FALSE);
-				return $referenced;
-			}
-		} catch(MissingReferenceException $e) {}
+		$referenced = $this->table->getReferencedTable($this, $key);
+		if ($referenced !== FALSE) {
+			$this->accessColumn($key, FALSE);
+			return $referenced;
+		}
 
 		$this->removeAccessColumn($key);
-		throw new Nette\MemberAccessException("Cannot read an undeclared column '$key'.");
+		$hint = Nette\Utils\ObjectMixin::getSuggestion(array_keys($this->data), $key);
+		throw new Nette\MemberAccessException("Cannot read an undeclared column '$key'" . ($hint ? ", did you mean '$hint'?" : '.'));
 	}
 
 
 	public function __isset($key)
 	{
-		$this->accessColumn($key);
-		if (array_key_exists($key, $this->data)) {
+		if ($this->accessColumn($key)) {
 			return isset($this->data[$key]);
 		}
 		$this->removeAccessColumn($key);
@@ -304,13 +316,19 @@ class ActiveRow implements \IteratorAggregate, IRow
 	}
 
 
-	protected function accessColumn($key, $selectColumn = TRUE)
+	/**
+	 * @internal
+	 */
+	public function accessColumn($key, $selectColumn = TRUE)
 	{
-		$this->table->accessColumn($key, $selectColumn);
-		if ($this->table->getDataRefreshed() && !$this->dataRefreshed) {
+		if ($this->table->accessColumn($key, $selectColumn) && !$this->dataRefreshed) {
+			if (!isset($this->table[$this->getSignature()])) {
+				throw new Nette\InvalidStateException("Database refetch failed; row with signature '{$this->getSignature()}' does not exist!");
+			}
 			$this->data = $this->table[$this->getSignature()]->data;
 			$this->dataRefreshed = TRUE;
 		}
+		return isset($this->data[$key]) || array_key_exists($key, $this->data);
 	}
 
 
@@ -319,17 +337,4 @@ class ActiveRow implements \IteratorAggregate, IRow
 		$this->table->removeAccessColumn($key);
 	}
 
-
-	protected function getReference($table, $column)
-	{
-		$this->accessColumn($column);
-		if (array_key_exists($column, $this->data)) {
-			$value = $this->data[$column];
-			$referenced = $this->table->getReferencedTable($table, $column, $value);
-			return isset($referenced[$value]) ? $referenced[$value] : NULL; // referenced row may not exist
-		}
-
-		return FALSE;
-	}
-
 }
