@@ -1,21 +1,20 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database\Table;
 
 use Nette;
+use Nette\Database\Context;
+use Nette\Database\IConventions;
 
 
 /**
  * Representation of filtered table grouped by some column.
  * GroupedSelection is based on the great library NotORM http://www.notorm.com written by Jakub Vrana.
- *
- * @author     Jakub Vrana
- * @author     Jan Skrasek
  */
 class GroupedSelection extends Selection
 {
@@ -34,15 +33,18 @@ class GroupedSelection extends Selection
 
 	/**
 	 * Creates filtered and grouped table representation.
-	 * @param  Selection  $refTable
+	 * @param  Context
+	 * @param  IConventions
 	 * @param  string  database table name
 	 * @param  string  joining column
+	 * @param  Selection
+	 * @param  Nette\Caching\IStorage|NULL
 	 */
-	public function __construct(Selection $refTable, $table, $column)
+	public function __construct(Context $context, IConventions $conventions, $tableName, $column, Selection $refTable, Nette\Caching\IStorage $cacheStorage = NULL)
 	{
 		$this->refTable = $refTable;
 		$this->column = $column;
-		parent::__construct($refTable->connection, $table, $refTable->reflection, $refTable->cache ? $refTable->cache->getStorage() : NULL);
+		parent::__construct($context, $conventions, $tableName, $cacheStorage);
 	}
 
 
@@ -50,7 +52,7 @@ class GroupedSelection extends Selection
 	 * Sets active group.
 	 * @internal
 	 * @param  int  primary key of grouped rows
-	 * @return GroupedSelection
+	 * @return self
 	 */
 	public function setActive($active)
 	{
@@ -59,34 +61,24 @@ class GroupedSelection extends Selection
 	}
 
 
-	/** @deprecated */
-	public function through($column)
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use ' . __CLASS__ . '::related("' . $this->name . '", "' . $column . '") instead.', E_USER_DEPRECATED);
-		$this->column = $column;
-		$this->delimitedColumn = $this->refTable->connection->getSupplementalDriver()->delimite($this->column);
-		return $this;
-	}
-
-
-	public function select($columns)
+	public function select($columns, ...$params)
 	{
 		if (!$this->sqlBuilder->getSelect()) {
 			$this->sqlBuilder->addSelect("$this->name.$this->column");
 		}
 
-		return call_user_func_array('parent::select', func_get_args());
+		return parent::select($columns, ...$params);
 	}
 
 
-	public function order($columns)
+	public function order($columns, ...$params)
 	{
 		if (!$this->sqlBuilder->getOrder()) {
 			// improve index utilization
 			$this->sqlBuilder->addOrder("$this->name.$this->column" . (preg_match('~\bDESC\z~i', $columns) ? ' DESC' : ''));
 		}
 
-		return call_user_func_array('parent::order', func_get_args());
+		return parent::order($columns, ...$params);
 	}
 
 
@@ -95,10 +87,10 @@ class GroupedSelection extends Selection
 
 	public function aggregation($function)
 	{
-		$aggregation = & $this->getRefTable($refPath)->aggregation[$refPath . $function . $this->getSql() . json_encode($this->sqlBuilder->getParameters())];
+		$aggregation = & $this->getRefTable($refPath)->aggregation[$refPath . $function . $this->sqlBuilder->getSelectQueryHash($this->getPreviousAccessedColumns())];
 
 		if ($aggregation === NULL) {
-			$aggregation = array();
+			$aggregation = [];
 
 			$selection = $this->createSelectionInstance();
 			$selection->getSqlBuilder()->importConditions($this->getSqlBuilder());
@@ -150,8 +142,8 @@ class GroupedSelection extends Selection
 			}
 			parent::execute();
 			$this->sqlBuilder->setLimit($limit, NULL);
-			$data = array();
-			$offset = array();
+			$data = [];
+			$offset = [];
 			$this->accessColumn($this->column);
 			foreach ((array) $this->rows as $key => $row) {
 				$ref = & $data[$row[$this->column]];
@@ -171,7 +163,7 @@ class GroupedSelection extends Selection
 
 		$this->observeCache = $this;
 		if ($this->data === NULL) {
-			$this->data = array();
+			$this->data = [];
 		} else {
 			foreach ($this->data as $row) {
 				$row->setTable($this); // injects correct parent GroupedSelection
@@ -185,7 +177,7 @@ class GroupedSelection extends Selection
 	{
 		$refObj = $this->refTable;
 		$refPath = $this->name . '.';
-		while ($refObj instanceof GroupedSelection) {
+		while ($refObj instanceof self) {
 			$refPath .= $refObj->name . '.';
 			$refObj = $refObj->refTable;
 		}
@@ -198,11 +190,11 @@ class GroupedSelection extends Selection
 	{
 		$hash = $this->getSpecificCacheKey();
 		$referencing = & $this->refCache['referencing'][$this->getGeneralCacheKey()];
-		$this->observeCache      = & $referencing['observeCache'];
-		$this->refCacheCurrent   = & $referencing[$hash];
-		$this->accessedColumns   = & $referencing[$hash]['accessed'];
-		$this->specificCacheKey  = & $referencing[$hash]['specificCacheKey'];
-		$this->rows              = & $referencing[$hash]['rows'];
+		$this->observeCache = & $referencing['observeCache'];
+		$this->refCacheCurrent = & $referencing[$hash];
+		$this->accessedColumns = & $referencing[$hash]['accessed'];
+		$this->specificCacheKey = & $referencing[$hash]['specificCacheKey'];
+		$this->rows = & $referencing[$hash]['rows'];
 
 		if (isset($referencing[$hash]['data'][$this->active])) {
 			$this->data = & $referencing[$hash]['data'][$this->active];
@@ -210,6 +202,12 @@ class GroupedSelection extends Selection
 	}
 
 
+	protected function emptyResultSet($saveCache = TRUE, $deleteRererencedCache = TRUE)
+	{
+		parent::emptyResultSet($saveCache, FALSE);
+	}
+
+
 	/********************* manipulation ****************d*g**/
 
 
