@@ -1,32 +1,30 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database\Table;
 
-use Nette,
-	Nette\Database\ISupplementalDriver;
+use Nette;
+use Nette\Database\Context;
+use Nette\Database\IConventions;
 
 
 /**
  * Filtered table representation.
  * Selection is based on the great library NotORM http://www.notorm.com written by Jakub Vrana.
- *
- * @author     Jakub Vrana
- * @author     Jan Skrasek
- *
- * @property-read string $sql
  */
-class Selection extends Nette\Object implements \Iterator, IRowContainer, \ArrayAccess, \Countable
+class Selection implements \Iterator, IRowContainer, \ArrayAccess, \Countable
 {
-	/** @var Nette\Database\Connection */
-	protected $connection;
+	use Nette\SmartObject;
 
-	/** @var Nette\Database\IReflection */
-	protected $reflection;
+	/** @var Context */
+	protected $context;
+
+	/** @var IConventions */
+	protected $conventions;
 
 	/** @var Nette\Caching\Cache */
 	protected $cache;
@@ -37,7 +35,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	/** @var string table name */
 	protected $name;
 
-	/** @var string primary key field name */
+	/** @var string|array|NULL primary key field name */
 	protected $primary;
 
 	/** @var string|bool primary column sequence name, FALSE for autodetection */
@@ -65,7 +63,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	protected $specificCacheKey;
 
 	/** @var array of [conditions => [key => IRow]]; used by GroupedSelection */
-	protected $aggregation = array();
+	protected $aggregation = [];
 
 	/** @var array of touched columns */
 	protected $accessedColumns;
@@ -77,22 +75,25 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	protected $observeCache = FALSE;
 
 	/** @var array of primary key values */
-	protected $keys = array();
+	protected $keys = [];
 
 
 	/**
 	 * Creates filtered table representation.
-	 * @param  Nette\Database\Connection
-	 * @param  string  database table name
+	 * @param  Context
+	 * @param  IConventions
+	 * @param  string  table name
+	 * @param  Nette\Caching\IStorage|NULL
 	 */
-	public function __construct(Nette\Database\Connection $connection, $table, Nette\Database\IReflection $reflection, Nette\Caching\IStorage $cacheStorage = NULL)
+	public function __construct(Context $context, IConventions $conventions, $tableName, Nette\Caching\IStorage $cacheStorage = NULL)
 	{
-		$this->name = $table;
-		$this->connection = $connection;
-		$this->reflection = $reflection;
-		$this->cache = $cacheStorage ? new Nette\Caching\Cache($cacheStorage, 'Nette.Database.' . md5($connection->getDsn())) : NULL;
-		$this->primary = $reflection->getPrimary($table);
-		$this->sqlBuilder = new SqlBuilder($table, $connection, $reflection);
+		$this->context = $context;
+		$this->conventions = $conventions;
+		$this->name = $tableName;
+
+		$this->cache = $cacheStorage ? new Nette\Caching\Cache($cacheStorage, 'Nette.Database.' . md5($context->getConnection()->getDsn())) : NULL;
+		$this->primary = $conventions->getPrimary($tableName);
+		$this->sqlBuilder = new SqlBuilder($tableName, $context);
 		$this->refCache = & $this->getRefTable($refPath)->globalRefCache[$refPath];
 	}
 
@@ -109,24 +110,6 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	}
 
 
-	/**
-	 * @return Nette\Database\Connection
-	 */
-	public function getConnection()
-	{
-		return $this->connection;
-	}
-
-
-	/**
-	 * @return Nette\Database\IReflection
-	 */
-	public function getDatabaseReflection()
-	{
-		return $this->reflection;
-	}
-
-
 	/**
 	 * @return string
 	 */
@@ -138,7 +121,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 
 	/**
 	 * @param  bool
-	 * @return string|array
+	 * @return string|array|NULL
 	 */
 	public function getPrimary($need = TRUE)
 	{
@@ -155,16 +138,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	public function getPrimarySequence()
 	{
 		if ($this->primarySequence === FALSE) {
-			$this->primarySequence = NULL;
-			$driver = $this->connection->getSupplementalDriver();
-			if ($driver->isSupported(ISupplementalDriver::SUPPORT_SEQUENCE) && $this->primary !== NULL) {
-				foreach ($driver->getColumns($this->name) as $column) {
-					if ($column['name'] === $this->primary) {
-						$this->primarySequence = $column['vendor']['sequence'];
-						break;
-					}
-				}
-			}
+			$this->primarySequence = $this->context->getStructure()->getPrimaryKeySequence($this->name);
 		}
 
 		return $this->primarySequence;
@@ -201,7 +175,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 		if ($this->cache && $this->previousAccessedColumns === NULL) {
 			$this->accessedColumns = $this->previousAccessedColumns = $this->cache->load($this->getGeneralCacheKey());
 			if ($this->previousAccessedColumns === NULL) {
-				$this->previousAccessedColumns = array();
+				$this->previousAccessedColumns = [];
 			}
 		}
 
@@ -246,12 +220,32 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	}
 
 
+	/**
+	 * Fetches single field.
+	 * @param  string|NULL
+	 * @return mixed|FALSE
+	 */
+	public function fetchField($column = NULL)
+	{
+		if ($column) {
+			$this->select($column);
+		}
+
+		$row = $this->fetch();
+		if ($row) {
+			return $column ? $row[$column] : array_values($row->toArray())[0];
+		}
+
+		return FALSE;
+	}
+
+
 	/**
 	 * @inheritDoc
 	 */
 	public function fetchPairs($key = NULL, $value = NULL)
 	{
-		return Nette\Database\Helpers::toPairs(iterator_to_array($this), $key, $value);
+		return Nette\Database\Helpers::toPairs($this->fetchAll(), $key, $value);
 	}
 
 
@@ -264,6 +258,16 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	}
 
 
+	/**
+	 * @inheritDoc
+	 */
+	public function fetchAssoc($path)
+	{
+		$rows = array_map('iterator_to_array', $this->fetchAll());
+		return Nette\Utils\Arrays::associate($rows, $path);
+	}
+
+
 	/********************* sql selectors ****************d*g**/
 
 
@@ -272,24 +276,14 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 * @param  string for example "column, MD5(column) AS column_md5"
 	 * @return self
 	 */
-	public function select($columns)
+	public function select($columns, ...$params)
 	{
 		$this->emptyResultSet();
-		call_user_func_array(array($this->sqlBuilder, 'addSelect'), func_get_args());
+		$this->sqlBuilder->addSelect($columns, ...$params);
 		return $this;
 	}
 
 
-	/**
-	 * @deprecated
-	 */
-	public function find($key)
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use $selection->wherePrimary() instead.', E_USER_DEPRECATED);
-		return $this->wherePrimary($key);
-	}
-
-
 	/**
 	 * Adds condition for primary key.
 	 * @param  mixed
@@ -319,25 +313,84 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 * Adds where condition, more calls appends with AND.
 	 * @param  string condition possibly containing ?
 	 * @param  mixed
-	 * @param  mixed ...
 	 * @return self
 	 */
-	public function where($condition, $parameters = array())
+	public function where($condition, ...$params)
 	{
-		if (is_array($condition) && $parameters === array()) { // where(array('column1' => 1, 'column2 > ?' => 2))
+		$this->condition($condition, $params);
+		return $this;
+	}
+
+
+	/**
+	 * Adds ON condition when joining specified table, more calls appends with AND.
+	 * @param  string table chain or table alias for which you need additional left join condition
+	 * @param  string condition possibly containing ?
+	 * @param  mixed
+	 * @return self
+	 */
+	public function joinWhere($tableChain, $condition, ...$params)
+	{
+		$this->condition($condition, $params, $tableChain);
+		return $this;
+	}
+
+
+	/**
+	 * Adds condition, more calls appends with AND.
+	 * @param  string condition possibly containing ?
+	 * @return void
+	 */
+	protected function condition($condition, array $params, $tableChain = NULL)
+	{
+		$this->emptyResultSet();
+		if (is_array($condition) && $params === []) { // where(array('column1' => 1, 'column2 > ?' => 2))
 			foreach ($condition as $key => $val) {
 				if (is_int($key)) {
-					$this->where($val); // where('full condition')
+					$this->condition($val, [], $tableChain); // where('full condition')
 				} else {
-					$this->where($key, $val); // where('column', 1)
+					$this->condition($key, [$val], $tableChain); // where('column', 1)
 				}
 			}
-			return $this;
+		} elseif ($tableChain) {
+			$this->sqlBuilder->addJoinCondition($tableChain, $condition, ...$params);
+		} else {
+			$this->sqlBuilder->addWhere($condition, ...$params);
 		}
+	}
 
-		$this->emptyResultSet();
-		call_user_func_array(array($this->sqlBuilder, 'addWhere'), func_get_args());
-		return $this;
+
+	/**
+	 * Adds where condition using the OR operator between parameters.
+	 * More calls appends with AND.
+	 * @param  array ['column1' => 1, 'column2 > ?' => 2, 'full condition']
+	 * @return self
+	 * @throws \Nette\InvalidArgumentException
+	 */
+	public function whereOr(array $parameters)
+	{
+		if (count($parameters) < 2) {
+			return $this->where($parameters);
+		}
+		$columns = [];
+		$values = [];
+		foreach ($parameters as $key => $val) {
+			if (is_int($key)) { // whereOr(['full condition'])
+				$columns[] = $val;
+			} elseif (strpos($key, '?') === FALSE) { // whereOr(['column1' => 1])
+				$columns[] = $key . ' ?';
+				$values[] = $val;
+			} else { // whereOr(['column1 > ?' => 1])
+				$qNumber = substr_count($key, '?');
+				if ($qNumber > 1 && (!is_array($val) || $qNumber !== count($val))) {
+					throw new Nette\InvalidArgumentException('Argument count does not match placeholder count.');
+				}
+				$columns[] = $key;
+				$values = array_merge($values, $qNumber > 1 ? $val : [$val]);
+			}
+		}
+		$columnsString = '(' . implode(') OR (', $columns) . ')';
+		return $this->where($columnsString, $values);
 	}
 
 
@@ -346,10 +399,10 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 * @param  string for example 'column1, column2 DESC'
 	 * @return self
 	 */
-	public function order($columns)
+	public function order($columns, ...$params)
 	{
 		$this->emptyResultSet();
-		call_user_func_array(array($this->sqlBuilder, 'addOrder'), func_get_args());
+		$this->sqlBuilder->addOrder($columns, ...$params);
 		return $this;
 	}
 
@@ -379,6 +432,9 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 		if (func_num_args() > 2) {
 			$numOfPages = (int) ceil($this->count('*') / $itemsPerPage);
 		}
+		if ($page < 1) {
+			$itemsPerPage = 0;
+		}
 		return $this->limit($itemsPerPage, ($page - 1) * $itemsPerPage);
 	}
 
@@ -388,16 +444,10 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 * @param  string
 	 * @return self
 	 */
-	public function group($columns)
+	public function group($columns, ...$params)
 	{
 		$this->emptyResultSet();
-		if (func_num_args() === 2 && strpos($columns, '?') === FALSE) {
-			trigger_error('Calling ' . __METHOD__ . '() with second argument is deprecated; use $selection->having() instead.', E_USER_DEPRECATED);
-			$this->having(func_get_arg(1));
-			$this->sqlBuilder->setGroup($columns);
-		} else {
-			call_user_func_array(array($this->sqlBuilder, 'setGroup'), func_get_args());
-		}
+		$this->sqlBuilder->setGroup($columns, ...$params);
 		return $this;
 	}
 
@@ -407,10 +457,23 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 * @param  string
 	 * @return self
 	 */
-	public function having($having)
+	public function having($having, ...$params)
 	{
 		$this->emptyResultSet();
-		call_user_func_array(array($this->sqlBuilder, 'setHaving'), func_get_args());
+		$this->sqlBuilder->setHaving($having, ...$params);
+		return $this;
+	}
+
+
+	/**
+	 * Aliases table. Example ':book:book_tag.tag', 'tg'
+	 * @param  string
+	 * @param  string
+	 * @return self
+	 */
+	public function alias($tableChain, $alias)
+	{
+		$this->sqlBuilder->addAlias($tableChain, $alias);
 		return $this;
 	}
 
@@ -500,23 +563,23 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 		try {
 			$result = $this->query($this->getSql());
 
-		} catch (\PDOException $exception) {
+		} catch (Nette\Database\DriverException $exception) {
 			if (!$this->sqlBuilder->getSelect() && $this->previousAccessedColumns) {
 				$this->previousAccessedColumns = FALSE;
-				$this->accessedColumns = array();
+				$this->accessedColumns = [];
 				$result = $this->query($this->getSql());
 			} else {
 				throw $exception;
 			}
 		}
 
-		$this->rows = array();
+		$this->rows = [];
 		$usedPrimary = TRUE;
 		foreach ($result->getPdoStatement() as $key => $row) {
 			$row = $this->createRow($result->normalizeRow($row));
 			$primary = $row->getSignature(FALSE);
-			$usedPrimary = $usedPrimary && $primary;
-			$this->rows[$primary ?: $key] = $row;
+			$usedPrimary = $usedPrimary && (string) $primary !== '';
+			$this->rows[$usedPrimary ? $primary : $key] = $row;
 		}
 		$this->data = $this->rows;
 
@@ -536,32 +599,40 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 
 	public function createSelectionInstance($table = NULL)
 	{
-		return new Selection($this->connection, $table ?: $this->name, $this->reflection, $this->cache ? $this->cache->getStorage() : NULL);
+		return new self($this->context, $this->conventions, $table ?: $this->name, $this->cache ? $this->cache->getStorage() : NULL);
 	}
 
 
 	protected function createGroupedSelectionInstance($table, $column)
 	{
-		return new GroupedSelection($this, $table, $column);
+		return new GroupedSelection($this->context, $this->conventions, $table, $column, $this, $this->cache ? $this->cache->getStorage() : NULL);
 	}
 
 
 	protected function query($query)
 	{
-		return $this->connection->queryArgs($query, $this->sqlBuilder->getParameters());
+		return $this->context->queryArgs($query, $this->sqlBuilder->getParameters());
 	}
 
 
-	protected function emptyResultSet($saveCache = TRUE)
+	protected function emptyResultSet($clearCache = TRUE, $deleteRererencedCache = TRUE)
 	{
-		if ($this->rows !== NULL && $saveCache) {
+		if ($this->rows !== NULL && $clearCache) {
 			$this->saveCacheState();
 		}
 
+		if ($clearCache) {
+			// not null in case of missing some column
+			$this->previousAccessedColumns = NULL;
+			$this->generalCacheKey = NULL;
+		}
+
 		$this->rows = NULL;
 		$this->specificCacheKey = NULL;
-		$this->generalCacheKey = NULL;
-		$this->refCache['referencingPrototype'] = array();
+		$this->refCache['referencingPrototype'] = [];
+		if ($deleteRererencedCache) {
+			$this->refCache['referenced'] = [];
+		}
 	}
 
 
@@ -585,7 +656,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 
 	/**
 	 * Returns Selection parent for caching.
-	 * @return Selection
+	 * @return self
 	 */
 	protected function getRefTable(& $refPath)
 	{
@@ -612,7 +683,14 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 			return $this->generalCacheKey;
 		}
 
-		return $this->generalCacheKey = md5(serialize(array(__CLASS__, $this->name, $this->sqlBuilder->getConditions())));
+		$key = [__CLASS__, $this->name, $this->sqlBuilder->getConditions()];
+		$trace = [];
+		foreach (debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS) as $item) {
+			$trace[] = isset($item['file'], $item['line']) ? $item['file'] . $item['line'] : NULL;
+		};
+
+		$key[] = $trace;
+		return $this->generalCacheKey = md5(serialize($key));
 	}
 
 
@@ -627,7 +705,7 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 			return $this->specificCacheKey;
 		}
 
-		return $this->specificCacheKey = md5($this->getSql() . json_encode($this->sqlBuilder->getParameters()));
+		return $this->specificCacheKey = $this->sqlBuilder->getSelectQueryHash($this->getPreviousAccessedColumns());
 	}
 
 
@@ -635,11 +713,12 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 * @internal
 	 * @param  string|NULL column name or NULL to reload all columns
 	 * @param  bool
+	 * @return bool if selection requeried for more columns.
 	 */
 	public function accessColumn($key, $selectColumn = TRUE)
 	{
 		if (!$this->cache) {
-			return;
+			return FALSE;
 		}
 
 		if ($key === NULL) {
@@ -649,14 +728,12 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 			$this->accessedColumns[$key] = $selectColumn;
 		}
 
-		if ($selectColumn && !$this->sqlBuilder->getSelect() && $this->previousAccessedColumns && ($key === NULL || !isset($this->previousAccessedColumns[$key]))) {
-			$this->previousAccessedColumns = array();
-
+		if ($selectColumn && $this->previousAccessedColumns && ($key === NULL || !isset($this->previousAccessedColumns[$key])) && !$this->sqlBuilder->getSelect()) {
 			if ($this->sqlBuilder->getLimit()) {
 				$generalCacheKey = $this->generalCacheKey;
 				$sqlBuilder = $this->sqlBuilder;
 
-				$primaryValues = array();
+				$primaryValues = [];
 				foreach ((array) $this->rows as $row) {
 					$primary = $row->getPrimary();
 					$primaryValues[] = is_array($primary) ? array_values($primary) : $primary;
@@ -668,10 +745,12 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 				$this->wherePrimary($primaryValues);
 
 				$this->generalCacheKey = $generalCacheKey;
+				$this->previousAccessedColumns = [];
 				$this->execute();
 				$this->sqlBuilder = $sqlBuilder;
 			} else {
 				$this->emptyResultSet(FALSE);
+				$this->previousAccessedColumns = [];
 				$this->execute();
 			}
 
@@ -679,11 +758,12 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 
 			// move iterator to specific key
 			if (isset($currentKey)) {
-				while (key($this->data) !== $currentKey) {
+				while (key($this->data) !== NULL && key($this->data) !== $currentKey) {
 					next($this->data);
 				}
 			}
 		}
+		return $this->dataRefreshed;
 	}
 
 
@@ -719,29 +799,35 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 */
 	public function insert($data)
 	{
-		if ($data instanceof Selection) {
-			$data = new Nette\Database\SqlLiteral($data->getSql(), $data->getSqlBuilder()->getParameters());
+		if ($data instanceof self) {
+			$return = $this->context->queryArgs($this->sqlBuilder->buildInsertQuery() . ' ' . $data->getSql(), $data->getSqlBuilder()->getParameters());
 
-		} elseif ($data instanceof \Traversable) {
-			$data = iterator_to_array($data);
+		} else {
+			if ($data instanceof \Traversable) {
+				$data = iterator_to_array($data);
+			}
+			$return = $this->context->query($this->sqlBuilder->buildInsertQuery() . ' ?values', $data);
 		}
 
-		$return = $this->connection->query($this->sqlBuilder->buildInsertQuery(), $data);
 		$this->loadRefCache();
 
-		if ($data instanceof Nette\Database\SqlLiteral || $this->primary === NULL) {
+		if ($data instanceof self || $this->primary === NULL) {
 			unset($this->refCache['referencing'][$this->getGeneralCacheKey()][$this->getSpecificCacheKey()]);
 			return $return->getRowCount();
 		}
 
-		$primaryKey = $this->connection->getInsertId($this->getPrimarySequence());
+		$primaryKey = $this->context->getInsertId(
+			($tmp = $this->getPrimarySequence())
+				? implode('.', array_map([$this->context->getConnection()->getSupplementalDriver(), 'delimite'], explode('.', $tmp)))
+				: NULL
+		);
 		if ($primaryKey === FALSE) {
 			unset($this->refCache['referencing'][$this->getGeneralCacheKey()][$this->getSpecificCacheKey()]);
 			return $return->getRowCount();
 		}
 
 		if (is_array($this->getPrimary())) {
-			$primaryKey = array();
+			$primaryKey = [];
 
 			foreach ((array) $this->getPrimary() as $key) {
 				if (!isset($data[$key])) {
@@ -793,9 +879,9 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 			return 0;
 		}
 
-		return $this->connection->queryArgs(
+		return $this->context->queryArgs(
 			$this->sqlBuilder->buildUpdateQuery(),
-			array_merge(array($data), $this->sqlBuilder->getParameters())
+			array_merge([$data], $this->sqlBuilder->getParameters())
 		)->getRowCount();
 	}
 
@@ -815,19 +901,32 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 
 	/**
 	 * Returns referenced row.
+	 * @param  ActiveRow
 	 * @param  string
-	 * @param  string
-	 * @param  mixed   primary key to check for $table and $column references
-	 * @return Selection or array() if the row does not exist
+	 * @param  string|NULL
+	 * @return ActiveRow|NULL|FALSE NULL if the row does not exist, FALSE if the relationship does not exist
 	 */
-	public function getReferencedTable($table, $column, $checkPrimaryKey)
+	public function getReferencedTable(ActiveRow $row, $table, $column = NULL)
 	{
+		if (!$column) {
+			$belongsTo = $this->conventions->getBelongsToReference($this->name, $table);
+			if (!$belongsTo) {
+				return FALSE;
+			}
+			list($table, $column) = $belongsTo;
+		}
+		if (!$row->accessColumn($column)) {
+			return FALSE;
+		}
+
+		$checkPrimaryKey = $row[$column];
+
 		$referenced = & $this->refCache['referenced'][$this->getSpecificCacheKey()]["$table.$column"];
 		$selection = & $referenced['selection'];
 		$cacheKeys = & $referenced['cacheKeys'];
-		if ($selection === NULL || !isset($cacheKeys[$checkPrimaryKey])) {
+		if ($selection === NULL || ($checkPrimaryKey !== NULL && !isset($cacheKeys[$checkPrimaryKey]))) {
 			$this->execute();
-			$cacheKeys = array();
+			$cacheKeys = [];
 			foreach ($this->rows as $row) {
 				if ($row[$column] === NULL) {
 					continue;
@@ -841,11 +940,11 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 				$selection = $this->createSelectionInstance($table);
 				$selection->where($selection->getPrimary(), array_keys($cacheKeys));
 			} else {
-				$selection = array();
+				$selection = [];
 			}
 		}
 
-		return $selection;
+		return isset($selection[$checkPrimaryKey]) ? $selection[$checkPrimaryKey] : NULL;
 	}
 
 
@@ -858,7 +957,17 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 	 */
 	public function getReferencingTable($table, $column, $active = NULL)
 	{
-		$prototype = & $this->refCache['referencingPrototype']["$table.$column"];
+		if (strpos($table, '.') !== FALSE) {
+			list($table, $column) = explode('.', $table);
+		} elseif (!$column) {
+			$hasMany = $this->conventions->getHasManyReference($this->name, $table);
+			if (!$hasMany) {
+				return FALSE;
+			}
+			list($table, $column) = $hasMany;
+		}
+
+		$prototype = & $this->refCache['referencingPrototype'][$this->getSpecificCacheKey()]["$table.$column"];
 		if (!$prototype) {
 			$prototype = $this->createGroupedSelectionInstance($table, $column);
 			$prototype->where("$table.$column", array_keys((array) $this->rows));
@@ -903,7 +1012,9 @@ class Selection extends Nette\Object implements \Iterator, IRowContainer, \Array
 
 	public function next()
 	{
-		next($this->keys);
+		do {
+			next($this->keys);
+		} while (($key = current($this->keys)) !== FALSE && !isset($this->data[$key]));
 	}
 
 
