@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms;
@@ -13,18 +13,13 @@ use Nette;
 /**
  * Container for form controls.
  *
- * @author     David Grudl
- *
- * @property-write $defaults
- * @property   Nette\ArrayHash $values
- * @property-read bool $valid
- * @property   ControlGroup $currentGroup
+ * @property   Nette\Utils\ArrayHash $values
  * @property-read \ArrayIterator $controls
  * @property-read Form $form
  */
 class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 {
-	/** @var array of function(Container $sender); Occurs when the form is validated */
+	/** @var callable[]  function (Container $sender); Occurs when the form is validated */
 	public $onValidate;
 
 	/** @var ControlGroup */
@@ -58,6 +53,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  array|\Traversable  values used to fill the form
 	 * @param  bool     erase other controls?
 	 * @return self
+	 * @internal
 	 */
 	public function setValues($values, $erase = FALSE)
 	{
@@ -77,12 +73,12 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 					$control->setValue(NULL);
 				}
 
-			} elseif ($control instanceof Container) {
+			} elseif ($control instanceof self) {
 				if (array_key_exists($name, $values)) {
 					$control->setValues($values[$name], $erase);
 
 				} elseif ($erase) {
-					$control->setValues(array(), $erase);
+					$control->setValues([], $erase);
 				}
 			}
 		}
@@ -93,16 +89,16 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	/**
 	 * Returns the values submitted by the form.
 	 * @param  bool  return values as an array?
-	 * @return Nette\ArrayHash|array
+	 * @return Nette\Utils\ArrayHash|array
 	 */
 	public function getValues($asArray = FALSE)
 	{
-		$values = $asArray ? array() : new Nette\ArrayHash;
+		$values = $asArray ? [] : new Nette\Utils\ArrayHash;
 		foreach ($this->getComponents() as $name => $control) {
 			if ($control instanceof IControl && !$control->isOmitted()) {
 				$values[$name] = $control->getValue();
 
-			} elseif ($control instanceof Container) {
+			} elseif ($control instanceof self) {
 				$values[$name] = $control->getValues($asArray);
 			}
 		}
@@ -136,10 +132,21 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 */
 	public function validate(array $controls = NULL)
 	{
-		foreach ($controls === NULL ? $this->getControls() : $controls as $control) {
-			$control->validate();
+		foreach ($controls === NULL ? $this->getComponents() : $controls as $control) {
+			if ($control instanceof IControl || $control instanceof self) {
+				$control->validate();
+			}
+		}
+		if ($this->onValidate !== NULL) {
+			if (!is_array($this->onValidate) && !$this->onValidate instanceof \Traversable) {
+				throw new Nette\UnexpectedValueException('Property Form::$onValidate must be array or Traversable, ' . gettype($this->onValidate) . ' given.');
+			}
+			foreach ($this->onValidate as $handler) {
+				$params = Nette\Utils\Callback::toReflection($handler)->getParameters();
+				$values = isset($params[1]) ? $this->getValues($params[1]->isArray()) : NULL;
+				Nette\Utils\Callback::invoke($handler, $this, $values);
+			}
 		}
-		$this->onValidate($this);
 		$this->validated = TRUE;
 	}
 
@@ -150,7 +157,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 */
 	public function getErrors()
 	{
-		$errors = array();
+		$errors = [];
 		foreach ($this->getControls() as $control) {
 			$errors = array_merge($errors, $control->getErrors());
 		}
@@ -183,7 +190,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 
 	/**
 	 * Adds the specified component to the IContainer.
-	 * @param  IComponent
+	 * @param  Nette\ComponentModel\IComponent
 	 * @param  string
 	 * @param  string
 	 * @return self
@@ -192,7 +199,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	public function addComponent(Nette\ComponentModel\IComponent $component, $name, $insertBefore = NULL)
 	{
 		parent::addComponent($component, $name, $insertBefore);
-		if ($this->currentGroup !== NULL && $component instanceof IControl) {
+		if ($this->currentGroup !== NULL) {
 			$this->currentGroup->add($component);
 		}
 		return $this;
@@ -205,7 +212,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 */
 	public function getControls()
 	{
-		return $this->getComponents(TRUE, 'Nette\Forms\IControl');
+		return $this->getComponents(TRUE, IControl::class);
 	}
 
 
@@ -216,7 +223,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 */
 	public function getForm($need = TRUE)
 	{
-		return $this->lookup('Nette\Forms\Form', $need);
+		return $this->lookup(Form::class, $need);
 	}
 
 
@@ -229,13 +236,12 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  label
 	 * @param  int  width of the control (deprecated)
 	 * @param  int  maximum number of characters the user may enter
-	 * @return Nette\Forms\Controls\TextInput
+	 * @return Controls\TextInput
 	 */
 	public function addText($name, $label = NULL, $cols = NULL, $maxLength = NULL)
 	{
-		$control = new Controls\TextInput($label, $maxLength);
-		$control->setAttribute('size', $cols);
-		return $this[$name] = $control;
+		return $this[$name] = (new Controls\TextInput($label, $maxLength))
+			->setAttribute('size', $cols);
 	}
 
 
@@ -245,13 +251,13 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  label
 	 * @param  int  width of the control (deprecated)
 	 * @param  int  maximum number of characters the user may enter
-	 * @return Nette\Forms\Controls\TextInput
+	 * @return Controls\TextInput
 	 */
 	public function addPassword($name, $label = NULL, $cols = NULL, $maxLength = NULL)
 	{
-		$control = new Controls\TextInput($label, $maxLength);
-		$control->setAttribute('size', $cols);
-		return $this[$name] = $control->setType('password');
+		return $this[$name] = (new Controls\TextInput($label, $maxLength))
+			->setAttribute('size', $cols)
+			->setType('password');
 	}
 
 
@@ -261,13 +267,41 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  label
 	 * @param  int  width of the control
 	 * @param  int  height of the control in text lines
-	 * @return Nette\Forms\Controls\TextArea
+	 * @return Controls\TextArea
 	 */
 	public function addTextArea($name, $label = NULL, $cols = NULL, $rows = NULL)
 	{
-		$control = new Controls\TextArea($label);
-		$control->setAttribute('cols', $cols)->setAttribute('rows', $rows);
-		return $this[$name] = $control;
+		return $this[$name] = (new Controls\TextArea($label))
+			->setAttribute('cols', $cols)->setAttribute('rows', $rows);
+	}
+
+
+	/**
+	 * Adds input for email.
+	 * @param  string  control name
+	 * @param  string  label
+	 * @return Controls\TextInput
+	 */
+	public function addEmail($name, $label = NULL)
+	{
+		return $this[$name] = (new Controls\TextInput($label))
+			->setRequired(FALSE)
+			->addRule(Form::EMAIL);
+	}
+
+
+	/**
+	 * Adds input for integer.
+	 * @param  string  control name
+	 * @param  string  label
+	 * @return Controls\TextInput
+	 */
+	public function addInteger($name, $label = NULL)
+	{
+		return $this[$name] = (new Controls\TextInput($label))
+			->setNullable()
+			->setRequired(FALSE)
+			->addRule(Form::INTEGER);
 	}
 
 
@@ -276,7 +310,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  control name
 	 * @param  string  label
 	 * @param  bool  allows to upload multiple files
-	 * @return Nette\Forms\Controls\UploadControl
+	 * @return Controls\UploadControl
 	 */
 	public function addUpload($name, $label = NULL, $multiple = FALSE)
 	{
@@ -284,17 +318,28 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	}
 
 
+	/**
+	 * Adds control that allows the user to upload multiple files.
+	 * @param  string  control name
+	 * @param  string  label
+	 * @return Controls\UploadControl
+	 */
+	public function addMultiUpload($name, $label = NULL)
+	{
+		return $this[$name] = new Controls\UploadControl($label, TRUE);
+	}
+
+
 	/**
 	 * Adds hidden form control used to store a non-displayed value.
 	 * @param  string  control name
 	 * @param  mixed   default value
-	 * @return Nette\Forms\Controls\HiddenField
+	 * @return Controls\HiddenField
 	 */
 	public function addHidden($name, $default = NULL)
 	{
-		$control = new Controls\HiddenField;
-		$control->setDefaultValue($default);
-		return $this[$name] = $control;
+		return $this[$name] = (new Controls\HiddenField)
+			->setDefaultValue($default);
 	}
 
 
@@ -302,7 +347,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * Adds check box control to the form.
 	 * @param  string  control name
 	 * @param  string  caption
-	 * @return Nette\Forms\Controls\Checkbox
+	 * @return Controls\Checkbox
 	 */
 	public function addCheckbox($name, $caption = NULL)
 	{
@@ -315,7 +360,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  control name
 	 * @param  string  label
 	 * @param  array   options from which to choose
-	 * @return Nette\Forms\Controls\RadioList
+	 * @return Controls\RadioList
 	 */
 	public function addRadioList($name, $label = NULL, array $items = NULL)
 	{
@@ -325,7 +370,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 
 	/**
 	 * Adds set of checkbox controls to the form.
-	 * @return Nette\Forms\Controls\CheckboxList
+	 * @return Controls\CheckboxList
 	 */
 	public function addCheckboxList($name, $label = NULL, array $items = NULL)
 	{
@@ -339,15 +384,12 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  label
 	 * @param  array   items from which to choose
 	 * @param  int     number of rows that should be visible
-	 * @return Nette\Forms\Controls\SelectBox
+	 * @return Controls\SelectBox
 	 */
 	public function addSelect($name, $label = NULL, array $items = NULL, $size = NULL)
 	{
-		$control = new Controls\SelectBox($label, $items);
-		if ($size > 1) {
-			$control->setAttribute('size', (int) $size);
-		}
-		return $this[$name] = $control;
+		return $this[$name] = (new Controls\SelectBox($label, $items))
+			->setAttribute('size', $size > 1 ? (int) $size : NULL);
 	}
 
 
@@ -357,15 +399,12 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  label
 	 * @param  array   options from which to choose
 	 * @param  int     number of rows that should be visible
-	 * @return Nette\Forms\Controls\MultiSelectBox
+	 * @return Controls\MultiSelectBox
 	 */
 	public function addMultiSelect($name, $label = NULL, array $items = NULL, $size = NULL)
 	{
-		$control = new Controls\MultiSelectBox($label, $items);
-		if ($size > 1) {
-			$control->setAttribute('size', (int) $size);
-		}
-		return $this[$name] = $control;
+		return $this[$name] = (new Controls\MultiSelectBox($label, $items))
+			->setAttribute('size', $size > 1 ? (int) $size : NULL);
 	}
 
 
@@ -373,7 +412,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * Adds button used to submit form.
 	 * @param  string  control name
 	 * @param  string  caption
-	 * @return Nette\Forms\Controls\SubmitButton
+	 * @return Controls\SubmitButton
 	 */
 	public function addSubmit($name, $caption = NULL)
 	{
@@ -385,7 +424,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * Adds push buttons with no default behavior.
 	 * @param  string  control name
 	 * @param  string  caption
-	 * @return Nette\Forms\Controls\Button
+	 * @return Controls\Button
 	 */
 	public function addButton($name, $caption = NULL)
 	{
@@ -398,7 +437,7 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 * @param  string  control name
 	 * @param  string  URI of the image
 	 * @param  string  alternate text for the image
-	 * @return Nette\Forms\Controls\ImageButton
+	 * @return Controls\ImageButton
 	 */
 	public function addImage($name, $src = NULL, $alt = NULL)
 	{
@@ -413,12 +452,36 @@ class Container extends Nette\ComponentModel\Container implements \ArrayAccess
 	 */
 	public function addContainer($name)
 	{
-		$control = new Container;
+		$control = new self;
 		$control->currentGroup = $this->currentGroup;
+		if ($this->currentGroup !== NULL) {
+			$this->currentGroup->add($control);
+		}
 		return $this[$name] = $control;
 	}
 
 
+	/********************* extension methods ****************d*g**/
+
+
+	public function __call($name, $args)
+	{
+		if ($callback = Nette\Utils\ObjectMixin::getExtensionMethod(__CLASS__, $name)) {
+			return Nette\Utils\Callback::invoke($callback, $this, ...$args);
+		}
+		return parent::__call($name, $args);
+	}
+
+
+	public static function extensionMethod($name, $callback = NULL)
+	{
+		if (strpos($name, '::') !== FALSE) { // back compatibility
+			list(, $name) = explode('::', $name);
+		}
+		Nette\Utils\ObjectMixin::setExtensionMethod(__CLASS__, $name, $callback);
+	}
+
+
 	/********************* interface \ArrayAccess ****************d*g**/
 
 
