@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms;
@@ -12,19 +12,16 @@ use Nette;
 
 /**
  * A user group of form controls.
- *
- * @author     David Grudl
- *
- * @property-read array $controls
- * @property-read array $options
  */
-class ControlGroup extends Nette\Object
+class ControlGroup
 {
+	use Nette\SmartObject;
+
 	/** @var \SplObjectStorage */
 	protected $controls;
 
 	/** @var array user options */
-	private $options = array();
+	private $options = [];
 
 
 	public function __construct()
@@ -36,19 +33,22 @@ class ControlGroup extends Nette\Object
 	/**
 	 * @return self
 	 */
-	public function add()
+	public function add(...$items)
 	{
-		foreach (func_get_args() as $num => $item) {
+		foreach ($items as $item) {
 			if ($item instanceof IControl) {
 				$this->controls->attach($item);
 
-			} elseif ($item instanceof \Traversable || is_array($item)) {
-				foreach ($item as $control) {
-					$this->controls->attach($control);
+			} elseif ($item instanceof Container) {
+				foreach ($item->getComponents() as $component) {
+					$this->add($component);
 				}
+			} elseif ($item instanceof \Traversable || is_array($item)) {
+				$this->add(...$item);
 
 			} else {
-				throw new Nette\InvalidArgumentException("Only IFormControl items are allowed, the #$num parameter is invalid.");
+				$type = is_object($item) ? get_class($item) : gettype($item);
+				throw new Nette\InvalidArgumentException("IControl or Container items expected, $type given.");
 			}
 		}
 		return $this;
@@ -56,7 +56,7 @@ class ControlGroup extends Nette\Object
 
 
 	/**
-	 * @return array IFormControl
+	 * @return IControl[]
 	 */
 	public function getControls()
 	{
