@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms\Controls;
@@ -12,8 +12,6 @@ use Nette;
 
 /**
  * Check box control. Allows the user to select a true or false condition.
- *
- * @author     David Grudl
  */
 class Checkbox extends BaseControl
 {
@@ -29,6 +27,7 @@ class Checkbox extends BaseControl
 		parent::__construct($label);
 		$this->control->type = 'checkbox';
 		$this->wrapper = Nette\Utils\Html::el();
+		$this->setOption('type', 'checkbox');
 	}
 
 
@@ -36,6 +35,7 @@ class Checkbox extends BaseControl
 	 * Sets control's value.
 	 * @param  bool
 	 * @return self
+	 * @internal
 	 */
 	public function setValue($value)
 	{
