@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms\Controls;
 
-use Nette;
-
 
 /**
  * Submittable image button form control.
- *
- * @author     David Grudl
  */
 class ImageButton extends SubmitButton
 {
@@ -39,7 +35,7 @@ class ImageButton extends SubmitButton
 	{
 		parent::loadHttpData();
 		$this->value = $this->value
-			? array((int) array_shift($this->value), (int) array_shift($this->value))
+			? [(int) array_shift($this->value), (int) array_shift($this->value)]
 			: NULL;
 	}
 
