@@ -1,23 +1,24 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms\Controls;
 
-use Nette,
-	Nette\Http\FileUpload;
+use Nette;
+use Nette\Forms;
+use Nette\Http\FileUpload;
 
 
 /**
  * Text box and browse button that allow users to select a file to upload to the server.
- *
- * @author     David Grudl
  */
 class UploadControl extends BaseControl
 {
+	/** validation rule */
+	const VALID = ':uploadControlValid';
 
 	/**
 	 * @param  string  label
@@ -28,6 +29,9 @@ class UploadControl extends BaseControl
 		parent::__construct($label);
 		$this->control->type = 'file';
 		$this->control->multiple = (bool) $multiple;
+		$this->setOption('type', 'file');
+		$this->addCondition(Forms\Form::FILLED)
+			->addRule([$this, 'isOk'], Forms\Validator::$messages[self::VALID]);
 	}
 
 
@@ -40,7 +44,7 @@ class UploadControl extends BaseControl
 	protected function attached($form)
 	{
 		if ($form instanceof Nette\Forms\Form) {
-			if ($form->getMethod() !== Nette\Forms\Form::POST) {
+			if (!$form->isMethod('post')) {
 				throw new Nette\InvalidStateException('File upload requires method POST.');
 			}
 			$form->getElementPrototype()->enctype = 'multipart/form-data';
@@ -74,6 +78,7 @@ class UploadControl extends BaseControl
 
 	/**
 	 * @return self
+	 * @internal
 	 */
 	public function setValue($value)
 	{
@@ -87,69 +92,23 @@ class UploadControl extends BaseControl
 	 */
 	public function isFilled()
 	{
-		return $this->value instanceof FileUpload ? $this->value->isOk() : (bool) $this->value; // ignore NULL object
+		return $this->value instanceof FileUpload
+			? $this->value->getError() !== UPLOAD_ERR_NO_FILE // ignore NULL object
+			: (bool) $this->value;
 	}
 
 
-	/********************* validators ****************d*g**/
-
-
 	/**
-	 * Is file size in limit?
+	 * Have been all files succesfully uploaded?
 	 * @return bool
-	 * @internal
 	 */
-	public static function validateFileSize(UploadControl $control, $limit)
+	public function isOk()
 	{
-		foreach (static::toArray($control->getValue()) as $file) {
-			if ($file->getSize() > $limit || $file->getError() === UPLOAD_ERR_INI_SIZE) {
-				return FALSE;
-			}
-		}
-		return TRUE;
-	}
-
-
-	/**
-	 * Has file specified mime type?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateMimeType(UploadControl $control, $mimeType)
-	{
-		$mimeTypes = is_array($mimeType) ? $mimeType : explode(',', $mimeType);
-		foreach (static::toArray($control->getValue()) as $file) {
-			$type = strtolower($file->getContentType());
-			if (!in_array($type, $mimeTypes, TRUE) && !in_array(preg_replace('#/.*#', '/*', $type), $mimeTypes, TRUE)) {
-				return FALSE;
-			}
-		}
-		return TRUE;
-	}
-
-
-	/**
-	 * Is file image?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateImage(UploadControl $control)
-	{
-		foreach (static::toArray($control->getValue()) as $file) {
-			if (!$file->isImage()) {
-				return FALSE;
-			}
-		}
-		return TRUE;
-	}
-
-
-	/**
-	 * @return array
-	 */
-	public static function toArray($value)
-	{
-		return $value instanceof FileUpload ? array($value) : (array) $value;
+		return $this->value instanceof FileUpload
+			? $this->value->isOk()
+			: $this->value && array_reduce($this->value, function ($carry, $fileUpload) {
+				return $carry && $fileUpload->isOk();
+			}, TRUE);
 	}
 
 }
