@@ -15,12 +15,14 @@ use Nette\Utils\Html;
 /**
  * Forms helpers.
  */
-class Helpers extends Nette\Object
+class Helpers
 {
-	private static $unsafeNames = array(
+	use Nette\StaticClass;
+
+	private static $unsafeNames = [
 		'attributes', 'children', 'elements', 'focus', 'length', 'reset', 'style', 'submit', 'onsubmit', 'form',
 		'presenter', 'action',
-	);
+	];
 
 
 	/**
@@ -33,13 +35,13 @@ class Helpers extends Nette\Object
 	 */
 	public static function extractHttpData(array $data, $htmlName, $type)
 	{
-		$name = explode('[', str_replace(array('[]', ']', '.'), array('', '', '_'), $htmlName));
+		$name = explode('[', str_replace(['[]', ']', '.'], ['', '', '_'], $htmlName));
 		$data = Nette\Utils\Arrays::get($data, $name, NULL);
 		$itype = $type & ~Form::DATA_KEYS;
 
 		if (substr($htmlName, -2) === '[]') {
 			if (!is_array($data)) {
-				return array();
+				return [];
 			}
 			foreach ($data as $k => $v) {
 				$data[$k] = $v = static::sanitize($itype, $v);
@@ -96,7 +98,7 @@ class Helpers extends Nette\Object
 	 */
 	public static function exportRules(Rules $rules)
 	{
-		$payload = array();
+		$payload = [];
 		foreach ($rules as $rule) {
 			if (!is_string($op = $rule->validator)) {
 				if (!Nette\Utils\Callback::isStatic($op)) {
@@ -105,29 +107,34 @@ class Helpers extends Nette\Object
 				$op = Nette\Utils\Callback::toString($op);
 			}
 			if ($rule->branch) {
-				$item = array(
+				$item = [
 					'op' => ($rule->isNegative ? '~' : '') . $op,
 					'rules' => static::exportRules($rule->branch),
 					'control' => $rule->control->getHtmlName(),
-				);
+				];
 				if ($rule->branch->getToggles()) {
 					$item['toggle'] = $rule->branch->getToggles();
+				} elseif (!$item['rules']) {
+					continue;
 				}
 			} else {
-				$item = array('op' => ($rule->isNegative ? '~' : '') . $op, 'msg' => Validator::formatMessage($rule, FALSE));
+				$item = ['op' => ($rule->isNegative ? '~' : '') . $op, 'msg' => Validator::formatMessage($rule, FALSE)];
 			}
 
 			if (is_array($rule->arg)) {
-				$item['arg'] = array();
+				$item['arg'] = [];
 				foreach ($rule->arg as $key => $value) {
-					$item['arg'][$key] = $value instanceof IControl ? array('control' => $value->getHtmlName()) : $value;
+					$item['arg'][$key] = $value instanceof IControl ? ['control' => $value->getHtmlName()] : $value;
 				}
 			} elseif ($rule->arg !== NULL) {
-				$item['arg'] = $rule->arg instanceof IControl ? array('control' => $rule->arg->getHtmlName()) : $rule->arg;
+				$item['arg'] = $rule->arg instanceof IControl ? ['control' => $rule->arg->getHtmlName()] : $rule->arg;
 			}
 
 			$payload[] = $item;
 		}
+		if ($payload && $rules->isOptional()) {
+			array_unshift($payload, ['op' => 'optional']);
+		}
 		return $payload;
 	}
 
@@ -142,7 +149,7 @@ class Helpers extends Nette\Object
 		$res = '';
 		$input = Html::el();
 		$label = Html::el();
-		list($wrapper, $wrapperEnd) = $wrapper instanceof Html ? array($wrapper->startTag(), $wrapper->endTag()) : array((string) $wrapper, '');
+		list($wrapper, $wrapperEnd) = $wrapper instanceof Html ? [$wrapper->startTag(), $wrapper->endTag()] : [(string) $wrapper, ''];
 
 		foreach ($items as $value => $caption) {
 			foreach ($inputAttrs as $k => $v) {
@@ -179,7 +186,7 @@ class Helpers extends Nette\Object
 				$res .= Html::el('optgroup')->label($group)->startTag();
 				$tmp = '</optgroup>';
 			} else {
-				$subitems = array($group => $subitems);
+				$subitems = [$group => $subitems];
 			}
 			foreach ($subitems as $value => $caption) {
 				$option->value = $value;
@@ -204,7 +211,7 @@ class Helpers extends Nette\Object
 
 	private static function prepareAttrs($attrs, $name)
 	{
-		$dynamic = array();
+		$dynamic = [];
 		foreach ((array) $attrs as $k => $v) {
 			$p = str_split($k, strlen($k) - 1);
 			if ($p[1] === '?' || $p[1] === ':') {
@@ -218,7 +225,7 @@ class Helpers extends Nette\Object
 				}
 			}
 		}
-		return array($dynamic, '<' . $name . Html::el(NULL, $attrs)->attributes());
+		return [$dynamic, '<' . $name . Html::el(NULL, $attrs)->attributes()];
 	}
 
 }
