@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms;
 
-use Nette;
-
 
 /**
  * Defines method that must be implemented to allow a component to act like a form control.
- *
- * @author     David Grudl
  */
 interface IControl
 {
@@ -48,12 +44,4 @@ interface IControl
 	 */
 	function isOmitted();
 
-	/**
-	 * Returns translated string.
-	 * @param  string
-	 * @param  int      plural count
-	 * @return string
-	 */
-	function translate($s, $count = NULL);
-
 }
