@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms;
 
-use Nette;
-
 
 /**
  * Defines method that must be implemented to allow a control to submit web form.
- *
- * @author     David Grudl
  */
 interface ISubmitterControl extends IControl
 {
