@@ -1,23 +1,23 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms\Rendering;
 
-use Nette,
-	Nette\Utils\Html;
+use Nette;
+use Nette\Utils\Html;
 
 
 /**
  * Converts a Form into the HTML output.
- *
- * @author     David Grudl
  */
-class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRenderer
+class DefaultFormRenderer implements Nette\Forms\IFormRenderer
 {
+	use Nette\SmartObject;
+
 	/**
 	 *  /--- form.container
 	 *
@@ -55,35 +55,35 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 	 *  \--
 	 *
 	 * @var array of HTML tags */
-	public $wrappers = array(
-		'form' => array(
+	public $wrappers = [
+		'form' => [
 			'container' => NULL,
-		),
+		],
 
-		'error' => array(
+		'error' => [
 			'container' => 'ul class=error',
 			'item' => 'li',
-		),
+		],
 
-		'group' => array(
+		'group' => [
 			'container' => 'fieldset',
 			'label' => 'legend',
 			'description' => 'p',
-		),
+		],
 
-		'controls' => array(
+		'controls' => [
 			'container' => 'table',
-		),
+		],
 
-		'pair' => array(
+		'pair' => [
 			'container' => 'tr',
 			'.required' => 'required',
 			'.optional' => NULL,
 			'.odd' => NULL,
 			'.error' => NULL,
-		),
+		],
 
-		'control' => array(
+		'control' => [
 			'container' => 'td',
 			'.odd' => NULL,
 
@@ -96,21 +96,23 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 			'.text' => 'text',
 			'.password' => 'text',
 			'.file' => 'text',
+			'.email' => 'text',
+			'.number' => 'text',
 			'.submit' => 'button',
 			'.image' => 'imagebutton',
 			'.button' => 'button',
-		),
+		],
 
-		'label' => array(
+		'label' => [
 			'container' => 'th',
 			'suffix' => NULL,
 			'requiredsuffix' => '',
-		),
+		],
 
-		'hidden' => array(
-			'container' => 'div',
-		),
-	);
+		'hidden' => [
+			'container' => NULL,
+		],
+	];
 
 	/** @var Nette\Forms\Form */
 	protected $form;
@@ -129,7 +131,6 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 	{
 		if ($this->form !== $form) {
 			$this->form = $form;
-			$this->init();
 		}
 
 		$s = '';
@@ -152,26 +153,6 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 	}
 
 
-	/**
-	 * Initializes form.
-	 * @return void
-	 */
-	protected function init()
-	{
-		$wrapper = & $this->wrappers['control'];
-		foreach ($this->form->getControls() as $control) {
-			if ($control->isRequired() && isset($wrapper['.required'])) {
-				$control->getLabelPrototype()->class($wrapper['.required'], TRUE);
-			}
-
-			$el = $control->getControlPrototype();
-			if ($el->getName() === 'input' && isset($wrapper['.' . $el->type])) {
-				$el->class($wrapper['.' . $el->type], TRUE);
-			}
-		}
-	}
-
-
 	/**
 	 * Renders form begin.
 	 * @return string
@@ -184,7 +165,7 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 			$control->setOption('rendered', FALSE);
 		}
 
-		if (strcasecmp($this->form->getMethod(), 'get') === 0) {
+		if ($this->form->isMethod('get')) {
 			$el = clone $this->form->getElementPrototype();
 			$query = parse_url($el->action, PHP_URL_QUERY);
 			$el->action = str_replace("?$query", '', $el->action);
@@ -193,7 +174,7 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 				$parts = explode('=', $param, 2);
 				$name = urldecode($parts[0]);
 				if (!isset($this->form[$name])) {
-					$s .= Html::el('input', array('type' => 'hidden', 'name' => $name, 'value' => urldecode($parts[1])));
+					$s .= Html::el('input', ['type' => 'hidden', 'name' => $name, 'value' => urldecode($parts[1])]);
 				}
 			}
 			return $el->startTag() . ($s ? "\n\t" . $this->getWrapper('hidden container')->setHtml($s) : '');
@@ -212,11 +193,11 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 	{
 		$s = '';
 		foreach ($this->form->getControls() as $control) {
-			if ($control instanceof Nette\Forms\Controls\HiddenField && !$control->getOption('rendered')) {
+			if ($control->getOption('type') === 'hidden' && !$control->getOption('rendered')) {
 				$s .= $control->getControl();
 			}
 		}
-		if (iterator_count($this->form->getComponents(TRUE, 'Nette\Forms\Controls\TextInput')) < 2) {
+		if (iterator_count($this->form->getComponents(TRUE, Nette\Forms\Controls\TextInput::class)) < 2) {
 			$s .= '<!--[if IE]><input type=IEbug disabled style="display:none"><![endif]-->';
 		}
 		if ($s) {
@@ -245,11 +226,11 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 		foreach ($errors as $error) {
 			$item = clone $item;
 			if ($error instanceof Html) {
-				$item->add($error);
+				$item->addHtml($error);
 			} else {
 				$item->setText($error);
 			}
-			$container->add($item);
+			$container->addHtml($item);
 		}
 		return "\n" . $container->render($control ? 1 : 0);
 	}
@@ -274,11 +255,16 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 			$container = $group->getOption('container', $defaultContainer);
 			$container = $container instanceof Html ? clone $container : Html::el($container);
 
+			$id = $group->getOption('id');
+			if ($id) {
+				$container->id = $id;
+			}
+
 			$s .= "\n" . $container->startTag();
 
 			$text = $group->getOption('label');
 			if ($text instanceof Html) {
-				$s .= $this->getWrapper('group label')->add($text);
+				$s .= $this->getWrapper('group label')->addHtml($text);
 
 			} elseif (is_string($text)) {
 				if ($translator !== NULL) {
@@ -317,36 +303,36 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 
 	/**
 	 * Renders group of controls.
-	 * @param  Nette\Forms\Container|FormGroup
+	 * @param  Nette\Forms\Container|Nette\Forms\ControlGroup
 	 * @return string
 	 */
 	public function renderControls($parent)
 	{
 		if (!($parent instanceof Nette\Forms\Container || $parent instanceof Nette\Forms\ControlGroup)) {
-			throw new Nette\InvalidArgumentException('Argument must be FormContainer or FormGroup instance.');
+			throw new Nette\InvalidArgumentException('Argument must be Nette\Forms\Container or Nette\Forms\ControlGroup instance.');
 		}
 
 		$container = $this->getWrapper('controls container');
 
 		$buttons = NULL;
 		foreach ($parent->getControls() as $control) {
-			if ($control->getOption('rendered') || $control instanceof Nette\Forms\Controls\HiddenField || $control->getForm(FALSE) !== $this->form) {
+			if ($control->getOption('rendered') || $control->getOption('type') === 'hidden' || $control->getForm(FALSE) !== $this->form) {
 				// skip
 
-			} elseif ($control instanceof Nette\Forms\Controls\Button) {
+			} elseif ($control->getOption('type') === 'button') {
 				$buttons[] = $control;
 
 			} else {
 				if ($buttons) {
-					$container->add($this->renderPairMulti($buttons));
+					$container->addHtml($this->renderPairMulti($buttons));
 					$buttons = NULL;
 				}
-				$container->add($this->renderPair($control));
+				$container->addHtml($this->renderPair($control));
 			}
 		}
 
 		if ($buttons) {
-			$container->add($this->renderPairMulti($buttons));
+			$container->addHtml($this->renderPairMulti($buttons));
 		}
 
 		$s = '';
@@ -365,8 +351,8 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 	public function renderPair(Nette\Forms\IControl $control)
 	{
 		$pair = $this->getWrapper('pair container');
-		$pair->add($this->renderLabel($control));
-		$pair->add($this->renderControl($control));
+		$pair->addHtml($this->renderLabel($control));
+		$pair->addHtml($this->renderControl($control));
 		$pair->class($this->getValue($control->isRequired() ? 'pair .required' : 'pair .optional'), TRUE);
 		$pair->class($control->hasErrors() ? $this->getValue('pair .error') : NULL, TRUE);
 		$pair->class($control->getOption('class'), TRUE);
@@ -380,32 +366,40 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 
 	/**
 	 * Renders single visual row of multiple controls.
-	 * @param  IFormControl[]
+	 * @param  Nette\Forms\IControl[]
 	 * @return string
 	 */
 	public function renderPairMulti(array $controls)
 	{
-		$s = array();
+		$s = [];
 		foreach ($controls as $control) {
 			if (!$control instanceof Nette\Forms\IControl) {
-				throw new Nette\InvalidArgumentException('Argument must be array of IFormControl instances.');
+				throw new Nette\InvalidArgumentException('Argument must be array of Nette\Forms\IControl instances.');
 			}
 			$description = $control->getOption('description');
 			if ($description instanceof Html) {
 				$description = ' ' . $control->getOption('description');
 
 			} elseif (is_string($description)) {
-				$description = ' ' . $this->getWrapper('control description')->setText($control->translate($description));
+				if ($control instanceof Nette\Forms\Controls\BaseControl) {
+					$description = $control->translate($description);
+				}
+				$description = ' ' . $this->getWrapper('control description')->setText($description);
 
 			} else {
 				$description = '';
 			}
 
-			$s[] = $control->getControl() . $description;
+			$control->setOption('rendered', TRUE);
+			$el = $control->getControl();
+			if ($el instanceof Html && $el->getName() === 'input') {
+				$el->class($this->getValue("control .$el->type"), TRUE);
+			}
+			$s[] = $el . $description;
 		}
 		$pair = $this->getWrapper('pair container');
-		$pair->add($this->renderLabel($control));
-		$pair->add($this->getWrapper('control container')->setHtml(implode(' ', $s)));
+		$pair->addHtml($this->renderLabel($control));
+		$pair->addHtml($this->getWrapper('control container')->setHtml(implode(' ', $s)));
 		return $pair->render(0);
 	}
 
@@ -419,7 +413,10 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 		$suffix = $this->getValue('label suffix') . ($control->isRequired() ? $this->getValue('label requiredsuffix') : '');
 		$label = $control->getLabel();
 		if ($label instanceof Html) {
-			$label->add($suffix);
+			$label->addHtml($suffix);
+			if ($control->isRequired()) {
+				$label->class($this->getValue('control .required'), TRUE);
+			}
 		} elseif ($label != NULL) { // @intentionally ==
 			$label .= $suffix;
 		}
@@ -443,7 +440,10 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 			$description = ' ' . $description;
 
 		} elseif (is_string($description)) {
-			$description = ' ' . $this->getWrapper('control description')->setText($control->translate($description));
+			if ($control instanceof Nette\Forms\Controls\BaseControl) {
+				$description = $control->translate($description);
+			}
+			$description = ' ' . $this->getWrapper('control description')->setText($description);
 
 		} else {
 			$description = '';
@@ -453,14 +453,18 @@ class DefaultFormRenderer extends Nette\Object implements Nette\Forms\IFormRende
 			$description = $this->getValue('control requiredsuffix') . $description;
 		}
 
+		$control->setOption('rendered', TRUE);
 		$el = $control->getControl();
+		if ($el instanceof Html && $el->getName() === 'input') {
+			$el->class($this->getValue("control .$el->type"), TRUE);
+		}
 		return $body->setHtml($el . $description . $this->renderErrors($control));
 	}
 
 
 	/**
 	 * @param  string
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
 	protected function getWrapper($name)
 	{
