@@ -13,19 +13,21 @@ use Nette;
 /**
  * Provides access to session sections as well as session settings and management methods.
  */
-class Session extends Nette\Object
+class Session
 {
-	/** Default file lifetime is 3 hours */
-	const DEFAULT_FILE_LIFETIME = 10800;
+	use Nette\SmartObject;
+
+	/** Default file lifetime */
+	const DEFAULT_FILE_LIFETIME = 3 * Nette\Utils\DateTime::HOUR;
 
 	/** @var bool  has been session ID regenerated? */
-	private $regenerated;
+	private $regenerated = FALSE;
 
 	/** @var bool  has been session started? */
-	private static $started;
+	private static $started = FALSE;
 
 	/** @var array default configuration */
-	private $options = array(
+	private $options = [
 		// security
 		'referer_check' => '',    // must be disabled because PHP implementation is invalid
 		'use_cookies' => 1,       // must be enabled to prevent Session Hijacking and Fixation
@@ -45,7 +47,7 @@ class Session extends Nette\Object
 		'cache_expire' => NULL,   // (default "180")
 		'hash_function' => NULL,  // (default "0", means MD5)
 		'hash_bits_per_character' => NULL, // (default "4")
-	);
+	];
 
 	/** @var IRequest */
 	private $request;
@@ -86,13 +88,12 @@ class Session extends Nette\Object
 
 		try {
 			// session_start returns FALSE on failure only sometimes
-			Nette\Utils\Callback::invokeSafe('session_start', array(), function ($message) use (& $e) {
+			Nette\Utils\Callback::invokeSafe('session_start', [], function ($message) use (& $e) {
 				$e = new Nette\InvalidStateException($message);
 			});
 		} catch (\Exception $e) {
 		}
 
-		Helpers::removeDuplicateCookies();
 		if ($e) {
 			@session_write_close(); // this is needed
 			throw $e;
@@ -148,7 +149,7 @@ class Session extends Nette\Object
 			$this->regenerateId();
 		}
 
-		register_shutdown_function(array($this, 'clean'));
+		register_shutdown_function([$this, 'clean']);
 	}
 
 
@@ -217,14 +218,13 @@ class Session extends Nette\Object
 			if (headers_sent($file, $line)) {
 				throw new Nette\InvalidStateException('Cannot regenerate session ID after HTTP headers have been sent' . ($file ? " (output started at $file:$line)." : '.'));
 			}
-			if (session_id() !== '') {
+			if (session_status() === PHP_SESSION_ACTIVE) {
 				session_regenerate_id(TRUE);
+				session_write_close();
 			}
-			session_write_close();
 			$backup = $_SESSION;
 			session_start();
 			$_SESSION = $backup;
-			Helpers::removeDuplicateCookies();
 		}
 		$this->regenerated = TRUE;
 	}
@@ -252,9 +252,9 @@ class Session extends Nette\Object
 		}
 
 		session_name($name);
-		return $this->setOptions(array(
+		return $this->setOptions([
 			'name' => $name,
-		));
+		]);
 	}
 
 
@@ -278,7 +278,7 @@ class Session extends Nette\Object
 	 * @return SessionSection
 	 * @throws Nette\InvalidArgumentException
 	 */
-	public function getSection($section, $class = 'Nette\Http\SessionSection')
+	public function getSection($section, $class = SessionSection::class)
 	{
 		return new $class($this, $section);
 	}
@@ -388,7 +388,7 @@ class Session extends Nette\Object
 	 */
 	private function configure(array $config)
 	{
-		$special = array('cache_expire' => 1, 'cache_limiter' => 1, 'save_path' => 1, 'name' => 1);
+		$special = ['cache_expire' => 1, 'cache_limiter' => 1, 'save_path' => 1, 'name' => 1];
 
 		foreach ($config as $key => $value) {
 			if (!strncmp($key, 'session.', 8)) { // back compatibility
@@ -406,7 +406,7 @@ class Session extends Nette\Object
 				$cookie[substr($key, 7)] = $value;
 
 			} else {
-				if (defined('SID')) {
+				if (session_status() === PHP_SESSION_ACTIVE) {
 					throw new Nette\InvalidStateException("Unable to set 'session.$key' to value '$value' when session has been started" . ($this->started ? '.' : ' by session.auto_start or session_start().'));
 				}
 				if (isset($special[$key])) {
@@ -414,9 +414,9 @@ class Session extends Nette\Object
 					$key($value);
 
 				} elseif (function_exists('ini_set')) {
-					ini_set("session.$key", $value);
+					ini_set("session.$key", (string) $value);
 
-				} elseif (ini_get("session.$key") != $value) { // intentionally ==
+				} elseif (ini_get("session.$key") != $value) { // intentionally !=
 					throw new Nette\NotSupportedException("Unable to set 'session.$key' to '$value' because function ini_set() is disabled.");
 				}
 			}
@@ -440,23 +440,23 @@ class Session extends Nette\Object
 
 	/**
 	 * Sets the amount of time allowed between requests before the session will be terminated.
-	 * @param  string|int|\DateTime  time, value 0 means "until the browser is closed"
+	 * @param  string|int|\DateTimeInterface  time, value 0 means "until the browser is closed"
 	 * @return self
 	 */
 	public function setExpiration($time)
 	{
 		if (empty($time)) {
-			return $this->setOptions(array(
+			return $this->setOptions([
 				'gc_maxlifetime' => self::DEFAULT_FILE_LIFETIME,
 				'cookie_lifetime' => 0,
-			));
+			]);
 
 		} else {
 			$time = Nette\Utils\DateTime::from($time)->format('U') - time();
-			return $this->setOptions(array(
+			return $this->setOptions([
 				'gc_maxlifetime' => $time,
 				'cookie_lifetime' => $time,
-			));
+			]);
 		}
 	}
 
@@ -470,11 +470,11 @@ class Session extends Nette\Object
 	 */
 	public function setCookieParameters($path, $domain = NULL, $secure = NULL)
 	{
-		return $this->setOptions(array(
+		return $this->setOptions([
 			'cookie_path' => $path,
 			'cookie_domain' => $domain,
 			'cookie_secure' => $secure,
-		));
+		]);
 	}
 
 
@@ -494,14 +494,14 @@ class Session extends Nette\Object
 	 */
 	public function setSavePath($path)
 	{
-		return $this->setOptions(array(
+		return $this->setOptions([
 			'save_path' => $path,
-		));
+		]);
 	}
 
 
 	/**
-	 * Sets user session storage for PHP < 5.4. For PHP >= 5.4, use setHandler().
+	 * @deprecated  use setHandler().
 	 * @return self
 	 */
 	public function setStorage(ISessionStorage $storage)
@@ -510,8 +510,8 @@ class Session extends Nette\Object
 			throw new Nette\InvalidStateException('Unable to set storage when session has been started.');
 		}
 		session_set_save_handler(
-			array($storage, 'open'), array($storage, 'close'), array($storage, 'read'),
-			array($storage, 'write'), array($storage, 'remove'), array($storage, 'clean')
+			[$storage, 'open'], [$storage, 'close'], [$storage, 'read'],
+			[$storage, 'write'], [$storage, 'remove'], [$storage, 'clean']
 		);
 		return $this;
 	}
@@ -537,10 +537,6 @@ class Session extends Nette\Object
 	 */
 	private function sendCookie()
 	{
-		if (!headers_sent() && ob_get_level() && ob_get_length()) {
-			trigger_error('Possible problem: you are starting session while already having some data in output buffer. This may not work if the outputted data grows. Try starting the session earlier.', E_USER_NOTICE);
-		}
-
 		$cookie = $this->getCookieParameters();
 		$this->response->setCookie(
 			session_name(), session_id(),
