@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Iterators;
@@ -13,8 +13,6 @@ use Nette;
 /**
  * Smarter caching iterator.
  *
- * @author     David Grudl
- *
  * @property-read bool $first
  * @property-read bool $last
  * @property-read bool $empty
@@ -23,12 +21,11 @@ use Nette;
  * @property-read int $counter
  * @property-read mixed $nextKey
  * @property-read mixed $nextValue
- * @property-read $innerIterator
- * @property   $flags
- * @property-read $cache
  */
 class CachingIterator extends \CachingIterator implements \Countable
 {
+	use Nette\SmartObject;
+
 	/** @var int */
 	private $counter = 0;
 
@@ -48,7 +45,7 @@ class CachingIterator extends \CachingIterator implements \Countable
 				$iterator = new \IteratorIterator($iterator);
 			}
 		} else {
-			throw new Nette\InvalidArgumentException(sprintf('Invalid argument passed to foreach resp. %s; array or Traversable expected, %s given.', __CLASS__, is_object($iterator) ? get_class($iterator) : gettype($iterator)));
+			throw new Nette\InvalidArgumentException(sprintf('Invalid argument passed to %s; array or Traversable expected, %s given.', __CLASS__, is_object($iterator) ? get_class($iterator) : gettype($iterator)));
 		}
 
 		parent::__construct($iterator, 0);
@@ -176,69 +173,4 @@ class CachingIterator extends \CachingIterator implements \Countable
 		return $this->getInnerIterator()->current();
 	}
 
-
-	/********************* Nette\Object behaviour ****************d*g**/
-
-
-	/**
-	 * Call to undefined method.
-	 * @param  string  method name
-	 * @param  array   arguments
-	 * @return mixed
-	 * @throws Nette\MemberAccessException
-	 */
-	public function __call($name, $args)
-	{
-		return Nette\ObjectMixin::call($this, $name, $args);
-	}
-
-
-	/**
-	 * Returns property value. Do not call directly.
-	 * @param  string  property name
-	 * @return mixed   property value
-	 * @throws Nette\MemberAccessException if the property is not defined.
-	 */
-	public function &__get($name)
-	{
-		return Nette\ObjectMixin::get($this, $name);
-	}
-
-
-	/**
-	 * Sets value of a property. Do not call directly.
-	 * @param  string  property name
-	 * @param  mixed   property value
-	 * @return void
-	 * @throws Nette\MemberAccessException if the property is not defined or is read-only
-	 */
-	public function __set($name, $value)
-	{
-		Nette\ObjectMixin::set($this, $name, $value);
-	}
-
-
-	/**
-	 * Is property defined?
-	 * @param  string  property name
-	 * @return bool
-	 */
-	public function __isset($name)
-	{
-		return Nette\ObjectMixin::has($this, $name);
-	}
-
-
-	/**
-	 * Access to undeclared property.
-	 * @param  string  property name
-	 * @return void
-	 * @throws Nette\MemberAccessException
-	 */
-	public function __unset($name)
-	{
-		Nette\ObjectMixin::remove($this, $name);
-	}
-
-
 }
