@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Iterators;
@@ -12,8 +12,7 @@ use Nette;
 
 /**
  * CallbackFilterIterator for PHP < 5.4.
- *
- * @author     David Grudl
+ * @deprecated use CallbackFilterIterator
  */
 class Filter extends \FilterIterator
 {
@@ -23,6 +22,7 @@ class Filter extends \FilterIterator
 
 	public function __construct(\Iterator $iterator, $callback)
 	{
+		trigger_error(__CLASS__ . ' is deprecated, use CallbackFilterIterator.', E_USER_WARNING);
 		parent::__construct($iterator);
 		$this->callback = Nette\Utils\Callback::check($callback);
 	}
