@@ -1,22 +1,23 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Latte (https://latte.nette.org)
+ * Copyright (c) 2008 David Grudl (https://davidgrudl.com)
  */
 
-namespace Nette\Latte;
-
-use Nette;
+namespace Latte;
 
 
 /**
  * Latte macro.
- *
- * @author     David Grudl
  */
 interface IMacro
 {
+	const
+		AUTO_EMPTY = 4,
+		AUTO_CLOSE = 64,
+		ALLOWED_IN_HEAD = 128,
+		DEFAULT_FLAGS = 0;
 
 	/**
 	 * Initializes before template parsing.
