@@ -66,33 +66,37 @@ class NetteLoader
 	/** @var array */
 	public $list = array(
 		'Latte\CompileException' => 'Latte/exceptions',
-		'Latte\Compiler' => 'Latte/Compiler',
+		'Latte\Compiler' => 'Latte/Compiler/Compiler',
 		'Latte\Engine' => 'Latte/Engine',
 		'Latte\Helpers' => 'Latte/Helpers',
-		'Latte\HtmlNode' => 'Latte/HtmlNode',
+		'Latte\HtmlNode' => 'Latte/Compiler/HtmlNode',
 		'Latte\ILoader' => 'Latte/ILoader',
 		'Latte\IMacro' => 'Latte/IMacro',
 		'Latte\Loaders\FileLoader' => 'Latte/Loaders/FileLoader',
 		'Latte\Loaders\StringLoader' => 'Latte/Loaders/StringLoader',
-		'Latte\MacroNode' => 'Latte/MacroNode',
-		'Latte\MacroTokens' => 'Latte/MacroTokens',
+		'Latte\MacroNode' => 'Latte/Compiler/MacroNode',
+		'Latte\MacroTokens' => 'Latte/Compiler/MacroTokens',
 		'Latte\Macros\BlockMacros' => 'Latte/Macros/BlockMacros',
-		'Latte\Macros\BlockMacrosRuntime' => 'Latte/Macros/BlockMacrosRuntime',
 		'Latte\Macros\CoreMacros' => 'Latte/Macros/CoreMacros',
 		'Latte\Macros\MacroSet' => 'Latte/Macros/MacroSet',
-		'Latte\Object' => 'Latte/Object',
-		'Latte\Parser' => 'Latte/Parser',
-		'Latte\PhpWriter' => 'Latte/PhpWriter',
+		'Latte\Parser' => 'Latte/Compiler/Parser',
+		'Latte\PhpHelpers' => 'Latte/Compiler/PhpHelpers',
+		'Latte\PhpWriter' => 'Latte/Compiler/PhpWriter',
 		'Latte\RegexpException' => 'Latte/exceptions',
 		'Latte\RuntimeException' => 'Latte/exceptions',
 		'Latte\Runtime\CachingIterator' => 'Latte/Runtime/CachingIterator',
+		'Latte\Runtime\FilterExecutor' => 'Latte/Runtime/FilterExecutor',
+		'Latte\Runtime\FilterInfo' => 'Latte/Runtime/FilterInfo',
 		'Latte\Runtime\Filters' => 'Latte/Runtime/Filters',
 		'Latte\Runtime\Html' => 'Latte/Runtime/Html',
 		'Latte\Runtime\IHtmlString' => 'Latte/Runtime/IHtmlString',
-		'Latte\Template' => 'Latte/Template',
-		'Latte\Token' => 'Latte/Token',
-		'Latte\TokenIterator' => 'Latte/TokenIterator',
-		'Latte\Tokenizer' => 'Latte/Tokenizer',
+		'Latte\Runtime\ISnippetBridge' => 'Latte/Runtime/ISnippetBridge',
+		'Latte\Runtime\SnippetDriver' => 'Latte/Runtime/SnippetDriver',
+		'Latte\Runtime\Template' => 'Latte/Runtime/Template',
+		'Latte\Strict' => 'Latte/Strict',
+		'Latte\Token' => 'Latte/Compiler/Token',
+		'Latte\TokenIterator' => 'Latte/Compiler/TokenIterator',
+		'Latte\Tokenizer' => 'Latte/Compiler/Tokenizer',
 		'NetteModule\ErrorPresenter' => 'Application/ErrorPresenter',
 		'NetteModule\MicroPresenter' => 'Application/MicroPresenter',
 		'Nette\Application\AbortException' => 'Application/exceptions',
@@ -110,12 +114,7 @@ class NetteLoader
 		'Nette\Database\ConstraintViolationException' => 'Database/exceptions',
 		'Nette\Database\Conventions\AmbiguousReferenceKeyException' => 'Database/Conventions/exceptions',
 		'Nette\Database\ForeignKeyConstraintViolationException' => 'Database/exceptions',
-		'Nette\Database\IReflection' => 'Database/deprecated/IReflection',
 		'Nette\Database\NotNullConstraintViolationException' => 'Database/exceptions',
-		'Nette\Database\Reflection\AmbiguousReferenceKeyException' => 'Database/deprecated/exceptions',
-		'Nette\Database\Reflection\ConventionalReflection' => 'Database/deprecated/ConventionalReflection',
-		'Nette\Database\Reflection\DiscoveredReflection' => 'Database/deprecated/DiscoveredReflection',
-		'Nette\Database\Reflection\MissingReferenceException' => 'Database/deprecated/exceptions',
 		'Nette\Database\UniqueConstraintViolationException' => 'Database/exceptions',
 		'Nette\DeprecatedException' => 'Utils/exceptions',
 		'Nette\Diagnostics\Debugger' => 'deprecated/Diagnostics/Debugger',
@@ -132,6 +131,7 @@ class NetteLoader
 		'Nette\Latte\Engine' => 'deprecated/Latte/Engine',
 		'Nette\Loaders\RobotLoader' => 'RobotLoader/RobotLoader',
 		'Nette\Localization\ITranslator' => 'Utils/ITranslator',
+		'Nette\Mail\FallbackMailerException' => 'Mail/exceptions',
 		'Nette\Mail\SendException' => 'Mail/exceptions',
 		'Nette\Mail\SmtpException' => 'Mail/exceptions',
 		'Nette\MemberAccessException' => 'Utils/exceptions',
@@ -139,6 +139,8 @@ class NetteLoader
 		'Nette\NotSupportedException' => 'Utils/exceptions',
 		'Nette\Object' => 'Utils/Object',
 		'Nette\OutOfRangeException' => 'Utils/exceptions',
+		'Nette\SmartObject' => 'Utils/SmartObject',
+		'Nette\StaticClass' => 'Utils/StaticClass',
 		'Nette\StaticClassException' => 'Utils/exceptions',
 		'Nette\Templating\FileTemplate' => 'deprecated/Templating/FileTemplate',
 		'Nette\Templating\Helpers' => 'deprecated/Templating/Helpers',
@@ -147,13 +149,11 @@ class NetteLoader
 		'Nette\Templating\Template' => 'deprecated/Templating/Template',
 		'Nette\UnexpectedValueException' => 'Utils/exceptions',
 		'Nette\Utils\AssertionException' => 'Utils/exceptions',
-		'Nette\Utils\CallbackFilterIterator' => 'Finder/CallbackFilterIterator',
 		'Nette\Utils\Finder' => 'Finder/Finder',
 		'Nette\Utils\ImageException' => 'Utils/exceptions',
 		'Nette\Utils\JsonException' => 'Utils/exceptions',
 		'Nette\Utils\LimitedScope' => 'deprecated/Utils/LimitedScope',
 		'Nette\Utils\MimeTypeDetector' => 'deprecated/Utils/MimeTypeDetector',
-		'Nette\Utils\RecursiveCallbackFilterIterator' => 'Finder/RecursiveCallbackFilterIterator',
 		'Nette\Utils\RegexpException' => 'Utils/exceptions',
 		'Nette\Utils\SafeStream' => 'SafeStream/SafeStream',
 		'Nette\Utils\TokenIterator' => 'Tokenizer/TokenIterator',
