@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Mail;
 
-use Nette;
-
 
 /**
  * Mailer interface.
- *
- * @author     David Grudl
  */
 interface IMailer
 {
@@ -21,6 +17,7 @@ interface IMailer
 	/**
 	 * Sends email.
 	 * @return void
+	 * @throws SendException
 	 */
 	function send(Message $mail);
 
