@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Mail;
@@ -12,11 +12,11 @@ use Nette;
 
 /**
  * Sends emails via the PHP internal mail() function.
- *
- * @author     David Grudl
  */
-class SendmailMailer extends Nette\Object implements IMailer
+class SendmailMailer implements IMailer
 {
+	use Nette\SmartObject;
+
 	/** @var string */
 	public $commandArgs;
 
@@ -24,6 +24,7 @@ class SendmailMailer extends Nette\Object implements IMailer
 	/**
 	 * Sends email.
 	 * @return void
+	 * @throws SendException
 	 */
 	public function send(Message $mail)
 	{
@@ -33,20 +34,20 @@ class SendmailMailer extends Nette\Object implements IMailer
 
 		$parts = explode(Message::EOL . Message::EOL, $tmp->generateMessage(), 2);
 
-		$args = array(
+		$args = [
 			str_replace(Message::EOL, PHP_EOL, $mail->getEncodedHeader('To')),
 			str_replace(Message::EOL, PHP_EOL, $mail->getEncodedHeader('Subject')),
 			str_replace(Message::EOL, PHP_EOL, $parts[1]),
 			str_replace(Message::EOL, PHP_EOL, $parts[0]),
-		);
+		];
 		if ($this->commandArgs) {
 			$args[] = (string) $this->commandArgs;
 		}
-		$res = Nette\Utils\Callback::invokeSafe('mail', $args, function($message) use (& $info) {
+		$res = Nette\Utils\Callback::invokeSafe('mail', $args, function ($message) use (& $info) {
 			$info = ": $message";
 		});
 		if ($res === FALSE) {
-			throw new Nette\InvalidStateException("Unable to send email$info.");
+			throw new SendException("Unable to send email$info.");
 		}
 	}
 
