@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\PhpGenerator;
@@ -12,12 +12,14 @@ use Nette;
 
 /**
  * PHP code generator utils.
- *
- * @author     David Grudl
  */
 class Helpers
 {
+	use Nette\StaticClass;
+
 	const PHP_IDENT = '[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*';
+	const MAX_DEPTH = 50;
+	const WRAP_LENGTH = 70;
 
 
 	/**
@@ -36,8 +38,11 @@ class Helpers
 			return (string) $var;
 
 		} elseif (is_float($var)) {
-			$var = var_export($var, TRUE);
-			return strpos($var, '.') === FALSE ? $var . '.0' : $var;
+			if (is_finite($var)) {
+				$var = var_export($var, TRUE);
+				return strpos($var, '.') === FALSE ? $var . '.0' : $var; // workaround for PHP < 7.0.2
+			}
+			return str_replace('.0', '', var_export($var, TRUE)); // workaround for PHP 7.0.2
 
 		} elseif (is_bool($var)) {
 			return $var ? 'TRUE' : 'FALSE';
@@ -57,6 +62,9 @@ class Helpers
 			}
 			return '"' . strtr($var, $table) . '"';
 
+		} elseif (is_string($var)) {
+			return "'" . preg_replace('#\'|\\\\(?=[\'\\\\]|\z)#', '\\\\$0', $var) . "'";
+
 		} elseif (is_array($var)) {
 			$space = str_repeat("\t", $level);
 
@@ -67,7 +75,7 @@ class Helpers
 			if (empty($var)) {
 				$out = '';
 
-			} elseif ($level > 50 || isset($var[$marker])) {
+			} elseif ($level > self::MAX_DEPTH || isset($var[$marker])) {
 				throw new Nette\InvalidArgumentException('Nesting level too deep or recursive dependency.');
 
 			} else {
@@ -85,19 +93,29 @@ class Helpers
 				}
 				unset($var[$marker]);
 			}
-			return 'array(' . (strpos($out, "\n") === FALSE && strlen($out) < 40 ? $out : $outAlt) . ')';
+			return '[' . (strpos($out, "\n") === FALSE && strlen($out) < self::WRAP_LENGTH ? $out : $outAlt) . ']';
 
 		} elseif ($var instanceof \Serializable) {
 			$var = serialize($var);
 			return 'unserialize(' . self::_dump($var, $level) . ')';
 
+		} elseif ($var instanceof \Closure) {
+			throw new Nette\InvalidArgumentException('Cannot dump closure.');
+
 		} elseif (is_object($var)) {
+			$class = get_class($var);
+			if (PHP_VERSION_ID >= 70000 && (new \ReflectionObject($var))->isAnonymous()) {
+				throw new Nette\InvalidArgumentException('Cannot dump anonymous class.');
+
+			} elseif (in_array($class, ['DateTime', 'DateTimeImmutable'], TRUE)) {
+				return self::formatArgs("new $class(?, new DateTimeZone(?))", [$var->format('Y-m-d H:i:s.u'), $var->getTimeZone()->getName()]);
+			}
+
 			$arr = (array) $var;
 			$space = str_repeat("\t", $level);
-			$class = get_class($var);
 
-			static $list = array();
-			if ($level > 50 || in_array($var, $list, TRUE)) {
+			static $list = [];
+			if ($level > self::MAX_DEPTH || in_array($var, $list, TRUE)) {
 				throw new Nette\InvalidArgumentException('Nesting level too deep or recursive dependency.');
 
 			} else {
@@ -110,15 +128,15 @@ class Helpers
 				}
 				foreach ($arr as $k => & $v) {
 					if (!isset($props) || isset($props[$k])) {
-						$out .= "$space\t" . self::_dump($k, $level + 1) . " => " . self::_dump($v, $level + 1) . ",\n";
+						$out .= "$space\t" . self::_dump($k, $level + 1) . ' => ' . self::_dump($v, $level + 1) . ",\n";
 					}
 				}
 				array_pop($list);
 				$out .= $space;
 			}
 			return $class === 'stdClass'
-				? "(object) array($out)"
-				: __CLASS__ . "::createObject('$class', array($out))";
+				? "(object) [$out]"
+				: __CLASS__ . "::createObject('$class', [$out])";
 
 		} elseif (is_resource($var)) {
 			throw new Nette\InvalidArgumentException('Cannot dump resource.');
@@ -133,10 +151,9 @@ class Helpers
 	 * Generates PHP statement.
 	 * @return string
 	 */
-	public static function format($statement)
+	public static function format($statement, ...$args)
 	{
-		$args = func_get_args();
-		return self::formatArgs(array_shift($args), $args);
+		return self::formatArgs($statement, $args);
 	}
 
 
@@ -156,15 +173,22 @@ class Helpers
 				if (!is_array($arg)) {
 					throw new Nette\InvalidArgumentException('Argument must be an array.');
 				}
-				$arg = implode(', ', array_map(array(__CLASS__, 'dump'), $arg));
-				$statement = substr_replace($statement, $arg, $a, 2);
+				$s = substr($statement, 0, $a);
+				$sep = '';
+				foreach ($arg as $tmp) {
+					$s .= $sep . self::dump($tmp);
+					$sep = strlen($s) - strrpos($s, "\n") > self::WRAP_LENGTH ? ",\n\t" : ', ';
+				}
+				$statement = $s . substr($statement, $a + 2);
+				$a = strlen($s);
 
 			} else {
-				$arg = substr($statement, $a - 1, 1) === '$' || in_array(substr($statement, $a - 2, 2), array('->', '::'), TRUE)
+				$arg = substr($statement, $a - 1, 1) === '$' || in_array(substr($statement, $a - 2, 2), ['->', '::'], TRUE)
 					? self::formatMember($arg) : self::_dump($arg);
 				$statement = substr_replace($statement, $arg, $a, 1);
+				$a += strlen($arg);
 			}
-			$a = strpos($statement, '?', $a + strlen($arg));
+			$a = strpos($statement, '?', $a);
 		}
 		return $statement;
 	}
@@ -178,7 +202,7 @@ class Helpers
 	{
 		return $name instanceof PhpLiteral || !self::isIdentifier($name)
 			? '{' . self::_dump($name) . '}'
-			: $name ;
+			: $name;
 	}
 
 
@@ -197,4 +221,24 @@ class Helpers
 		return unserialize('O' . substr(serialize((string) $class), 1, -1) . substr(serialize($props), 1));
 	}
 
+
+	/**
+	 * @param  string
+	 * @return string
+	 */
+	public static function extractNamespace($name)
+	{
+		return ($pos = strrpos($name, '\\')) ? substr($name, 0, $pos) : '';
+	}
+
+
+	/**
+	 * @param  string
+	 * @return string
+	 */
+	public static function extractShortName($name)
+	{
+		return ($pos = strrpos($name, '\\')) === FALSE ? $name : substr($name, $pos + 1);
+	}
+
 }
