@@ -1,20 +1,16 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Security;
 
-use Nette;
-
 
 /**
  * Authorizator checks if a given role has authorization
  * to access a given resource.
- *
- * @author     David Grudl
  */
 interface IAuthorizator
 {
