@@ -1,29 +1,25 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Security;
 
-use Nette;
-
 
 /**
  * Interface for persistent storage for user object data.
- *
- * @author David Grudl, Jan Tichý
  */
 interface IUserStorage
 {
 	/** Log-out reason {@link IUserStorage::getLogoutReason()} */
-	const MANUAL = 1,
-		INACTIVITY = 2,
-		BROWSER_CLOSED = 4;
+	const MANUAL = 0b0001,
+		INACTIVITY = 0b0010,
+		BROWSER_CLOSED = 0b0100;
 
 	/** Log-out behavior */
-	const CLEAR_IDENTITY = 8;
+	const CLEAR_IDENTITY = 0b1000;
 
 	/**
 	 * Sets the authenticated status of this user.
@@ -46,13 +42,13 @@ interface IUserStorage
 
 	/**
 	 * Returns current user identity, if any.
-	 * @return Nette\Security\IIdentity|NULL
+	 * @return IIdentity|NULL
 	 */
 	function getIdentity();
 
 	/**
 	 * Enables log out from the persistent storage after inactivity.
-	 * @param  string|int|DateTime number of seconds or timestamp
+	 * @param  string|int|\DateTimeInterface number of seconds or timestamp
 	 * @param  int Log out when the browser is closed | Clear the identity from persistent storage?
 	 * @return void
 	 */
