@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Security;
@@ -13,16 +13,18 @@ use Nette;
 /**
  * Default implementation of IIdentity.
  *
- * @author     David Grudl
- *
- * @serializationVersion 1.0
- *
  * @property   mixed $id
  * @property   array $roles
- * @property-read array $data
+ * @property   array $data
  */
-class Identity extends Nette\Object implements IIdentity
+class Identity implements IIdentity
 {
+	use Nette\SmartObject {
+		__get as private parentGet;
+		__set as private parentSet;
+		__isset as private parentIsSet;
+	}
+
 	/** @var mixed */
 	private $id;
 
@@ -53,7 +55,7 @@ class Identity extends Nette\Object implements IIdentity
 	 */
 	public function setId($id)
 	{
-		$this->id = is_numeric($id) ? 1 * $id : $id;
+		$this->id = is_numeric($id) && !is_float($tmp = $id * 1) ? $tmp : $id;
 		return $this;
 	}
 
@@ -108,8 +110,8 @@ class Identity extends Nette\Object implements IIdentity
 	 */
 	public function __set($key, $value)
 	{
-		if (parent::__isset($key)) {
-			parent::__set($key, $value);
+		if ($this->parentIsSet($key)) {
+			$this->parentSet($key, $value);
 
 		} else {
 			$this->data[$key] = $value;
@@ -124,8 +126,8 @@ class Identity extends Nette\Object implements IIdentity
 	 */
 	public function &__get($key)
 	{
-		if (parent::__isset($key)) {
-			return parent::__get($key);
+		if ($this->parentIsSet($key)) {
+			return $this->parentGet($key);
 
 		} else {
 			return $this->data[$key];
@@ -140,19 +142,7 @@ class Identity extends Nette\Object implements IIdentity
 	 */
 	public function __isset($key)
 	{
-		return isset($this->data[$key]) || parent::__isset($key);
-	}
-
-
-	/**
-	 * Removes property.
-	 * @param  string  property name
-	 * @return void
-	 * @throws Nette\MemberAccessException
-	 */
-	public function __unset($name)
-	{
-		Nette\ObjectMixin::remove($this, $name);
+		return isset($this->data[$key]) || $this->parentIsSet($key);
 	}
 
 }
