@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Security;
@@ -16,35 +16,31 @@ use Nette;
  * This solution is mostly based on Zend_Acl (c) Zend Technologies USA Inc. (http://www.zend.com), new BSD license
  *
  * @copyright  Copyright (c) 2005, 2007 Zend Technologies USA Inc.
- * @author     David Grudl
- *
- * @property-read array $roles
- * @property-read array $resources
- * @property-read mixed $queriedRole
- * @property-read mixed $queriedResource
  */
-class Permission extends Nette\Object implements IAuthorizator
+class Permission implements IAuthorizator
 {
+	use Nette\SmartObject;
+
 	/** @var array  Role storage */
-	private $roles = array();
+	private $roles = [];
 
 	/** @var array  Resource storage */
-	private $resources = array();
+	private $resources = [];
 
 	/** @var array  Access Control List rules; whitelist (deny everything to all) by default */
-	private $rules = array(
-		'allResources' => array(
-			'allRoles' => array(
-				'allPrivileges' => array(
+	private $rules = [
+		'allResources' => [
+			'allRoles' => [
+				'allPrivileges' => [
 					'type' => self::DENY,
 					'assert' => NULL,
-				),
-				'byPrivilege' => array(),
-			),
-			'byRole' => array(),
-		),
-		'byResource' => array(),
-	);
+				],
+				'byPrivilege' => [],
+			],
+			'byRole' => [],
+		],
+		'byResource' => [],
+	];
 
 	/** @var mixed */
 	private $queriedRole, $queriedResource;
@@ -69,11 +65,11 @@ class Permission extends Nette\Object implements IAuthorizator
 			throw new Nette\InvalidStateException("Role '$role' already exists in the list.");
 		}
 
-		$roleParents = array();
+		$roleParents = [];
 
 		if ($parents !== NULL) {
 			if (!is_array($parents)) {
-				$parents = array($parents);
+				$parents = [$parents];
 			}
 
 			foreach ($parents as $parent) {
@@ -83,10 +79,10 @@ class Permission extends Nette\Object implements IAuthorizator
 			}
 		}
 
-		$this->roles[$role] = array(
-			'parents'  => $roleParents,
-			'children' => array(),
-		);
+		$this->roles[$role] = [
+			'parents' => $roleParents,
+			'children' => [],
+		];
 
 		return $this;
 	}
@@ -222,7 +218,7 @@ class Permission extends Nette\Object implements IAuthorizator
 	 */
 	public function removeAllRoles()
 	{
-		$this->roles = array();
+		$this->roles = [];
 
 		foreach ($this->rules['allResources']['byRole'] as $roleCurrent => $rules) {
 			unset($this->rules['allResources']['byRole'][$roleCurrent]);
@@ -263,10 +259,10 @@ class Permission extends Nette\Object implements IAuthorizator
 			$this->resources[$parent]['children'][$resource] = TRUE;
 		}
 
-		$this->resources[$resource] = array(
-			'parent'   => $parent,
-			'children' => array()
-		);
+		$this->resources[$resource] = [
+			'parent' => $parent,
+			'children' => [],
+		];
 
 		return $this;
 	}
@@ -366,7 +362,7 @@ class Permission extends Nette\Object implements IAuthorizator
 			unset($this->resources[$parent]['children'][$resource]);
 		}
 
-		$removed = array($resource);
+		$removed = [$resource];
 		foreach ($this->resources[$resource]['children'] as $child => $foo) {
 			$this->removeResource($child);
 			$removed[] = $child;
@@ -399,7 +395,7 @@ class Permission extends Nette\Object implements IAuthorizator
 			}
 		}
 
-		$this->resources = array();
+		$this->resources = [];
 		return $this;
 	}
 
@@ -486,11 +482,11 @@ class Permission extends Nette\Object implements IAuthorizator
 	{
 		// ensure that all specified Roles exist; normalize input to array of Roles or NULL
 		if ($roles === self::ALL) {
-			$roles = array(self::ALL);
+			$roles = [self::ALL];
 
 		} else {
 			if (!is_array($roles)) {
-				$roles = array($roles);
+				$roles = [$roles];
 			}
 
 			foreach ($roles as $role) {
@@ -500,11 +496,11 @@ class Permission extends Nette\Object implements IAuthorizator
 
 		// ensure that all specified Resources exist; normalize input to array of Resources or NULL
 		if ($resources === self::ALL) {
-			$resources = array(self::ALL);
+			$resources = [self::ALL];
 
 		} else {
 			if (!is_array($resources)) {
-				$resources = array($resources);
+				$resources = [$resources];
 			}
 
 			foreach ($resources as $resource) {
@@ -514,10 +510,10 @@ class Permission extends Nette\Object implements IAuthorizator
 
 		// normalize privileges to array
 		if ($privileges === self::ALL) {
-			$privileges = array();
+			$privileges = [];
 
 		} elseif (!is_array($privileges)) {
-			$privileges = array($privileges);
+			$privileges = [$privileges];
 		}
 
 		if ($toAdd) { // add to the rules
@@ -528,7 +524,7 @@ class Permission extends Nette\Object implements IAuthorizator
 						$rules['allPrivileges']['type'] = $type;
 						$rules['allPrivileges']['assert'] = $assertion;
 						if (!isset($rules['byPrivilege'])) {
-							$rules['byPrivilege'] = array();
+							$rules['byPrivilege'] = [];
 						}
 					} else {
 						foreach ($privileges as $privilege) {
@@ -549,13 +545,13 @@ class Permission extends Nette\Object implements IAuthorizator
 					if (count($privileges) === 0) {
 						if ($resource === self::ALL && $role === self::ALL) {
 							if ($type === $rules['allPrivileges']['type']) {
-								$rules = array(
-									'allPrivileges' => array(
+								$rules = [
+									'allPrivileges' => [
 										'type' => self::DENY,
-										'assert' => NULL
-										),
-									'byPrivilege' => array()
-									);
+										'assert' => NULL,
+										],
+									'byPrivilege' => [],
+								];
 							}
 							continue;
 						}
@@ -681,10 +677,10 @@ class Permission extends Nette\Object implements IAuthorizator
 	 */
 	private function searchRolePrivileges($all, $role, $resource, $privilege)
 	{
-		$dfs = array(
-			'visited' => array(),
-			'stack' => array($role),
-		);
+		$dfs = [
+			'visited' => [],
+			'stack' => [$role],
+		];
 
 		while (NULL !== ($role = array_pop($dfs['stack']))) {
 			if (isset($dfs['visited'][$role])) {
@@ -778,7 +774,7 @@ class Permission extends Nette\Object implements IAuthorizator
 				if (!$create) {
 					return $null;
 				}
-				$this->rules['byResource'][$resource] = array();
+				$this->rules['byResource'][$resource] = [];
 			}
 			$visitor = & $this->rules['byResource'][$resource];
 		}
@@ -788,7 +784,7 @@ class Permission extends Nette\Object implements IAuthorizator
 				if (!$create) {
 					return $null;
 				}
-				$visitor['allRoles']['byPrivilege'] = array();
+				$visitor['allRoles']['byPrivilege'] = [];
 			}
 			return $visitor['allRoles'];
 		}
@@ -797,7 +793,7 @@ class Permission extends Nette\Object implements IAuthorizator
 			if (!$create) {
 				return $null;
 			}
-			$visitor['byRole'][$role]['byPrivilege'] = array();
+			$visitor['byRole'][$role]['byPrivilege'] = [];
 		}
 
 		return $visitor['byRole'][$role];
