@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Security;
@@ -12,21 +12,26 @@ use Nette;
 
 /**
  * Trivial implementation of IAuthenticator.
- *
- * @author     David Grudl
  */
-class SimpleAuthenticator extends Nette\Object implements IAuthenticator
+class SimpleAuthenticator implements IAuthenticator
 {
+	use Nette\SmartObject;
+
 	/** @var array */
 	private $userlist;
 
+	/** @var array */
+	private $usersRoles;
+
 
 	/**
 	 * @param  array  list of pairs username => password
+	 * @param  array  list of pairs username => role[]
 	 */
-	public function __construct(array $userlist)
+	public function __construct(array $userlist, array $usersRoles = [])
 	{
 		$this->userlist = $userlist;
+		$this->usersRoles = $usersRoles;
 	}
 
 
@@ -42,7 +47,7 @@ class SimpleAuthenticator extends Nette\Object implements IAuthenticator
 		foreach ($this->userlist as $name => $pass) {
 			if (strcasecmp($name, $username) === 0) {
 				if ((string) $pass === (string) $password) {
-					return new Identity($name);
+					return new Identity($name, isset($this->usersRoles[$name]) ? $this->usersRoles[$name] : NULL);
 				} else {
 					throw new AuthenticationException('Invalid password.', self::INVALID_CREDENTIAL);
 				}
