@@ -47,6 +47,7 @@ use Nette;
  * @method void copyMergeGray(Image $src, $dstX, $dstY, $srcX, $srcY, $srcW, $srcH, $opacity)
  * @method void copyResampled(Image $src, $dstX, $dstY, $srcX, $srcY, $dstW, $dstH, $srcW, $srcH)
  * @method void copyResized(Image $src, $dstX, $dstY, $srcX, $srcY, $dstW, $dstH, $srcW, $srcH)
+ * @method Image cropAuto(int $mode = -1, float $threshold = .5, int $color = -1)
  * @method void dashedLine($x1, $y1, $x2, $y2, $color)
  * @method void ellipse($cx, $cy, $w, $h, $color)
  * @method void fill($x, $y, $color)
@@ -56,6 +57,7 @@ use Nette;
  * @method void filledRectangle($x1, $y1, $x2, $y2, $color)
  * @method void fillToBorder($x, $y, $border, $color)
  * @method void filter($filtertype)
+ * @method void flip(int $mode)
  * @method array ftText($size, $angle, $x, $y, $col, string $fontFile, string $text, array $extrainfo = NULL)
  * @method void gammaCorrect(float $inputgamma, float $outputgamma)
  * @method int interlace($interlace = NULL)
@@ -63,11 +65,13 @@ use Nette;
  * @method void layerEffect($effect)
  * @method void line($x1, $y1, $x2, $y2, $color)
  * @method void paletteCopy(Image $source)
+ * @method void paletteToTrueColor()
  * @method void polygon(array $points, $numPoints, $color)
  * @method array psText(string $text, $font, $size, $color, $backgroundColor, $x, $y, $space = NULL, $tightness = NULL, float $angle = NULL, $antialiasSteps = NULL)
  * @method void rectangle($x1, $y1, $x2, $y2, $col)
  * @method Image rotate(float $angle, $backgroundColor)
  * @method void saveAlpha(bool $saveflag)
+ * @method Image scale(int $newWidth, int $newHeight = -1, int $mode = IMG_BILINEAR_FIXED)
  * @method void setBrush(Image $brush)
  * @method void setPixel($x, $y, $color)
  * @method void setStyle(array $style)
@@ -81,22 +85,24 @@ use Nette;
  * @property-read int $height
  * @property-read resource $imageResource
  */
-class Image extends Nette\Object
+class Image
 {
+	use Nette\SmartObject;
+
 	/** {@link resize()} only shrinks images */
-	const SHRINK_ONLY = 1;
+	const SHRINK_ONLY = 0b0001;
 
 	/** {@link resize()} will ignore aspect ratio */
-	const STRETCH = 2;
+	const STRETCH = 0b0010;
 
 	/** {@link resize()} fits in given area so its dimensions are less than or equal to the required dimensions */
-	const FIT = 0;
+	const FIT = 0b0000;
 
 	/** {@link resize()} fills given area so its dimensions are greater than or equal to the required dimensions */
-	const FILL = 4;
+	const FILL = 0b0100;
 
 	/** {@link resize()} fills given area exactly */
-	const EXACT = 8;
+	const EXACT = 0b1000;
 
 	/** image types */
 	const JPEG = IMAGETYPE_JPEG,
@@ -122,12 +128,12 @@ class Image extends Nette\Object
 	 */
 	public static function rgb($red, $green, $blue, $transparency = 0)
 	{
-		return array(
+		return [
 			'red' => max(0, min(255, (int) $red)),
 			'green' => max(0, min(255, (int) $green)),
 			'blue' => max(0, min(255, (int) $blue)),
 			'alpha' => max(0, min(127, (int) $transparency)),
-		);
+		];
 	}
 
 
@@ -145,35 +151,22 @@ class Image extends Nette\Object
 			throw new Nette\NotSupportedException('PHP extension GD is not loaded.');
 		}
 
-		static $funcs = array(
+		static $funcs = [
 			self::JPEG => 'imagecreatefromjpeg',
 			self::PNG => 'imagecreatefrompng',
 			self::GIF => 'imagecreatefromgif',
-		);
-		$info = @getimagesize($file); // @ - files smaller than 12 bytes causes read error
-		$format = $info[2];
+		];
+		$format = @getimagesize($file)[2]; // @ - files smaller than 12 bytes causes read error
 
 		if (!isset($funcs[$format])) {
 			throw new UnknownImageFileException(is_file($file) ? "Unknown type of file '$file'." : "File '$file' not found.");
 		}
-		return new static(Callback::invokeSafe($funcs[$format], array($file), function ($message) {
+		return new static(Callback::invokeSafe($funcs[$format], [$file], function ($message) {
 			throw new ImageException($message);
 		}));
 	}
 
 
-	/**
-	 * @deprecated
-	 */
-	public static function getFormatFromString($s)
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use finfo_buffer() instead.', E_USER_DEPRECATED);
-		$types = array('image/jpeg' => self::JPEG, 'image/gif' => self::GIF, 'image/png' => self::PNG);
-		$type = finfo_buffer(finfo_open(FILEINFO_MIME_TYPE), $s);
-		return isset($types[$type]) ? $types[$type] : NULL;
-	}
-
-
 	/**
 	 * Create a new image from the image stream in the string.
 	 * @param  string
@@ -188,10 +181,11 @@ class Image extends Nette\Object
 		}
 
 		if (func_num_args() > 1) {
-			$format = @static::getFormatFromString($s); // @ suppress trigger_error
+			$tmp = @getimagesizefromstring($s)[2]; // @ - strings smaller than 12 bytes causes read error
+			$format = in_array($tmp, [self::JPEG, self::PNG, self::GIF], TRUE) ? $tmp : NULL;
 		}
 
-		return new static(Callback::invokeSafe('imagecreatefromstring', array($s), function ($message) {
+		return new static(Callback::invokeSafe('imagecreatefromstring', [$s], function ($message) {
 			throw new ImageException($message);
 		}));
 	}
@@ -218,7 +212,7 @@ class Image extends Nette\Object
 
 		$image = imagecreatetruecolor($width, $height);
 		if (is_array($color)) {
-			$color += array('alpha' => 0);
+			$color += ['alpha' => 0];
 			$color = imagecolorresolvealpha($image, $color['red'], $color['green'], $color['blue'], $color['alpha']);
 			imagealphablending($image, FALSE);
 			imagefilledrectangle($image, 0, 0, $width - 1, $height - 1, $color);
@@ -309,14 +303,8 @@ class Image extends Nette\Object
 			$this->image = $newImage;
 		}
 
-		if ($width < 0 || $height < 0) { // flip is processed in two steps for better quality
-			$newImage = static::fromBlank($newWidth, $newHeight, self::RGB(0, 0, 0, 127))->getImageResource();
-			imagecopyresampled(
-				$newImage, $this->image,
-				0, 0, $width < 0 ? $newWidth - 1 : 0, $height < 0 ? $newHeight - 1 : 0,
-				$newWidth, $newHeight, $width < 0 ? -$newWidth : $newWidth, $height < 0 ? -$newHeight : $newHeight
-			);
-			$this->image = $newImage;
+		if ($width < 0 || $height < 0) {
+			imageflip($this->image, $width < 0 ? ($height < 0 ? IMG_FLIP_BOTH : IMG_FLIP_HORIZONTAL) : IMG_FLIP_VERTICAL);
 		}
 		return $this;
 	}
@@ -333,15 +321,15 @@ class Image extends Nette\Object
 	 */
 	public static function calculateSize($srcWidth, $srcHeight, $newWidth, $newHeight, $flags = self::FIT)
 	{
-		if (substr($newWidth, -1) === '%') {
-			$newWidth = round($srcWidth / 100 * abs($newWidth));
+		if (is_string($newWidth) && substr($newWidth, -1) === '%') {
+			$newWidth = (int) round($srcWidth / 100 * abs($newWidth));
 			$percents = TRUE;
 		} else {
 			$newWidth = (int) abs($newWidth);
 		}
 
-		if (substr($newHeight, -1) === '%') {
-			$newHeight = round($srcHeight / 100 * abs($newHeight));
+		if (is_string($newHeight) && substr($newHeight, -1) === '%') {
+			$newHeight = (int) round($srcHeight / 100 * abs($newHeight));
 			$flags |= empty($percents) ? 0 : self::STRETCH;
 		} else {
 			$newHeight = (int) abs($newHeight);
@@ -353,8 +341,8 @@ class Image extends Nette\Object
 			}
 
 			if ($flags & self::SHRINK_ONLY) {
-				$newWidth = round($srcWidth * min(1, $newWidth / $srcWidth));
-				$newHeight = round($srcHeight * min(1, $newHeight / $srcHeight));
+				$newWidth = (int) round($srcWidth * min(1, $newWidth / $srcWidth));
+				$newHeight = (int) round($srcHeight * min(1, $newHeight / $srcHeight));
 			}
 
 		} else {  // proportional
@@ -362,7 +350,7 @@ class Image extends Nette\Object
 				throw new Nette\InvalidArgumentException('At least width or height must be specified.');
 			}
 
-			$scale = array();
+			$scale = [];
 			if ($newWidth > 0) { // fit width
 				$scale[] = $newWidth / $srcWidth;
 			}
@@ -372,7 +360,7 @@ class Image extends Nette\Object
 			}
 
 			if ($flags & self::FILL) {
-				$scale = array(max($scale));
+				$scale = [max($scale)];
 			}
 
 			if ($flags & self::SHRINK_ONLY) {
@@ -380,11 +368,11 @@ class Image extends Nette\Object
 			}
 
 			$scale = min($scale);
-			$newWidth = round($srcWidth * $scale);
-			$newHeight = round($srcHeight * $scale);
+			$newWidth = (int) round($srcWidth * $scale);
+			$newHeight = (int) round($srcHeight * $scale);
 		}
 
-		return array(max((int) $newWidth, 1), max((int) $newHeight, 1));
+		return [max($newWidth, 1), max($newHeight, 1)];
 	}
 
 
@@ -398,10 +386,15 @@ class Image extends Nette\Object
 	 */
 	public function crop($left, $top, $width, $height)
 	{
-		list($left, $top, $width, $height) = static::calculateCutout($this->getWidth(), $this->getHeight(), $left, $top, $width, $height);
-		$newImage = static::fromBlank($width, $height, self::RGB(0, 0, 0, 127))->getImageResource();
-		imagecopy($newImage, $this->image, 0, 0, $left, $top, $width, $height);
-		$this->image = $newImage;
+		list($r['x'], $r['y'], $r['width'], $r['height'])
+			= static::calculateCutout($this->getWidth(), $this->getHeight(), $left, $top, $width, $height);
+		if (PHP_VERSION_ID > 50611) { // PHP bug #67447
+			$this->image = imagecrop($this->image, $r);
+		} else {
+			$newImage = static::fromBlank($r['width'], $r['height'], self::RGB(0, 0, 0, 127))->getImageResource();
+			imagecopy($newImage, $this->image, 0, 0, $r['x'], $r['y'], $r['width'], $r['height']);
+			$this->image = $newImage;
+		}
 		return $this;
 	}
 
@@ -418,17 +411,17 @@ class Image extends Nette\Object
 	 */
 	public static function calculateCutout($srcWidth, $srcHeight, $left, $top, $newWidth, $newHeight)
 	{
-		if (substr($newWidth, -1) === '%') {
-			$newWidth = round($srcWidth / 100 * $newWidth);
+		if (is_string($newWidth) && substr($newWidth, -1) === '%') {
+			$newWidth = (int) round($srcWidth / 100 * $newWidth);
 		}
-		if (substr($newHeight, -1) === '%') {
-			$newHeight = round($srcHeight / 100 * $newHeight);
+		if (is_string($newHeight) && substr($newHeight, -1) === '%') {
+			$newHeight = (int) round($srcHeight / 100 * $newHeight);
 		}
-		if (substr($left, -1) === '%') {
-			$left = round(($srcWidth - $newWidth) / 100 * $left);
+		if (is_string($left) && substr($left, -1) === '%') {
+			$left = (int) round(($srcWidth - $newWidth) / 100 * $left);
 		}
-		if (substr($top, -1) === '%') {
-			$top = round(($srcHeight - $newHeight) / 100 * $top);
+		if (is_string($top) && substr($top, -1) === '%') {
+			$top = (int) round(($srcHeight - $newHeight) / 100 * $top);
 		}
 		if ($left < 0) {
 			$newWidth += $left;
@@ -438,9 +431,9 @@ class Image extends Nette\Object
 			$newHeight += $top;
 			$top = 0;
 		}
-		$newWidth = min((int) $newWidth, $srcWidth - $left);
-		$newHeight = min((int) $newHeight, $srcHeight - $top);
-		return array($left, $top, $newWidth, $newHeight);
+		$newWidth = min($newWidth, $srcWidth - $left);
+		$newHeight = min($newHeight, $srcHeight - $top);
+		return [$left, $top, $newWidth, $newHeight];
 	}
 
 
@@ -450,11 +443,11 @@ class Image extends Nette\Object
 	 */
 	public function sharpen()
 	{
-		imageconvolution($this->image, array( // my magic numbers ;)
-			array(-1, -1, -1),
-			array(-1, 24, -1),
-			array(-1, -1, -1),
-		), 16, 0);
+		imageconvolution($this->image, [ // my magic numbers ;)
+			[-1, -1, -1],
+			[-1, 24, -1],
+			[-1, -1, -1],
+		], 16, 0);
 		return $this;
 	}
 
@@ -470,38 +463,48 @@ class Image extends Nette\Object
 	public function place(Image $image, $left = 0, $top = 0, $opacity = 100)
 	{
 		$opacity = max(0, min(100, (int) $opacity));
-
-		if (substr($left, -1) === '%') {
-			$left = round(($this->getWidth() - $image->getWidth()) / 100 * $left);
+		if ($opacity === 0) {
+			return $this;
 		}
 
-		if (substr($top, -1) === '%') {
-			$top = round(($this->getHeight() - $image->getHeight()) / 100 * $top);
+		$width = $image->getWidth();
+		$height = $image->getHeight();
+
+		if (is_string($left) && substr($left, -1) === '%') {
+			$left = (int) round(($this->getWidth() - $width) / 100 * $left);
 		}
 
-		if ($opacity === 100) {
-			imagecopy(
-				$this->image, $image->getImageResource(),
-				$left, $top, 0, 0, $image->getWidth(), $image->getHeight()
-			);
-
-		} elseif ($opacity != 0) {
-			$cutting = imagecreatetruecolor($image->getWidth(), $image->getHeight());
-			imagecopy(
-				$cutting, $this->image,
-				0, 0, $left, $top, $image->getWidth(), $image->getHeight()
-			);
-			imagecopy(
-				$cutting, $image->getImageResource(),
-				0, 0, 0, 0, $image->getWidth(), $image->getHeight()
-			);
-
-			imagecopymerge(
-				$this->image, $cutting,
-				$left, $top, 0, 0, $image->getWidth(), $image->getHeight(),
-				$opacity
-			);
+		if (is_string($top) && substr($top, -1) === '%') {
+			$top = (int) round(($this->getHeight() - $height) / 100 * $top);
 		}
+
+		$output = $input = $image->image;
+		if ($opacity < 100) {
+			for ($i = 0; $i < 128; $i++) {
+				$tbl[$i] = round(127 - (127 - $i) * $opacity / 100);
+			}
+
+			$output = imagecreatetruecolor($width, $height);
+			imagealphablending($output, FALSE);
+			if (!$image->isTrueColor()) {
+				$input = $output;
+				imagefilledrectangle($output, 0, 0, $width, $height, imagecolorallocatealpha($output, 0, 0, 0, 127));
+				imagecopy($output, $image->image, 0, 0, 0, 0, $width, $height);
+			}
+			for ($x = 0; $x < $width; $x++) {
+				for ($y = 0; $y < $height; $y++) {
+					$c = \imagecolorat($input, $x, $y);
+					$c = ($c & 0xFFFFFF) + ($tbl[$c >> 24] << 24);
+					\imagesetpixel($output, $x, $y, $c);
+				}
+			}
+			imagealphablending($output, TRUE);
+		}
+
+		imagecopy(
+			$this->image, $output,
+			$left, $top, 0, 0, $width, $height
+		);
 		return $this;
 	}
 
@@ -592,7 +595,7 @@ class Image extends Nette\Object
 	 */
 	public function send($type = self::JPEG, $quality = NULL)
 	{
-		if (!in_array($type, array(self::JPEG, self::PNG, self::GIF), TRUE)) {
+		if (!in_array($type, [self::JPEG, self::PNG, self::GIF], TRUE)) {
 			throw new Nette\InvalidArgumentException("Unsupported image type '$type'.");
 		}
 		header('Content-Type: ' . image_type_to_mime_type($type));
@@ -612,7 +615,7 @@ class Image extends Nette\Object
 	{
 		$function = 'image' . $name;
 		if (!function_exists($function)) {
-			return parent::__call($name, $args);
+			ObjectMixin::strictCall(get_class($this), $name);
 		}
 
 		foreach ($args as $key => $value) {
@@ -629,9 +632,7 @@ class Image extends Nette\Object
 				);
 			}
 		}
-		array_unshift($args, $this->image);
-
-		$res = call_user_func_array($function, $args);
+		$res = $function($this->image, ...$args);
 		return is_resource($res) && get_resource_type($res) === 'gd' ? $this->setImageResource($res) : $res;
 	}
 
