@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Utils;
@@ -13,8 +13,6 @@ use Nette;
 /**
  * Paginating math.
  *
- * @author     David Grudl
- *
  * @property   int $page
  * @property-read int $firstPage
  * @property-read int|NULL $lastPage
@@ -28,8 +26,10 @@ use Nette;
  * @property-read int|NULL $countdownOffset
  * @property-read int|NULL $length
  */
-class Paginator extends Nette\Object
+class Paginator
 {
+	use Nette\SmartObject;
+
 	/** @var int */
 	private $base = 1;
 
