@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Utils;
@@ -12,59 +12,49 @@ use Nette;
 
 /**
  * String tools library.
- *
- * @author     David Grudl
  */
 class Strings
 {
+	use Nette\StaticClass;
 
-	/**
-	 * Static class - cannot be instantiated.
-	 */
-	final public function __construct()
-	{
-		throw new Nette\StaticClassException;
-	}
+	const TRIM_CHARACTERS = " \t\n\r\0\x0B\xC2\xA0";
 
 
 	/**
-	 * Checks if the string is valid for the specified encoding.
+	 * Checks if the string is valid for UTF-8 encoding.
 	 * @param  string  byte stream to check
-	 * @param  string  expected encoding
 	 * @return bool
 	 */
-	public static function checkEncoding($s, $encoding = 'UTF-8')
+	public static function checkEncoding($s)
 	{
-		return $s === self::fixEncoding($s, $encoding);
+		return $s === self::fixEncoding($s);
 	}
 
 
 	/**
-	 * Returns correctly encoded string.
+	 * Removes invalid code unit sequences from UTF-8 string.
 	 * @param  string  byte stream to fix
-	 * @param  string  encoding
 	 * @return string
 	 */
-	public static function fixEncoding($s, $encoding = 'UTF-8')
+	public static function fixEncoding($s)
 	{
 		// removes xD800-xDFFF, x110000 and higher
-		if (PHP_VERSION_ID < 50400 || strcasecmp($encoding, 'UTF-8')) {
-			return @iconv('UTF-16', $encoding . '//IGNORE', iconv($encoding, 'UTF-16//IGNORE', $s)); // intentionally @
-		} else {
-			return htmlspecialchars_decode(htmlspecialchars($s, ENT_NOQUOTES | ENT_IGNORE, 'UTF-8'), ENT_NOQUOTES);
-		}
+		return htmlspecialchars_decode(htmlspecialchars($s, ENT_NOQUOTES | ENT_IGNORE, 'UTF-8'), ENT_NOQUOTES);
 	}
 
 
 	/**
-	 * Returns a specific character.
-	 * @param  int     codepoint
-	 * @param  string  encoding
+	 * Returns a specific character in UTF-8.
+	 * @param  int     code point (0x0 to 0xD7FF or 0xE000 to 0x10FFFF)
 	 * @return string
+	 * @throws Nette\InvalidArgumentException if code point is not in valid range
 	 */
-	public static function chr($code, $encoding = 'UTF-8')
+	public static function chr($code)
 	{
-		return iconv('UTF-32BE', $encoding . '//IGNORE', pack('N', $code));
+		if ($code < 0 || ($code >= 0xD800 && $code <= 0xDFFF) || $code > 0x10FFFF) {
+			throw new Nette\InvalidArgumentException('Code point must be in range 0x0 to 0xD7FF or 0xE000 to 0x10FFFF.');
+		}
+		return iconv('UTF-32BE', 'UTF-8//IGNORE', pack('N', $code));
 	}
 
 
@@ -107,17 +97,18 @@ class Strings
 	/**
 	 * Returns a part of UTF-8 string.
 	 * @param  string
-	 * @param  int
-	 * @param  int
+	 * @param  int in characters (code points)
+	 * @param  int in characters (code points)
 	 * @return string
 	 */
 	public static function substring($s, $start, $length = NULL)
 	{
-		if ($length === NULL) {
-			$length = self::length($s);
-		}
 		if (function_exists('mb_substr')) {
 			return mb_substr($s, $start, $length, 'UTF-8'); // MB is much faster
+		} elseif ($length === NULL) {
+			$length = self::length($s);
+		} elseif ($start < 0 && $length < 0) {
+			$start += self::length($s); // unifies iconv_substr behavior with mb_substr
 		}
 		return iconv_substr($s, $start, $length, 'UTF-8');
 	}
@@ -125,7 +116,7 @@ class Strings
 
 	/**
 	 * Removes special controls characters and normalizes line endings and spaces.
-	 * @param  string  UTF-8 encoding or 8-bit
+	 * @param  string  UTF-8 encoding
 	 * @return string
 	 */
 	public static function normalize($s)
@@ -133,7 +124,7 @@ class Strings
 		$s = self::normalizeNewLines($s);
 
 		// remove control characters; leave \t + \n
-		$s = preg_replace('#[\x00-\x08\x0B-\x1F\x7F]+#', '', $s);
+		$s = preg_replace('#[\x00-\x08\x0B-\x1F\x7F-\x9F]+#u', '', $s);
 
 		// right trim
 		$s = preg_replace('#[\t ]+$#m', '', $s);
@@ -152,7 +143,7 @@ class Strings
 	 */
 	public static function normalizeNewLines($s)
 	{
-		return str_replace(array("\r\n", "\r"), "\n", $s);
+		return str_replace(["\r\n", "\r"], "\n", $s);
 	}
 
 
@@ -163,23 +154,38 @@ class Strings
 	 */
 	public static function toAscii($s)
 	{
+		static $transliterator = NULL;
+		if ($transliterator === NULL && class_exists('Transliterator', FALSE)) {
+			$transliterator = \Transliterator::create('Any-Latin; Latin-ASCII');
+		}
+
 		$s = preg_replace('#[^\x09\x0A\x0D\x20-\x7E\xA0-\x{2FF}\x{370}-\x{10FFFF}]#u', '', $s);
-		$s = strtr($s, '`\'"^~', "\x01\x02\x03\x04\x05");
-		$s = str_replace(array("\xE2\x80\x9E", "\xE2\x80\x9C", "\xE2\x80\x9D", "\xE2\x80\x9A",
-			"\xE2\x80\x98", "\xE2\x80\x99", "\xC2\xBB", "\xC2\xAB"),
-			array("\x03", "\x03", "\x03", "\x02", "\x02", "\x02", ">>", "<<"), $s);
+		$s = strtr($s, '`\'"^~?', "\x01\x02\x03\x04\x05\x06");
+		$s = str_replace(
+			["\xE2\x80\x9E", "\xE2\x80\x9C", "\xE2\x80\x9D", "\xE2\x80\x9A", "\xE2\x80\x98", "\xE2\x80\x99", "\xC2\xB0"],
+			["\x03", "\x03", "\x03", "\x02", "\x02", "\x02", "\x04"], $s
+		);
+		if ($transliterator !== NULL) {
+			$s = $transliterator->transliterate($s);
+		}
 		if (ICONV_IMPL === 'glibc') {
-			$s = @iconv('UTF-8', 'WINDOWS-1250//TRANSLIT', $s); // intentionally @
+			$s = str_replace(
+				["\xC2\xBB", "\xC2\xAB", "\xE2\x80\xA6", "\xE2\x84\xA2", "\xC2\xA9", "\xC2\xAE"],
+				['>>', '<<', '...', 'TM', '(c)', '(R)'], $s
+			);
+			$s = iconv('UTF-8', 'WINDOWS-1250//TRANSLIT//IGNORE', $s);
 			$s = strtr($s, "\xa5\xa3\xbc\x8c\xa7\x8a\xaa\x8d\x8f\x8e\xaf\xb9\xb3\xbe\x9c\x9a\xba\x9d\x9f\x9e"
 				. "\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3"
 				. "\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8"
-				. "\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf8\xf9\xfa\xfb\xfc\xfd\xfe\x96",
-				"ALLSSSSTZZZallssstzzzRAAAALCCCEEEEIIDDNNOOOOxRUUUUYTsraaaalccceeeeiiddnnooooruuuuyt-");
+				. "\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf8\xf9\xfa\xfb\xfc\xfd\xfe"
+				. "\x96\xa0\x8b\x97\x9b\xa6\xad\xb7",
+				'ALLSSSSTZZZallssstzzzRAAAALCCCEEEEIIDDNNOOOOxRUUUUYTsraaaalccceeeeiiddnnooooruuuuyt- <->|-.');
+			$s = preg_replace('#[^\x00-\x7F]++#', '', $s);
 		} else {
-			$s = @iconv('UTF-8', 'ASCII//TRANSLIT', $s); // intentionally @
+			$s = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE', $s);
 		}
-		$s = str_replace(array('`', "'", '"', '^', '~'), '', $s);
-		return strtr($s, "\x01\x02\x03\x04\x05", '`\'"^~');
+		$s = str_replace(['`', "'", '"', '^', '~', '?'], '', $s);
+		return strtr($s, "\x01\x02\x03\x04\x05\x06", '`\'"^~?');
 	}
 
 
@@ -196,7 +202,7 @@ class Strings
 		if ($lower) {
 			$s = strtolower($s);
 		}
-		$s = preg_replace('#[^a-z0-9' . preg_quote($charlist, '#') . ']+#i', '-', $s);
+		$s = preg_replace('#[^a-z0-9' . ($charlist !== NULL ? preg_quote($charlist, '#') : '') . ']+#i', '-', $s);
 		$s = trim($s, '-');
 		return $s;
 	}
@@ -254,6 +260,17 @@ class Strings
 	}
 
 
+	/**
+	 * Convert first character to lower case.
+	 * @param  string  UTF-8 encoding
+	 * @return string
+	 */
+	public static function firstLower($s)
+	{
+		return self::lower(self::substring($s, 0, 1)) . self::substring($s, 1);
+	}
+
+
 	/**
 	 * Convert to upper case.
 	 * @param  string  UTF-8 encoding
@@ -312,16 +329,16 @@ class Strings
 	 * @param  string|array
 	 * @return string
 	 */
-	public static function findPrefix($strings)
+	public static function findPrefix(...$strings)
 	{
-		if (!is_array($strings)) {
-			$strings = func_get_args();
+		if (is_array($strings[0])) {
+			$strings = $strings[0];
 		}
 		$first = array_shift($strings);
 		for ($i = 0; $i < strlen($first); $i++) {
 			foreach ($strings as $s) {
 				if (!isset($s[$i]) || $first[$i] !== $s[$i]) {
-					while ($i && $first[$i-1] >= "\x80" && $first[$i] >= "\x80" && $first[$i] < "\xC0") {
+					while ($i && $first[$i - 1] >= "\x80" && $first[$i] >= "\x80" && $first[$i] < "\xC0") {
 						$i--;
 					}
 					return substr($first, 0, $i);
@@ -333,13 +350,14 @@ class Strings
 
 
 	/**
-	 * Returns UTF-8 string length.
+	 * Returns number of characters (not bytes) in UTF-8 string.
+	 * That is the number of Unicode code points which may differ from the number of graphemes.
 	 * @param  string
 	 * @return int
 	 */
 	public static function length($s)
 	{
-		return strlen(utf8_decode($s)); // fastest way
+		return function_exists('mb_strlen') ? mb_strlen($s, 'UTF-8') : strlen(utf8_decode($s));
 	}
 
 
@@ -349,7 +367,7 @@ class Strings
 	 * @param  string
 	 * @return string
 	 */
-	public static function trim($s, $charlist = " \t\n\r\0\x0B\xC2\xA0")
+	public static function trim($s, $charlist = self::TRIM_CHARACTERS)
 	{
 		$charlist = preg_quote($charlist, '#');
 		return self::replace($s, '#^['.$charlist.']+|['.$charlist.']+\z#u', '');
@@ -367,7 +385,7 @@ class Strings
 	{
 		$length = max(0, $length - self::length($s));
 		$padLen = self::length($pad);
-		return str_repeat($pad, $length / $padLen) . self::substring($pad, 0, $length % $padLen) . $s;
+		return str_repeat($pad, (int) ($length / $padLen)) . self::substring($pad, 0, $length % $padLen) . $s;
 	}
 
 
@@ -382,7 +400,7 @@ class Strings
 	{
 		$length = max(0, $length - self::length($s));
 		$padLen = self::length($pad);
-		return $s . str_repeat($pad, $length / $padLen) . self::substring($pad, 0, $length % $padLen);
+		return $s . str_repeat($pad, (int) ($length / $padLen)) . self::substring($pad, 0, $length % $padLen);
 	}
 
 
@@ -393,50 +411,96 @@ class Strings
 	 */
 	public static function reverse($s)
 	{
-		return @iconv('UTF-32LE', 'UTF-8', strrev(@iconv('UTF-8', 'UTF-32BE', $s)));
+		return iconv('UTF-32LE', 'UTF-8', strrev(iconv('UTF-8', 'UTF-32BE', $s)));
 	}
 
 
 	/**
-	 * Generate random string.
-	 * @param  int
-	 * @param  string
-	 * @return string
+	 * Use Nette\Utils\Random::generate
+	 * @deprecated
 	 */
 	public static function random($length = 10, $charlist = '0-9a-z')
 	{
-		$charlist = str_shuffle(preg_replace_callback('#.-.#', function($m) {
-			return implode('', range($m[0][0], $m[0][2]));
-		}, $charlist));
-		$chLen = strlen($charlist);
+		trigger_error(__METHOD__ . '() is deprecated, use Nette\Utils\Random::generate()', E_USER_DEPRECATED);
+		return Random::generate($length, $charlist);
+	}
 
-		if (function_exists('openssl_random_pseudo_bytes')
-			&& (PHP_VERSION_ID >= 50400 || !defined('PHP_WINDOWS_VERSION_BUILD')) // slow in PHP 5.3 & Windows
-		) {
-			$rand3 = openssl_random_pseudo_bytes($length);
-		}
-		if (empty($rand3) && function_exists('mcrypt_create_iv') && (PHP_VERSION_ID >= 50307 || !defined('PHP_WINDOWS_VERSION_BUILD'))) { // PHP bug #52523
-			$rand3 = mcrypt_create_iv($length, MCRYPT_DEV_URANDOM);
-		}
-		if (empty($rand3) && @is_readable('/dev/urandom')) {
-			$rand3 = file_get_contents('/dev/urandom', FALSE, NULL, -1, $length);
-		}
-		if (empty($rand3)) {
-			static $cache;
-			$rand3 = $cache ?: $cache = md5(serialize($_SERVER), TRUE);
-		}
 
-		$s = '';
-		for ($i = 0; $i < $length; $i++) {
-			if ($i % 5 === 0) {
-				list($rand, $rand2) = explode(' ', microtime());
-				$rand += lcg_value();
+	/**
+	 * Returns part of $haystack before $nth occurence of $needle.
+	 * @param  string
+	 * @param  string
+	 * @param  int  negative value means searching from the end
+	 * @return string|FALSE  returns FALSE if the needle was not found
+	 */
+	public static function before($haystack, $needle, $nth = 1)
+	{
+		$pos = self::pos($haystack, $needle, $nth);
+		return $pos === FALSE
+			? FALSE
+			: substr($haystack, 0, $pos);
+	}
+
+
+	/**
+	 * Returns part of $haystack after $nth occurence of $needle.
+	 * @param  string
+	 * @param  string
+	 * @param  int  negative value means searching from the end
+	 * @return string|FALSE  returns FALSE if the needle was not found
+	 */
+	public static function after($haystack, $needle, $nth = 1)
+	{
+		$pos = self::pos($haystack, $needle, $nth);
+		return $pos === FALSE
+			? FALSE
+			: (string) substr($haystack, $pos + strlen($needle));
+	}
+
+
+	/**
+	 * Returns position of $nth occurence of $needle in $haystack.
+	 * @param  string
+	 * @param  string
+	 * @param  int  negative value means searching from the end
+	 * @return int|FALSE  offset in characters or FALSE if the needle was not found
+	 */
+	public static function indexOf($haystack, $needle, $nth = 1)
+	{
+		$pos = self::pos($haystack, $needle, $nth);
+		return $pos === FALSE
+			? FALSE
+			: self::length(substr($haystack, 0, $pos));
+	}
+
+
+	/**
+	 * Returns position of $nth occurence of $needle in $haystack.
+	 * @return int|FALSE  offset in bytes or FALSE if the needle was not found
+	 */
+	private static function pos($haystack, $needle, $nth = 1)
+	{
+		if (!$nth) {
+			return FALSE;
+		} elseif ($nth > 0) {
+			if (strlen($needle) === 0) {
+				return 0;
+			}
+			$pos = 0;
+			while (FALSE !== ($pos = strpos($haystack, $needle, $pos)) && --$nth) {
+				$pos++;
+			}
+		} else {
+			$len = strlen($haystack);
+			if (strlen($needle) === 0) {
+				return $len;
+			}
+			$pos = $len - 1;
+			while (FALSE !== ($pos = strrpos($haystack, $needle, $pos - $len)) && ++$nth) {
+				$pos--;
 			}
-			$rand *= $chLen;
-			$s .= $charlist[($rand + $rand2 + ord($rand3[$i % strlen($rand3)])) % $chLen];
-			$rand -= (int) $rand;
 		}
-		return $s;
+		return $pos;
 	}
 
 
@@ -449,7 +513,7 @@ class Strings
 	 */
 	public static function split($subject, $pattern, $flags = 0)
 	{
-		return self::pcre('preg_split', array($pattern, $subject, -1, $flags | PREG_SPLIT_DELIM_CAPTURE));
+		return self::pcre('preg_split', [$pattern, $subject, -1, $flags | PREG_SPLIT_DELIM_CAPTURE]);
 	}
 
 
@@ -466,7 +530,7 @@ class Strings
 		if ($offset > strlen($subject)) {
 			return NULL;
 		}
-		return self::pcre('preg_match', array($pattern, $subject, & $m, $flags, $offset))
+		return self::pcre('preg_match', [$pattern, $subject, & $m, $flags, $offset])
 			? $m
 			: NULL;
 	}
@@ -483,13 +547,13 @@ class Strings
 	public static function matchAll($subject, $pattern, $flags = 0, $offset = 0)
 	{
 		if ($offset > strlen($subject)) {
-			return array();
+			return [];
 		}
-		self::pcre('preg_match_all', array(
+		self::pcre('preg_match_all', [
 			$pattern, $subject, & $m,
 			($flags & PREG_PATTERN_ORDER) ? $flags : ($flags | PREG_SET_ORDER),
-			$offset
-		));
+			$offset,
+		]);
 		return $m;
 	}
 
@@ -506,61 +570,47 @@ class Strings
 	{
 		if (is_object($replacement) || is_array($replacement)) {
 			if ($replacement instanceof Nette\Callback) {
+				trigger_error('Nette\Callback is deprecated, use PHP callback.', E_USER_DEPRECATED);
 				$replacement = $replacement->getNative();
 			}
 			if (!is_callable($replacement, FALSE, $textual)) {
 				throw new Nette\InvalidStateException("Callback '$textual' is not callable.");
 			}
 
-			return self::pcre('preg_replace_callback', array($pattern, $replacement, $subject, $limit));
+			return self::pcre('preg_replace_callback', [$pattern, $replacement, $subject, $limit]);
 
 		} elseif ($replacement === NULL && is_array($pattern)) {
 			$replacement = array_values($pattern);
 			$pattern = array_keys($pattern);
 		}
 
-		return self::pcre('preg_replace', array($pattern, $replacement, $subject, $limit));
+		return self::pcre('preg_replace', [$pattern, $replacement, $subject, $limit]);
 	}
 
 
 	/** @internal */
 	public static function pcre($func, $args)
 	{
-		$res = Callback::invokeSafe($func, $args, function($message) use ($args) {
+		static $messages = [
+			PREG_INTERNAL_ERROR => 'Internal error',
+			PREG_BACKTRACK_LIMIT_ERROR => 'Backtrack limit was exhausted',
+			PREG_RECURSION_LIMIT_ERROR => 'Recursion limit was exhausted',
+			PREG_BAD_UTF8_ERROR => 'Malformed UTF-8 data',
+			PREG_BAD_UTF8_OFFSET_ERROR => 'Offset didn\'t correspond to the begin of a valid UTF-8 code point',
+			6 => 'Failed due to limited JIT stack space', // PREG_JIT_STACKLIMIT_ERROR
+		];
+		$res = Callback::invokeSafe($func, $args, function ($message) use ($args) {
 			// compile-time error, not detectable by preg_last_error
 			throw new RegexpException($message . ' in pattern: ' . implode(' or ', (array) $args[0]));
 		});
 
 		if (($code = preg_last_error()) // run-time error, but preg_last_error & return code are liars
-			&& ($res === NULL || !in_array($func, array('preg_filter', 'preg_replace_callback', 'preg_replace')))
+			&& ($res === NULL || !in_array($func, ['preg_filter', 'preg_replace_callback', 'preg_replace']))
 		) {
-			throw new RegexpException(NULL, $code, implode(' or ', (array) $args[0]));
+			throw new RegexpException((isset($messages[$code]) ? $messages[$code] : 'Unknown error')
+				. ' (pattern: ' . implode(' or ', (array) $args[0]) . ')', $code);
 		}
 		return $res;
 	}
 
 }
-
-
-/**
- * The exception that indicates error of the last Regexp execution.
- */
-class RegexpException extends \Exception
-{
-	static public $messages = array(
-		PREG_INTERNAL_ERROR => 'Internal error',
-		PREG_BACKTRACK_LIMIT_ERROR => 'Backtrack limit was exhausted',
-		PREG_RECURSION_LIMIT_ERROR => 'Recursion limit was exhausted',
-		PREG_BAD_UTF8_ERROR => 'Malformed UTF-8 data',
-		5 => 'Offset didn\'t correspond to the begin of a valid UTF-8 code point', // PREG_BAD_UTF8_OFFSET_ERROR
-	);
-
-	public function __construct($message, $code = NULL, $pattern = NULL)
-	{
-		if (!$message) {
-			$message = (isset(self::$messages[$code]) ? self::$messages[$code] : 'Unknown error') . ($pattern ? " (pattern: $pattern)" : '');
-		}
-		parent::__construct($message, $code);
-	}
-
-}
