@@ -1,7 +1,3 @@
-<!DOCTYPE html><link rel="stylesheet" href="assets/style.css">
-
-<h1>Tracy Exception demo</h1>
-
 <?php
 
 require __DIR__ . '/../../Nette/loader.php';
@@ -10,6 +6,12 @@ use Tracy\Debugger;
 
 Debugger::enable(Debugger::DETECT, __DIR__ . '/log');
 
+?>
+<!DOCTYPE html><link rel="stylesheet" href="assets/style.css">
+
+<h1>Tracy: exception demo</h1>
+
+<?php
 
 function first($arg1, $arg2)
 {
@@ -18,7 +20,7 @@ function first($arg1, $arg2)
 
 function second($arg1, $arg2)
 {
-	third(array(1, 2, 3));
+	third([1, 2, 3]);
 }
 
 function third($arg1)
