@@ -4,7 +4,7 @@ require __DIR__ . '/../../Nette/loader.php';
 
 use Tracy\Debugger;
 
-$arr = array(10, 20, array('key1' => 'val1', 'key2' => TRUE));
+$arr = [10, 20, ['key1' => 'val1', 'key2' => TRUE]];
 
 // will show in FireLogger
 Debugger::fireLog('Hello World');
@@ -18,7 +18,7 @@ function first($arg1, $arg2)
 
 function second($arg1, $arg2)
 {
-	third(array(1, 2, 3));
+	third([1, 2, 3]);
 }
 
 function third($arg1)
